// -*- C++ -*-

/***************************************************************************
                          configdialog.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// System
//

#include <iostream>

//
// Qt
//

#include <qlayout.h>
#include <qgroupbox.h>
#include <qcheckbox.h>
#include <qstringlist.h>
#include <qfile.h>
#include <qdict.h>

#include <kdialogbase.h>
#include <kstddirs.h>
#include <klocale.h>


//
// App
//

#include "kapitalist.h"
#include "global.h"

#include "configdialog.h"

//
// View
//

#include "kapview.h"
#include "themedata.h"

//  KDialogBase ( int dialogFace, const QString &caption, int
//  buttonMask, ButtonCode defaultButton, QWidget *parent=0, const
//  char *name=0, bool modal=true, bool separator=false, const QString
//  &user1=QString::null, const QString &user2=QString::null, const
//  QString &user3=QString::null)

/**
 *
 */
ConfigDialog::ConfigDialog(KapitalistApp *app, QWidget *parent)
  : KDialogBase( IconList, i18n("Configure Kapitalist"), Ok|Apply|Cancel, Ok, 
		 parent, "configdialog", false, true ),
    _app(app)
{
  
  _themeFrm = addPage( QString(i18n("Theme")),
		       QString(i18n("Choose your preferred combination")), 
//  		      locate("appdata", "pics/water.xpm"));
		       BarIcon("config_theme", KIcon::SizeMedium) );
  _configTheme = new ConfigTheme(_app, _themeFrm);
  _appearanceFrm = addPage( QString(i18n("Appearance")),
			    QString(i18n("Choose Appearance")), 
//  		      locate("appdata", "pics/water.xpm"));
		       BarIcon("config_theme", KIcon::SizeMedium) );
  _configAppearance = new ConfigAppearance(_app, _appearanceFrm);

  setMinimumSize(sizeHint());

}

ConfigDialog::~ConfigDialog()
{
}

void ConfigDialog::slotApply()
{
  // apply changes in theme config dialog
  _configTheme->apply();
  _configAppearance->apply();
}

void ConfigDialog::slotOk()
{
  _configTheme->apply();
  _configAppearance->apply();
  accept();
}


ConfigTheme::ConfigTheme(KapitalistApp *app, QWidget *parent)
  : _app(app)
{
  _top = new QVBoxLayout(parent, 
			 KDialog::marginHint(), KDialog::spacingHint());

  QGroupBox *boardThmGrp = new QGroupBox(1, Qt::Horizontal, 
					i18n("Board Pixmaps"), 
					parent);
  _top->addWidget(boardThmGrp);
  QGroupBox *namesThmGrp = new QGroupBox(1, Qt::Horizontal, i18n("Names"), 
					 parent);
  _top->addWidget(namesThmGrp);
  
  _boardThm = new QListBox(boardThmGrp);
  _namesThm = new QListBox(namesThmGrp);

  // lookup all board and names theme files
  QStringList boardFiles 
    = KGlobal::dirs()->findAllResources("appdata",
					"themes/board/board*.conf",
					true);
  QStringList namesFiles 
    = KGlobal::dirs()->findAllResources("appdata",
					"themes/names/names*.conf", 
					true);

  QStringList::Iterator it;
  for (it = boardFiles.begin(); it != boardFiles.end(); ++it) {
    QString *fname = new QString(*it);
//     printf( "%s \n", fname->latin1() );
    // parse each file and extract theme name + description
    ThemeGeneralEntry genEntry;
    if (!ThemeData::readThemeGeneralEntry(*fname, genEntry)) {
      fprintf(stderr, "theme file not valid: %s\n ... ignored\n", 
	      fname->latin1());
      continue;
    } else { // theme was read successfully
      // put one line into the names listbox
      _boardThm->insertItem(genEntry.name);
      // store filename in a dictionary with theme name as key
      _boardDict.insert(genEntry.name, fname);
    }
  }
  for (it = namesFiles.begin(); it != namesFiles.end(); ++it) {
    QString *fname = new QString(*it);
//     printf( "%s \n", fname->latin1() );
    // parse each file and extract theme name + description
    ThemeGeneralEntry genEntry;
    if (!ThemeData::readThemeGeneralEntry(*fname, genEntry)) {
      fprintf(stderr, "theme file not valid: %s\n ... ignored\n", 
	      fname->latin1());
      continue;
    } else { // theme was read successfully
      // put one line into the names listbox
      _namesThm->insertItem(genEntry.name);
      // store filename in a dictionary with theme name as key
      _namesDict.insert(genEntry.name, fname);
    }
  }

  _boardThm->setMinimumSize( _boardThm->sizeHint() );
  _namesThm->setMinimumSize( _namesThm->sizeHint() );

  _top->addStretch(1);

  reset();
}

ConfigTheme::~ConfigTheme()
{
}

void ConfigTheme::reset()
{
//    _atlcityChk->setChecked(true);
//    _germanChk->setChecked(false);
}

void ConfigTheme::apply()
{
  // get the selected filenames
  QString *namesF = _namesDict[_namesThm->currentText()];
  QString *boardF = _boardDict[_boardThm->currentText()];
  QString namesFname;
  QString boardFname;
  if (namesF) namesFname = *namesF;
  if (boardF) boardFname = *boardF;
  // change the theme in view
  _app->view()->changeTheme(namesFname, boardFname);
}


ConfigAppearance::ConfigAppearance(KapitalistApp *app, QWidget */*parent*/)
  : _app(app)
{
}

ConfigAppearance::~ConfigAppearance()
{
}

void ConfigAppearance::reset()
{
}

void ConfigAppearance::apply()
{
}
