// -*- C++ -*-

/***************************************************************************
                          connectdialog.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// Qt
//

#include <qdialog.h>
#include <qframe.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qcheckbox.h>

#include <klocale.h>

//
// App
//

#include "kapitalist.h"
#include "global.h"

#include "connectdialog.h"

//
// View
//

#include "kapview.h"

/**
 *
 */

ConnectDialog::ConnectDialog(KapitalistApp *app, QWidget *parent,
			     const char *name)
  : QDialog(parent, name, TRUE),	// modal dialog
    _app(app)
{
  
//    int labw = 80, labh = 50, lew = 150, leh = 30, butw = 100, buth =30;

  setCaption(i18n("Connect to Server"));
//   setFixedSize(250, 160);

  // create Layouts
  QBoxLayout *topLayout = new QVBoxLayout(this, 10);
  QGridLayout *txtinputs = new QGridLayout(3, 2, 5);
  QBoxLayout *buttons = new QHBoxLayout();
  QBoxLayout *chkBoxLayout = new QHBoxLayout();

  // add txtinputs and buttons to topLayout
  topLayout->addStretch( 2 );
  topLayout->addLayout( txtinputs );
  topLayout->addLayout( chkBoxLayout );
  topLayout->addStretch( 1 );
  topLayout->addLayout( buttons );
  topLayout->addStretch( 2 );
  
  // set Spacing and Stretch
  txtinputs->setColStretch(0, 0);
  txtinputs->setColStretch(1, 10);

  // create three lines for name, server hostname and port and add them to 
  // Layout "txtinputs"

  QLabel *label;
  QLineEdit *ed;
  QCheckBox *chk;

  // name label and textfield
  label = new QLabel( this );
  label->setText(i18n("&Your Nickname:"));
  label->setMinimumSize( label->sizeHint() );
  txtinputs->addWidget( label, 0, 0 );
  
  nameLEdit = ed = new QLineEdit( this );
  ed->setMinimumSize( ed->sizeHint() );
  connect(ed, SIGNAL( returnPressed() ), this, SLOT( name_entered() ));
  txtinputs->addWidget( ed, 0, 1 );
  label->setBuddy( ed ); 

  
  // hostname label and textfield
  label = new QLabel( this );
  label->setText(i18n("&Hostname:"));
  label->setMinimumSize( label->sizeHint() );
  txtinputs->addWidget( label, 1, 0 );

  hostnameLEdit = ed = new QLineEdit( this );
  ed->setText(DEFAULT_HOST);
  ed->setMinimumSize( ed->sizeHint() );
  connect(ed, SIGNAL( returnPressed() ), this, SLOT( hostname_entered() ));
  txtinputs->addWidget( ed, 1, 1 );
  label->setBuddy( ed );


  // port label and textfield
  label = new QLabel( this );
  label->setText(i18n("&Port:"));
  label->setMinimumSize( label->sizeHint() );
  txtinputs->addWidget( label, 2, 0 );
  
  portLEdit = ed = new QLineEdit( this );
  QString text;
  text.setNum(DEFAULT_SOCK_PORT);
  ed->setText(text);
  ed->setMinimumSize( ed->sizeHint() );
  connect(ed, SIGNAL( returnPressed() ), this, SLOT( port_entered() ));
  txtinputs->addWidget( ed, 2, 1 );
  label->setBuddy( ed );

  serverStartChkBox = chk = new QCheckBox( i18n("Start new server"), this );
  chk->setMinimumHeight( chk->sizeHint().height() );
  chk->setChecked(false);
  chkBoxLayout->addWidget( chk );
  
  // setup buttons "Connect" and "Cancel" and add them to Layout "buttons"

  QPushButton* but;

  connectButton = but = new QPushButton( i18n("Connect") , this );
  but->setMinimumSize( but->sizeHint() );
  connect(but, SIGNAL( clicked() ), this, SLOT( slotConnect() ) );
  buttons->addWidget( but );

  cancelButton = but = new QPushButton( i18n("Cancel") , this );
  but->setMinimumSize( but->sizeHint() );
  connect(but, SIGNAL( clicked() ), this, SLOT( slotCancel() ));
  buttons->addWidget( but );

  nameLEdit->setFocus();
  topLayout->activate();

  hide();
}

ConnectDialog::~ConnectDialog()
{
}


void ConnectDialog::slotConnect()
{
  QString portstr(portLEdit->text());
  bool ok;
  int port;
  char errbuf[512];

  // try to convert portstr to an uint
  port = portstr.toUInt(&ok);

  if (!ok) {	// port string was not ok

    _app->view()->appendChatMsg("Connection impossible, wrong port");
    portLEdit->setText(portstr.sprintf("%d", DEFAULT_SOCK_PORT));

  } else {	// ok, so connect

    if (serverStartChkBox->isChecked()) {
      // start a new server
      if (!_app->client_start_server_and_connect(nameLEdit->text(), port)) {
	_app->view()->appendChatMsg("FAILED starting server");
      } else {
	accept();
      }

    } else {  // don't start internal server 

      if ( _app->connect_to_server(nameLEdit->text(), 
				   hostnameLEdit->text(), 
				   port,
				   errbuf,
				   sizeof(errbuf)) ) {	// an error occured
      } else {	// ok, we are connected
	accept();
      }

    }
  }
    
}
void ConnectDialog::slotCancel()
{
  reject();
}

