// -*- C++ -*-
/***************************************************************************
                          board.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// App
//

#include "global.h"

//
// Game
//

#include "board.h"
#include "field.h"
#include "street.h"
#include "railroad.h"
#include "utility.h"
#include "comchest.h"
#include "chance.h"
#include "taxfield.h"
#include "jailfield.h"
#include "monop.h"


Board::Board(KapGame *game, struct packet_board_init_info *binfo)
  : _game(game)
{
  int loc, no;
  
  assert(_game);

  // create Fields
  
  for (loc=0; loc<BOARD_N_FIELDS; ++loc) {
    switch (binfo->field[loc].type) {

    case STREET:
      _field[loc] = new Street(_game, &binfo->field[loc]);
      break;

    case RAILROAD:
      _field[loc] = new Railroad(_game, &binfo->field[loc]);
      break;

    case UTILITY:
      _field[loc] = new Utility(_game, &binfo->field[loc]);
      break;

    case COMCHEST:
      _field[loc] = new Comchest(_game, &binfo->field[loc]); 
      break;

    case CHANCE:
      _field[loc] = new Chance(_game, &binfo->field[loc]); 
      break;

    case TAXFIELD:
      _field[loc] = new Taxfield(_game, &binfo->field[loc]); 
      break;

    case JAILFIELD:
      _field[loc] = new Jailfield(_game, &binfo->field[loc]); 
      break;

    case SAFEFIELD: 
      _field[loc] = new Field(_game, &binfo->field[loc]); 
      break;

    default:
      assert(0);	// bug: no other types possible
    }
  }

  // create Monops
  
  for (no=0; no<BOARD_N_MONOPS; no++) {
    _monop[no] = new Monop(_game, _field, &binfo->monop[no]);
  }    

}

Board::~Board()
{
  int i;

  for (i=0; i<BOARD_N_FIELDS; i++)
    delete _field[i];
  for (i=0; i<BOARD_N_MONOPS; i++)
    delete _monop[i];
}


/*
 * fill info packets with data (not implemented here)
 */

//  void Board::init_info_for(Player *player, struct packet_board_init_info *pack);
//  void Board::info_for(Player *player, struct packet_board_info *pack);

void Board::update(struct packet_board_info *binfo)
{
  int i;

  for (i=0; i<BOARD_N_FIELDS; ++i)
    _field[i]->update(&binfo->field[i]);

  for (i=0; i<BOARD_N_MONOPS; ++i)
    _monop[i]->update(&binfo->monop[i]);
}


