// -*- C++ -*-
/***************************************************************************
                          field.h  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef FIELD_H
#define FIELD_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//
// QT
//

#include <qobject.h>

//
// App
//

#include "global.h"
#include "packets.h"

//
// Game
//

class KapGame;
class Player;



enum field_type {
  SAFEFIELD = 0,	/* safe spot			*/

  STREET,		/* normal property		*/
  RAILROAD,		/* railroad			*/
  UTILITY,		/* water works - electric co	*/

  COMCHEST,		/* community chest		*/
  CHANCE,		/* chance (surprise!!!)		*/

  TAXFIELD,		/* tax field			*/

  JAILFIELD		/* Go To Jail and Just visiting */
};

enum field_statusflags {
  IS_OWNED = 1,
  IS_YOURS = 2,
  IS_MORTGAGED = 4,
  IS_MONOP = 8
};

enum field_actionflags {
  CAN_BUY = 1,

  CAN_MORTGAGE = 2,
  CAN_UNMORTGAGE = 4,

  CAN_BUILD_HOUSES = 8,
  CAN_SELL_HOUSES = 16
};


/* need undefined state for int values */
#define UNDEFINED  -1


/**
 * TODO
 */
class Field : public QObject {
  Q_OBJECT
  
public:
  
//    Field(const int location);

  Field(KapGame *game, struct packet_field_init_info *);

  virtual ~Field();

  const int location() const { return _location; }

  enum field_type type() const { return _type; }

  const int status() const { return statusflags; }
  const int actions() const { return actionflags; }

//    virtual void enter(Player *);

//    virtual void init_info_for(Player *player, struct packet_field_init_info *);
//    virtual void info_for(Player *player, struct packet_field_info *, int event);

  virtual void update(struct packet_field_info *);

protected:
  
  KapGame *_game;

  int _location;
  enum field_type _type;
  
  /* 
   * server side:
   * These flags are player-specific and must be updated for other players.
   * This is done in each call of info_for or init_info_for.
   *
   * client side:
   * you can use these flags without updating them, they
   * are set by the method update each time the server sends info
   *
   */
  int statusflags;
  int actionflags;

};

#endif /* FIELD_H */
