// -*- C++ -*-
/***************************************************************************
                          player.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// System
//

#include <stdlib.h>
#include <assert.h>

//
// Game
//

#include "kapgame.h"
#include "board.h"
#include "property.h"
#include "player.h"



Player::Player(KapGame *game, struct packet_player_init_info *pinfo)
  : _game(game), 
//      _conn(NULL),
    _id(pinfo->playerid),
    _location(GO),
    _nturns_in_jail(0),
    _budget(GAME_STARTBUDGET),
    _last_transaction(0),
    _last_partner(0),
    _ncards(0),
    _bank_debts(0),
    _statusflags(0),
    _actionflags(0)
{
  int i;

  assert(_game);

  strcpy(_name, pinfo->name);
  strcpy(_addr, pinfo->addr);

  for (i=0; i<BOARD_N_FIELDS; ++i) {
    _holdings[i] = NULL;
  }

  for (i=0; i<GAME_MAX_PLAYERS; ++i) {
    _debts[i] = 0;
    _trade[i] = NULL;
  }

}

int const Player::bank_debts() const
{
  return _bank_debts;
}

int const Player::debts(const int plr_no) const
{
  return _debts[plr_no];
}

int const Player::debts_sum() const
{
  int i;
  int sum = 0;

  sum += _bank_debts;
  for(i=0; i<GAME_MAX_PLAYERS; ++i) {
    sum += _debts[i];
  }

  return sum;
}

void Player::update(struct packet_player_info *pinfo)
{
  int i;

  assert(_id == pinfo->playerid);

  _statusflags = pinfo->statusflags;
  _actionflags = pinfo->actionflags;

  _location = pinfo->location;
  _nturns_in_jail = pinfo->nturns_in_jail;

  _budget = pinfo->budget;
  _last_transaction = pinfo->transaction;
  _last_partner = ( (!_game->nobody(pinfo->partner)) 
		    ? _game->player(pinfo->partner) : NULL );

  for (i=0; i<BOARD_N_FIELDS; ++i)
    _holdings[i] = ( (pinfo->holdings[i]) 
		     ? dynamic_cast<Property *>(_game->board()->field(i))
		     : NULL );
  _ncards = pinfo->ncards;

  _bank_debts = pinfo->bank_debts;
  for (i=0; i<GAME_MAX_PLAYERS; ++i)
    _debts[i] = pinfo->debts[i];

}




