// -*- C++ -*-
/***************************************************************************
                          player.h  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef PLAYER_H
#define PLAYER_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//
// System
//

#include <assert.h>
#include <string.h>

//
// App
//

#include "global.h"
#include "packets.h"

//
// Game
//

#include "board.h"

class KapGame;
class Trade;
class Property;



enum player_statusflags {
  IS_CONNECTED = 1,
  IS_ALIVE = 2,
  IS_AFLOAT = 4
};

enum player_actionflags {
  CAN_DECLARE_BANKRUPT = 1
};

enum {
  PASS_GO,
  DONT_PASS_GO
};



/**
 * TODO
 */
class Player {
  
public:
  
  /*
   * client constructor
   */
  Player(KapGame *game, struct packet_player_init_info *);


  const char *name() const
    { return _name; }
  // to avoid warnings in C-code, return non-const connection
//    struct connection *conn() const 
//      { return _conn; }
  const char *addr() const 
    { return _addr; }

  const int id() const
    { return _id; }

  const int location() const
    { return _location; }
  const int nturns_in_jail() const
    { return _nturns_in_jail; }
  const bool is_in_jail() const
    { return _location == JAIL; }

  const int budget() const
    { return _budget; }

  int const bank_debts() const;
  int const debts(const int plr_no) const;
  int const debts_sum() const;

  const int last_transaction() const
    { return _last_transaction; }
  Player * const last_partner() const
    { return _last_partner; }

  const bool is_alive() const
    { return (_statusflags & IS_ALIVE); }
//    const bool is_solvent() const
//      { return _budget >= 0; }

  Property * const holdings(const int i) const
    { return _holdings[i]; }

  const int nrailroads() const
    { return ( ( (_holdings[RRSOUTH])  ? 1 : 0 ) +
	       ( (_holdings[RRWEST])   ? 1 : 0 ) +
	       ( (_holdings[RRNORTH])  ? 1 : 0 ) +
	       ( (_holdings[RRCENTRAL])? 1 : 0 )  ); }

  const int nutils() const
    { return ( ( (_holdings[UELECTRIC]) ? 1 : 0 ) +
	       ( (_holdings[UWATER])    ? 1 : 0 )  ); }
  const int ncards() const
    { return _ncards; }

  const bool has_enough_money(const int price) const
    { return (_budget>=price); }

  const int status() const { return _statusflags; }
  const int actions() const { return _actionflags; }

  void set_name(const char *name) 
    { strncpy(_name, name, PLAYER_MAX_LENGTH_NAME); }
  void set_id(const int id)
    { _id = id; }
  
  /*
   * update data with infos from server
   */
  
  void update(struct packet_player_info *);

  /*
   * trade affairs
   */
  Trade *get_trade_with(Player *plr)
    { return _trade[plr->id()]; }
  void set_trade_with(Player *plr, Trade *trade)
    { _trade[plr->id()] = trade; }
  void clear_trade_with(Player *plr)
    { _trade[plr->id()] = NULL; }

protected:

  KapGame *_game;

  char _name[PLAYER_MAX_LENGTH_NAME];

  // network stuff
//    struct connection *_conn;
  char _addr[PLAYER_MAX_LENGTH_ADDRESS];

  int _id;
  int _location;
  int _nturns_in_jail;

  int _budget;
  int _last_transaction;
  Player *_last_partner;

  // note: properties the player doesn't own are NULL
  Property * _holdings[BOARD_N_FIELDS];
  int _ncards;

  // debts
  int _bank_debts;
  int _debts[GAME_MAX_PLAYERS];

  Trade *_trade[GAME_MAX_PLAYERS];

  // note: trades that are not existent are NULL

  /* 
   * server side: statusflags aren't used, only generated in info_for and
   * sent to client
   *
   * client side: statusflags can be used after update
   */
  int _statusflags;
  int _actionflags;
};




#endif /* PLAYER_H */
