// -*- C++ -*-
/***************************************************************************
                          kapgame.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// System
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <assert.h>

//
// Qt
//

#include <qdir.h>
#include <qwidget.h>

//
// KDE
//

#include <klocale.h>
#include <kmessagebox.h>
#include <kio/job.h>
#include <kio/netaccess.h>

//
// App
//

#include "kapitalist.h"
#include "global.h"
#include "log.h"
#include "mem.h"
#include "shared.h"
#include "packets.h"
#include "events.h"
#include "console.h"

//
// Game
//

#include "kapgame.h"
#include "board.h"
#include "field.h"
#include "monop.h"
#include "player.h"

//
// View
//

#include "kapview.h"



QList<KapView> *KapGame::pViewList = 0L;

KapGame::KapGame(QWidget *parent, const char *name)
  : QObject(parent, name),
    _myPID(-1), 
    _board(0),
    _nplayers(0),
    _startbudget(0),
    _income_on_GO(0),
    _curr_plr(0),
    _rolled_dice(false),
    _ndoubles(0),
    _turn_done(false)
{
  int i,j;

  if(!pViewList)
  {
    pViewList = new QList<KapView>();
  }

  pViewList->setAutoDelete(true);

  for (i=0; i<GAME_MAX_PLAYERS; ++i)
    _player[i] = 0;
  for (i=0; i<GAME_MAX_PLAYERS; ++i)
    for (j=0; j<GAME_MAX_PLAYERS; ++j)
      _trade[i][j] = 0;

  _last_dice[0] = _last_dice[1] = 1;

  game_state = GAME_EMPTY_STATE;
}

KapGame::~KapGame()
{
  int i,j;

  if (_board) delete _board;

  for (i=0; i<GAME_MAX_PLAYERS; ++i) {
    if (_player[i]) delete _player[i];
  }

  for (i=0; i<GAME_MAX_PLAYERS; ++i)
    for (j=0; j<GAME_MAX_PLAYERS; ++j)
      if (_trade[i][j]) delete _trade[i][j];

}

KapitalistApp *KapGame::getApp() const
{
  KapitalistApp *theApp = dynamic_cast<KapitalistApp *>(parent());
  assert(theApp);
  return theApp;
}


Player * const KapGame::myPlr() const
{
  assert(!nobody(_myPID)); 
  return player(_myPID);
}

Player * const KapGame::currPlr() const
{

  if (nobody(_curr_plr)) return 0;
  else return player(_curr_plr);
}



void KapGame::addView(KapView *view)
{
  pViewList->append(view);
}

void KapGame::removeView(KapView *view)
{
  pViewList->remove(view);
}

void KapGame::slotUpdateAllViews(KapView *sender)
{
  KapView *w;
  if(pViewList)
  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())
    {
      if(w!=sender)
        w->repaint();
    }
  }

}

bool KapGame::newGame(struct packet_game_init_info *ginfo)
{
  /////////////////////////////////////////////////
  // TODO: Add your game initialization code here
  /////////////////////////////////////////////////

  _board = new Board(this, &ginfo->board);
  _nplayers = ginfo->nplayers;
  _startbudget = -1;
  _income_on_GO = -1;
  _curr_plr = -1;
  _rolled_dice = false;
  _ndoubles = 0;
  _turn_done = false;

  int i,j;
  _myPID = ginfo->your_plrid;
  
  for (i=0; i<_nplayers; ++i)
    _player[i] = new Player(this, &ginfo->player[i]);
  for (i=_nplayers; i<GAME_MAX_PLAYERS; ++i)
    _player[i] = NULL;

  _last_dice[0] = _last_dice[1] = 0;

  for (i=0; i<GAME_MAX_PLAYERS; ++i)
    for (j=0; j<GAME_MAX_PLAYERS; ++j)
      _trade[i][j] = _trade[j][i] = NULL;


  game_state = GAME_ESTABLISHED_STATE;
  
  return true;
}

//  void KapGame::closeGame()
//  {
//    int i,j;

//    if (_board) delete _board;
//    _board = 0;
//    for (i=0; i<GAME_MAX_PLAYERS; ++i) {
//      if (_player[i]) delete _player[i];
//      _player[i] = 0;
//    }
//    for (i=0; i<GAME_MAX_PLAYERS; ++i)
//      for (j=0; j<GAME_MAX_PLAYERS; ++j) {
//        if (_trade[i][j]) delete _trade[i][j];
//        _trade[i][j] = _trade[j][i] = 0;
//      }

//    game_state = GAME_EMPTY_STATE;

//  }


int KapGame::run()
{
  // TODO: client specific Initialization of game data structur

  return 0;
}


Player * const KapGame::player(int id) const
{
  assert(!nobody(id)); 
  return _player[id];
}
  

void KapGame::handle_your_turn(struct packet_generic_integer *packet)
{
  _curr_plr = packet->value;
  assert(!nobody(_curr_plr));

  freelog(LOG_NORMAL, "your_turn received");
}

void KapGame::handle_dice_rolled(struct packet_dice_rolled *packet)
{
  _last_dice[0] = packet->dice1;
  _last_dice[1] = packet->dice2;

  freelog(LOG_NORMAL, "dice_rolled received: plr=%d, dice1=%d, dice2=%d", 
	  packet->playerid, packet->dice1, packet->dice2);
}

void KapGame::handle_card_drawn(struct packet_generic_values *packet)
{

  freelog(LOG_NORMAL, "card_drawn received: plr=%d, type=%d, card_no=%d", 
	  packet->id, packet->value1, packet->value2);
}



void KapGame::handle_trade_info(struct packet_trade *packet)
{
  freelog(LOG_NORMAL, "trade info received:");

  assert(!nobody(packet->srcid) && !nobody(packet->destid));

  Player *plr = _player[packet->srcid];
  Player *dest = _player[packet->destid];
  Trade *trade = _trade[packet->srcid][packet->destid];

  if (!trade) {	// no trade object yet -> create one
    trade = new Trade(this, packet);
    set_trade(plr, dest, trade);
  } else {	// trade object exists -> update it
    assert(trade);
    trade->update(packet);
  }
  
}



void KapGame::handle_field_info(struct packet_field_info *packet)
{
  int loc = packet->location;

  freelog(LOG_NORMAL, "field_info received: loc=%d, event=%d", 
	  loc, packet->event);

  assert(loc>=0 && loc <BOARD_N_FIELDS);
  _board->field(loc)->update(packet);
}

void KapGame::handle_monop_info(struct packet_monop_info *packet)
{
  int id = packet->monopid;

  freelog(LOG_NORMAL, "monop_info received: id=%d, event=%d", 
	  id, packet->event);

  assert(id>=0 && id <BOARD_N_MONOPS);
  _board->monop(id)->update(packet);
}


// handle_game_init_info: see bottom of file

void KapGame::handle_game_info(struct packet_game_info *packet)
{
  freelog(LOG_NORMAL, "game_info received: event=%d", 
	  packet->event);

  update(packet);
}

/* this is not needed (so far), because game_init_info includes this
 */
//  void KapGame::handle_player_init_info(struct packet_player_init_info *packet)
//  { assert(0); }


void KapGame::handle_player_info(struct packet_player_info *packet)
{
  int plrid = packet->playerid;

  freelog(LOG_NORMAL, "player_info received: plrid=%d, event=%d", 
	  plrid, packet->event);

  if (packet->event == E_LOST_CONNECTION 
      && getApp()->client_state() != CLIENT_GAME_RUNNING_STATE) {
    // no player objects yet, so do nothing
    // there was already a message from server, so we are well informed
    return;
  }

  assert(plrid>=0 && plrid <_nplayers);
  _player[plrid]->update(packet);
}




/*
 * game flow (not implemented here, server controls game flow)
 */

/*
 * server send methods (not implemented here)
 */

/* 
 * client send methods
 */

void KapGame::send_turn_done()
{
  struct packet_generic_integer pack;
  
  pack.value = _myPID;

  send_packet_generic_integer(getApp()->conn(), PACKET_TURN_DONE, &pack);
}


void KapGame::send_dice_roll_request()
{
  struct packet_generic_integer pack;
  
  pack.value = _myPID;
  
  send_packet_generic_integer(getApp()->conn(), PACKET_DICE_ROLL_REQUEST, &pack);
}


/*
 * update data with infos from server
 */

void KapGame::update(struct packet_game_info *ginfo)
{
  int i;

  _board->update(&ginfo->board);
  
  _nplayers = ginfo->nplayers;
  
  for (i=0; i<_nplayers; ++i)
    _player[i]->update(&ginfo->player[i]);

}

void KapGame::handle_game_init_info(struct packet_game_init_info *packet)
{
  assert(game_state == GAME_EMPTY_STATE);
  newGame(packet);
}

bool KapGame::plr_have_trade(Player *p1, Player *p2)
{ 
  return (_trade[p1->id()][p2->id()] || _trade[p2->id()][p1->id()]); 
}

Trade *KapGame::get_trade(Player *p1, Player *p2)
{
  Trade *t = _trade[p1->id()][p2->id()];
  if (!t) t = _trade[p2->id()][p1->id()];
  else assert(_trade[p2->id()][p1->id()] == NULL);
  return t;
}

void KapGame::set_trade(Player *p1, Player *p2, Trade *t)
{
  assert(!plr_have_trade(p1, p2));
  _trade[p1->id()][p2->id()] = t;
  p1->set_trade_with(p2, t);
  p2->set_trade_with(p1, t);
}

void KapGame::clear_trade(Player *p1, Player *p2)
{
  if (_trade[p1->id()][p2->id()]) delete _trade[p1->id()][p2->id()];
  if (_trade[p2->id()][p1->id()]) delete _trade[p2->id()][p1->id()];
  _trade[p1->id()][p2->id()] = NULL;
  _trade[p2->id()][p1->id()] = NULL;
  p1->clear_trade_with(p2);
  p2->clear_trade_with(p1);
}

void KapGame::clear_trade_if_finished(struct packet_trade *packet)
{
  Trade *trade = _trade[packet->srcid][packet->destid];
  if (trade && trade->is_finished()) {
    clear_trade(trade->get_src(), trade->get_dest());
  }
}
