// -*- C++ -*-
/***************************************************************************
                          kapgame.h  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KAPGAME_H
#define KAPGAME_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//
// Qt
//

#include <qobject.h>
#include <qstring.h>
#include <qptrlist.h>

//
// KDE
//

#include <kurl.h>

//
// App
//

#include "global.h"
#include "shared.h"

class KapitalistApp;

//
// Game
//

#include "player.h"
#include "trade.h"

class Board;
class Field;
class Monop;

//
// View
//

class KapView;



/**	
 * KapGame provides a game object for a game-view model.
 *
 * The KapGame class provides a game object that can be used
 * in conjunction with the classes KapitalistApp and KapView to
 * create a game-view model for standard KDE applications based
 * on KApplication and KMainWindow. Thereby, the game object is
 * created by the KapitalistApp instance and contains the game
 * structure with the according methods for manipulation of the
 * game data by KapView objects. Also, KapGame contains
 * the methods for serialization of the game data from and to
 * files.
 *
 * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.  
 * @version KDevelop version 1.2 code generation
 */
class KapGame : public QObject
{
  Q_OBJECT
public:
  /** 
   * Constructor for the gameclass
   */
  KapGame(QWidget *parent, const char *name=0);
  /** 
   * Destructor for the gameclass
   */
  ~KapGame();
  
  /**
   * TODO
   */
  KapitalistApp *getApp() const;


  int const myPID() const { return _myPID; }
  Player * const myPlr() const;

  Player * const currPlr() const;

  /** 
   * adds a view to the game which represents the game
   * contents. Usually this is your main view.
   */
  void addView(KapView *view);
  /** 
   * removes a view from the list of currently connected views
   */
  void removeView(KapView *view);
  /**
   * initializes the game generally
   */
  bool newGame(struct packet_game_init_info *);
  /**
   * closes the actual game
   */
//    void closeGame();
  
  int run();

  Board * const board() const { return _board; }
  Player * const player(int id) const;
  
  const int nplayers() const { return _nplayers; }
  
  const int startbudget() const { return _startbudget; }
  const int income_on_GO() const { return _income_on_GO; }
  
  bool is_on_turn(Player *plr) { return (_curr_plr == plr->id()); }
  bool nobody(int plrid) const { return (plrid <0 || plrid >= _nplayers); }

  const int die1() const { return _last_dice[0]; }
  const int die2() const { return _last_dice[1]; }

  const int last_dice_total() const { return _last_dice[0]+_last_dice[1]; }

  int add_player(Player *player);

  int remove_player(const int plrno);
  int remove_all_players();

  Player *find_player_by_name(char *name);
  


  /*
   * network input handler (server side)
   */
  void handle_turn_done(int player_no);
  void handle_dice_roll_request(int player_no);

  void handle_property_buy(Player *, struct packet_generic_integer *);
  void handle_property_mortgage(Player *, struct packet_generic_integer *);
  void handle_property_unmortgage(Player *, struct packet_generic_integer *);

  void handle_houses_buy(Player *, struct packet_house_request *);
  void handle_houses_sell(Player *, struct packet_house_request *);

  void handle_jail_free_request(Player *, struct packet_generic_integer *);

  void handle_trade_request(Player *, struct packet_trade *);

  /*
   * network input handler (client side)
   */
  void handle_your_turn(struct packet_generic_integer *);
  void handle_dice_rolled(struct packet_dice_rolled *);
  
  void handle_card_drawn(struct packet_generic_values *);

  void handle_trade_info(struct packet_trade *);

  void handle_field_info(struct packet_field_info *);
  void handle_monop_info(struct packet_monop_info *);

  void handle_game_init_info(struct packet_game_init_info *);
  void handle_game_info(struct packet_game_info *);

  void handle_player_init_info(struct packet_player_init_info *);
  void handle_player_info(struct packet_player_info *);

  
  /*
   * server send methods (public)
   */
  void send_your_turn(Player *dest, int plr_id);
  void send_dice_rolled(Player *dest, int dicevalue[]);
  void send_card_drawn(Player *dest, Player *src, 
		       int cardtype, int card_no);
  void send_trade(Player *dest, Trade *, int event, int offer_from);

  void send_game_init_info(Player *dest);
  void send_game_info(Player *dest);

  void send_player_init_info(Player *dest, Player *obj);
  void send_player_info(Player *dest, Player *obj, int event);

  void send_field_info(Player *dest, Field *, int event);
  void send_monop_info(Player *dest, Monop *, int event);

  void send_all_info(Player *dest);

  /*
   * client send methods (public)
   */
  void send_turn_done();
  void send_dice_roll_request();
//    void send_trade(Trade *, int event);


  /*
   * sends used on both sides
   */


  /*
   * game flow
   */
  void next_turn();
  void pre_dice();	// not implemented yet
  void post_dice();	// not implemented yet
  void end_turn();

  void roll_dice(int dicevalue[] = NULL);

  void no_2nd_turn()
    { _ndoubles = 0; }

  bool plr_have_trade(Player *p1, Player *p2);
  Trade *get_trade(Player *p1, Player *p2);
  void set_trade(Player *p1, Player *p2, Trade *t);
  void clear_trade(Player *p1, Player *p2);
  void clear_trade_if_finished(struct packet_trade *);

public slots:
    /**
     * calls repaint() on all views connected to the game object and
     * is called by the view by which the game has been changed.  As
     * this view normally repaints itself, it is excluded from the
     * paintEvent.
     */
    void slotUpdateAllViews(KapView *sender);
 	
public:	
  /**
   * the list of the views currently connected to the game
   */
  static QList<KapView> *pViewList;	

protected:

  enum game_states { 
    GAME_EMPTY_STATE,
    GAME_ESTABLISHED_STATE
  };

  enum game_states game_state;

  /**
   * TODO
   */
  int _myPID;

  Board *_board;

  /*
   * TODO: Spieler nach einer eindeutigen ID numerieren, nicht nach Feldindex
   */
  Player *_player[GAME_MAX_PLAYERS];
  Trade *_trade[GAME_MAX_PLAYERS][GAME_MAX_PLAYERS];

  int _nplayers;
  int _startbudget;
  int _income_on_GO;

  int _curr_plr;  // current player on turn
  int _last_dice[2];
  bool _rolled_dice;	// true, if current player has rolled the dice yet
  int _ndoubles;	// number of times current player rolled doubles 
			// this turn
  bool _turn_done;
  

protected:

//    Player *& player_by_id(int id) { return _player[id]; }

  void buffer_all_connections();
  void unbuffer_all_connections();

  /*
   * generate infos (server side)
   */
  void init_info_for(Player *player, struct packet_game_init_info *) const;
  void info_for(Player *player, struct packet_game_info *) const;


  int check_trade_packet_consistency(Player *, struct packet_trade *);

  /*
   * update data with infos from server (on client side)
   */
  
  void update(struct packet_game_info *);

};

#endif // KAPGAME_H

