// -*- C++ -*-
/***************************************************************************
                          kapitalist.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// temporary (\todo)
#include <iostream>

//
// System / Network
//

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <ctime>
#include <cassert>

#include <unistd.h>
// #include <sys/types.h>
#include <fcntl.h>

#include <ctype.h>
#include <errno.h>

#include <sys/socket.h>
#include <sys/signal.h>
#include <sys/uio.h>
#include <sys/time.h>
#include <sys/wait.h>

#include <pwd.h>

#ifdef AIX
#include <sys/select.h>
#endif

#include <signal.h>

#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>

//
// Qt
//

#include <qdir.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qsocketnotifier.h>

//
// KDE
//

#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <kstatusbar.h>

//
// App
//

#include "kapitalist.h"

#include "global.h"
#include "shared.h"
#include "log.h"
#include "mem.h"
#include "packets.h"
#include "version.h"
#include "events.h"
#include "console.h"

#include "connectdialog.h"
#include "jaildialog.h"
#include "housesdialog.h"
#include "tradedialog.h"
#include "configdialog.h"

//
// Game
//

#include "kapgame.h"
#include "player.h"
#include "actions.h"

//
// View
//

#include "kapview.h"
#include "playerview.h"
#include "inputline.h"  // for SERVER_COMMAND_PREFIX

// for statusbar
#define ID_STATUS_MSG 1

extern int errno;


KapitalistApp::KapitalistApp(QWidget* , const char* name)
  : KMainWindow(0, name), 
    _packetQueue(),
    _animationInProgress(false),
    _game(0), _view(0), 
    _connectDlg(0),
    _tradeDlg(0),
    _configDlg(0),
    _client_state(CLIENT_NO_STATE),
    _conn(),
    _sockNotifier(0)
{
  _packetQueue.setAutoDelete(true);

  config=kapp->config();

//    for (i=0; i<BOARD_N_MONOPS; ++i) {
//      _housesDlg[i] = 0;
//    }

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initStatusBar();
  initActions();

  set_client_state(CLIENT_BOOT_STATE);

  readOptions();

}

KapitalistApp::~KapitalistApp()
{
  ::close(_conn.sock);
//   disconnect_from_server();
  if (_sockNotifier) delete _sockNotifier;
  if (_connectDlg) delete _connectDlg;
}


void KapitalistApp::initActions()
{
  // actions in game menu
  gameNewWindow = new KAction(i18n("New &Window"), 0, 0, this, 
			      SLOT(slotGameNewWindow()), actionCollection(),
			      "game_new_window");
  gameConnect = new KAction(i18n("&Connect..."), 0, 0, this,
			    SLOT(slotGameConnect()), actionCollection(),
			    "game_connect");
  gameDisconnect = new KAction(i18n("&Disconnect"), 0, 0, this,
			       SLOT(slotGameDisconnect()), actionCollection(),
			       "game_disconnect");
  gameStart = new KAction(i18n("&Start Game"), 0, 0, this,
			  SLOT(slotGameStart()), actionCollection(),
			  "game_start");

  gameClose = KStdAction::close(this, SLOT(slotGameClose()), 
				actionCollection(), "game_close");
  
  gameQuit = KStdAction::quit(this, SLOT(slotGameQuit()), actionCollection(),
			      "game_quit");

  // actions in action menu
  actionRollDice = new KAction(i18n("&Roll Dice"), 0, 0, this,
			       SLOT(slotActionRollDice()), actionCollection(),
			       "action_roll_dice");
  actionDone = new KAction(i18n("&End Turn"), 0, 0, this,
			   SLOT(slotActionDone()), actionCollection(),
			   "action_done");
  actionPropertyBuy = new KAction(i18n("&Buy Property"), 0, 0, this,
				  SLOT(slotActionPropertyBuy()), 
				  actionCollection(), "action_property_buy");
  actionPay = new KAction(i18n("&Settle Debts"), 0, 0, this,
			  SLOT(slotActionPay()), actionCollection(),
			  "action_pay");
  actionBankruptcy = new KAction(i18n("&Declare Bankruptcy"), 0, 0, this,
				 SLOT(slotActionBankruptcy()),
				 actionCollection(), "action_bankruptcy");

  actionJail = new KAction(i18n("&Jail..."), 0, 0, this,
			   SLOT(slotActionJail()), actionCollection(),
			   "action_jail");
//    actionProperty = new KAction(i18n("&Property..."), 0, 0, this,
//  			       SLOT(slotActionProperty()), actionCollection(),
//  			       "action_property");
//    actionHouses = new KAction(i18n("&Houses..."), 0, 0, this,
//  			     SLOT(slotActionHouses()), actionCollection(),
//  			     "action_houses");
  actionTrade = new KAction(i18n("&Trade..."), 0, 0, this,
			    SLOT(slotActionTrade()), actionCollection(),
			    "action_trade");

  // actions in settings menu
  viewToolBars = KStdAction::showToolbar(this, SLOT(slotViewToolBars()),
					 actionCollection());
  viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()),
					    actionCollection());
  settingsConfigure = 
    KStdAction::preferences(this, SLOT(slotSettingsConfigure()), 
			    actionCollection());

  // statustexts for game menu
  gameNewWindow->setStatusText(i18n("Opens a new application window"));
  gameConnect->setStatusText(i18n("Connects to a Capitalist server"));
  gameDisconnect->setStatusText(i18n("Closes connection to Capitalist "
				     "server"));
  gameStart->setStatusText(i18n("Starts the game on the Capitalist server"));
  gameClose->setStatusText(i18n("Closes this window"));
  gameQuit->setStatusText(i18n("Quits the application"));

  // statustexts for action menu
  actionRollDice->setStatusText(i18n("Rolls the dice"));
  actionDone->setStatusText(i18n("Ends your turn"));
  actionPropertyBuy->setStatusText(i18n("Buys the property you are "
					"standing on"));
  actionPay->setStatusText(i18n("Pays your debts"));
  actionBankruptcy->setStatusText(i18n("Declares yourself bankrupt"));

  actionJail->setStatusText(i18n("Opens Jail Dialog"));
//    actionProperty->setStatusText(i18n("Opens Property Dialog"));
//    actionHouses->setStatusText(i18n("Opens Houses Dialog"));
  actionTrade->setStatusText(i18n("Opens Trade Dialog"));

  // statustexts for settings menu
  viewToolBars->setStatusText(i18n("Enables/disables the toolbars"));
  viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));
  settingsConfigure->setStatusText(i18n("Configures Kapitalist"));

  // icons for games menu
  gameNewWindow->setIcon("filenew");
  gameConnect->setIcon("connect_creating");
  gameDisconnect->setIcon("connect_no");
  gameStart->setIcon("player_play");

  // icons for action menu
  actionRollDice->setIcon("roll");
  actionDone->setIcon("done");
  actionPropertyBuy->setIcon("property_buy");
  actionPay->setIcon("pay");
  actionBankruptcy->setIcon("bankruptcy");

  actionJail->setIcon("jail");
//    actionProperty->setIcon("property");
//    actionHouses->setIcon("/home/daniel/kapitalist/kapitalist/houses.png");
  actionTrade->setIcon("trade");
  // TODO: other action icons
  
  // use the absolute path to your kapitalistui.rc file for testing purpose 
  // in createGUI();
//    createGUI("/home/daniel/kapitalist/kapitalist/kapitalistui.rc");
  
  createGUI();
  toolBar("gameGeneral")->setFullSize(false);
  toolBar("gameServer")->setFullSize(false);
  toolBar("actionTurn")->setFullSize(false);
  toolBar("actionDialogs")->setFullSize(false);

}


void KapitalistApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG);
}

void KapitalistApp::initGame()
{
  if (_game) delete _game;
  _game = new KapGame(this);
}

void KapitalistApp::initView()
{ 
  ////////////////////////////////////////////////////////////////////
  // create the main widget here that is managed by KTMainWindow's view-region
  // and connect the widget to your game to display game contents.

  if (_view) delete _view;
  _view = new KapView(this);

  // every time an idle cycle is finished in kapview, the packet queue is
  // reprocessed
  QObject::connect( _view, SIGNAL(idleCycleFinished()),
		    this, SLOT(slotProcessPacketQueue()) );
  QObject::connect( this, SIGNAL(packetQueued()),
		    _view, SLOT(slotFinishIdleCycle()) );
  _game->addView(_view);
  setCentralWidget(_view);

  // delete the TradeDialog (if it exists)
  if (_tradeDlg) delete _tradeDlg;
  _tradeDlg = 0;

  // trigger slotProcessPacketQueue when view is initialized
  slotProcessPacketQueue();

}

void KapitalistApp::open_sockNotifier()
{
  if (!_sockNotifier) {	//no notifier yet
    _sockNotifier = new QSocketNotifier( _conn.sock, 
					 QSocketNotifier::Read);
    QObject::connect( _sockNotifier, SIGNAL(activated(int)),
		      this, SLOT(slotNetInput()) );
  }
}




void KapitalistApp::saveOptions()
{	
  config->setGroup("General Options");
  if (_client_state > CLIENT_BOOT_STATE) {
    config->writeEntry("Geometry", size());
  }
  config->writeEntry("Show Toolbars", viewToolBars->isChecked());
  config->writeEntry("Show Statusbar",viewStatusBar->isChecked());
  config->writeEntry("gameGeneralToolBarPos", 
		     (int) toolBar("gameGeneral")->barPos());
  config->writeEntry("gameServerToolBarPos", 
		     (int) toolBar("gameServer")->barPos());
  config->writeEntry("actionTurnToolBarPos", 
		     (int) toolBar("actionTurn")->barPos());
  config->writeEntry("actionDialogsToolBarPos", 
		     (int) toolBar("actionDialogs")->barPos());
}


void KapitalistApp::readOptions()
{
	
  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbars = config->readBoolEntry("Show Toolbars", true);
  viewToolBars->setChecked(bViewToolbars);
  slotViewToolBars();

  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", false);
  viewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();


  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos = 
    (KToolBar::BarPosition) config->readNumEntry("gameGeneralToolBarPos", 
						 KToolBar::Top);
  toolBar("gameGeneral")->setBarPos(toolBarPos);

  toolBarPos = 
    (KToolBar::BarPosition) config->readNumEntry("gameServerToolBarPos", 
						 KToolBar::Top);
  toolBar("gameServer")->setBarPos(toolBarPos);

  toolBarPos = 
    (KToolBar::BarPosition) config->readNumEntry("actionTurnToolBarPos", 
						 KToolBar::Top);
  toolBar("actionTurn")->setBarPos(toolBarPos);

  toolBarPos = 
    (KToolBar::BarPosition) config->readNumEntry("actionDialogsToolBarPos", 
						 KToolBar::Top);
  toolBar("actionDialogs")->setBarPos(toolBarPos);

	
  _preferredSize = config->readSizeEntry("Geometry");
//    if(!_preferredSize.isEmpty()) resize(_preferredSize);
}

bool KapitalistApp::queryClose()
{
//    return _game->saveModified();
  return true;
}

bool KapitalistApp::queryExit()
{
  saveOptions();
  return true;
}

void KapitalistApp::saveProperties(KConfig */*cfg*/)
{
    // TODO: get game data from server and save it locally
//    if(_game->URL().fileName()!=i18n("Untitled") && !_game->isModified())
//    {
//      // saving to tempfile not necessary
//    }
//    else
//    {
//      KURL url=_game->URL();	
//      cfg->writeEntry("filename", url.url());
//      cfg->writeEntry("modified", _game->isModified());
//      QString tempname = kapp->tempSaveName(url.url());
//      QString tempurl= KURL::encode_string(tempname);
//      KURL _url(tempurl);
//      _game->saveGame(_url);
//    }
}


void KapitalistApp::readProperties(KConfig* /*cfg*/)
{
    // TODO: restore game
//    QString filename = cfg->readEntry("filename", "");
//    KURL url(filename);
//    bool modified = cfg->readBoolEntry("modified", false);
//    if(modified)
//    {
//      bool canRecover;
//      QString tempname = kapp->checkRecoverFile(filename, canRecover);
//      KURL _url(tempname);
//      if(canRecover)
//      {
//        _game->openGame(_url);
//        _game->setModified();
//        setCaption(_url.fileName(),true);
//        QFile::remove(tempname);
//      }
//    }
//    else
//    {
//      if(!filename.isEmpty())
//      {
//        _game->openGame(url);
//        setCaption(url.fileName(),false);
//      }
//    }
}		



void KapitalistApp::handle_packet_input(char *packet, int type)
{
  switch(type) {
	
  case PACKET_JOIN_GAME_REPLY:	/* handled here */
    handle_join_game_reply((struct packet_join_game_reply *)packet);
    break;

  case PACKET_SERVER_SHUTDOWN:  /* handled here */
    freelog(LOG_VERBOSE, "server shutdown");
    disconnect_from_server();
    _view->appendChatMsg("server has shut down\n"
				"Please reconnect later...");
    break;

  case PACKET_GAME_STATE:	/* handled here */
    handle_game_state((struct packet_generic_integer *)packet);
    break;


  case PACKET_YOUR_TURN:	/* handled in game.cpp */
    _game->handle_your_turn((struct packet_generic_integer *)packet);
    _view->handle_your_turn((struct packet_generic_integer *)packet);
    break;

  case PACKET_DICE_ROLLED:	/* handled in game.cpp */
    _game->handle_dice_rolled((struct packet_dice_rolled *)packet);
    _view->handle_dice_rolled((struct packet_dice_rolled *)packet);
    break;

  case PACKET_CARD_DRAWN:	/* handled in game.cpp */
    _game->handle_card_drawn((struct packet_generic_values *)packet);
    _view->handle_card_drawn((struct packet_generic_values *)packet);
    break;

  case PACKET_TRADE:	/* handled in game.cpp */
    _game->handle_trade_info((struct packet_trade *)packet);
    _view->handle_trade_info((struct packet_trade *)packet);
    _tradeDlg->update((struct packet_trade *)packet);
    _game->clear_trade_if_finished((struct packet_trade *)packet);
    break;

  case PACKET_FIELD_INFO:	/* handled in game.cpp */
    _game->handle_field_info((struct packet_field_info *)packet);
    _view->handle_field_info((struct packet_field_info *)packet);
    break;

  case PACKET_MONOP_INFO:	/* handled in game.cpp */
    _game->handle_monop_info((struct packet_monop_info *)packet);
    _view->handle_monop_info((struct packet_monop_info *)packet);
    // TODO: emit signal monopChanged()
//      updateHousesDialog((struct packet_monop_info *)packet);
    break;


//    case PACKET_PLAYER_INIT_INFO:	/* handled in game.cpp */
//      _game->handle_player_init_info((struct packet_player_init_info *)packet);
//      break;

  case PACKET_PLAYER_INFO:	/* handled in game.cpp */
    _game->handle_player_info((struct packet_player_info *)packet);
    _view->handle_player_info((struct packet_player_info *)packet);
    break;


  case PACKET_GAME_INIT_INFO:	/* handled in game.cpp */
    _game->handle_game_init_info((struct packet_game_init_info *)packet);
    _view->handle_game_init_info((struct packet_game_init_info *)packet);
    break;

  case PACKET_GAME_INFO:	/* handled in game.cpp */
    _game->handle_game_info((struct packet_game_info *)packet);
    _view->handle_game_info((struct packet_game_info *)packet);
    if (!_tradeDlg) {
      _tradeDlg = new TradeDialog( this, this );
      _tradeDlg->hide();
    }
    break;


  case PACKET_CHAT_MSG:		/* handled here */
    handle_chat_msg((struct packet_generic_message *)packet);
    break;

//    case PACKET_PAGE_MSG:		/* handled here */
//      handle_page_msg((struct packet_generic_message *)packet);
//      break;

  case PACKET_CONFIRM_ANIM_FINISHED:
    handle_confirm_anim_finished((struct packet_generic_integer *)packet);
    break;

  default:
    freelog(LOG_FATAL, "Received unknown packet from server! byebye");
    assert(0);
    break;
  }

  free(packet);
}


void KapitalistApp::queue_packet_input(void *packet, int type)
{
  PacketQueueEntry *q = new PacketQueueEntry();

  q->type = type;
  q->packet = packet;

  _packetQueue.append( q );

  emit packetQueued();
}

void KapitalistApp::slotProcessPacketQueue()
{
  PacketQueueEntry *qe;
  int type;
  void *packet;

  // if there is action in progress, we should not process the queue here
  if (_animationInProgress) return;

//    _view->appendChatMsg("slotProcessPacketQueue");
  qe = _packetQueue.first();

  if (!qe) { // queue is empty, so we can do an idle cycle
    // do an idle cyle (triggers slotProcessPacketQueue after finished)
    _view->slotIdleCycle();
    return;
  }
  // if queue is not empty, we process the packets one by one

  packet = qe->packet;
  type = qe->type;

  // remove the action immediately from the queue before we execute the packetQueue
  _packetQueue.removeFirst();

  handle_packet_input((char *)packet, type);

  // if no action is involved in the handlers executed by handle_packet_input
  // we can process the next action in the queue
  if (!_animationInProgress) {
    slotProcessPacketQueue(); // recursive execution stops when queue is empty!
  }
  return;
}

void KapitalistApp::handle_join_game_reply(struct packet_join_game_reply *packet)
{
  char msg[MSG_SIZE];

  if (packet->you_can_join) {
    freelog(LOG_VERBOSE, "join game accept:%s", packet->message);
    set_client_state(CLIENT_PRE_GAME_STATE);
  } else {
    sprintf(msg, "You were rejected from the game:\n\t\"%s\"", 
	    packet->message);
    set_client_state(CLIENT_BOOT_STATE);
    _view->appendChatMsg(msg);
  }
  
}

void KapitalistApp::handle_chat_msg(struct packet_generic_message *packet)
{
  // distinguish between Server errors and real chat messages
  // TODO: distinguish by packet type on the server already ?
  QString startstr(packet->message);
  startstr.truncate(5);
  if (startstr == "Game:") { // server error
    _view->appendCurrOut(packet->message);
  } else {  // chat message
    _view->appendChatMsg(packet->message);
  }
  
  if (packet->event >= E_LAST)  {
    freelog(LOG_NORMAL,"Unknown event type %d!", packet->event);
  }
}
 
void KapitalistApp::handle_game_state(struct packet_generic_integer *packet)
{
  set_client_state((enum client_states) packet->value);

  if(_client_state==CLIENT_GAME_RUNNING_STATE) {
    _view->appendChatMsg("Game is running...");
    /* TODO: alle Widgets auffrischen, (evtl. auch alles in set_cli_state?) */
/*      refresh_overview_canvas(); */
/*      refresh_overview_viewrect(); */
/*      enable_turn_done_button(); */
  }
}

void KapitalistApp::handle_confirm_anim_finished(struct packet_generic_integer */*packet*/)
{
  send_anim_finished();
}

void KapitalistApp::send_anim_finished()
{
  struct packet_generic_integer pack;

  pack.value = 0;

  send_packet_generic_integer(&_conn, PACKET_ANIM_FINISHED, &pack);

}

void KapitalistApp::set_client_state(enum client_states newstate)
{
  if(_client_state==newstate) return;

  _client_state=newstate;

  slotStatusMsg(i18n("changing client state..."));
    
  switch (newstate) {
    
  case CLIENT_NO_STATE:
  case CLIENT_BOOT_STATE:
    // TODO: restore initial state
    // clear game and view
    initGame();
    initView();
    
    // two calls of resize are necessary !! (both dimensions?)
    // TODO: find out, why this resize behaves so strange !!!
    resize(800,600); // first enlarge the window a little
    resize(_view->minimumSize()); // before we minimize it
    
    gameConnect->setEnabled(true);
    gameDisconnect->setEnabled(false);
    gameStart->setEnabled(false);
    
    disable_actions(A_ROLL_DICE | A_DONE | A_PAY | A_BANKRUPTCY |
		    A_PROPERTY_BUY |
		    A_JAIL | A_PROPERTY | A_HOUSES | 
		    A_TRADE);
    
    // GUI
    if (_sockNotifier) delete _sockNotifier; _sockNotifier = 0;
    //        slotGameConnect();
    break;
    
  case CLIENT_WAITING_FOR_SERVER_REPLY_STATE:
    // this is a GUI-only-state
    // we must be able to listen for server packets
    open_sockNotifier();
    break;
    
  case CLIENT_PRE_GAME_STATE:
  case CLIENT_SELECT_TOKEN_STATE:
  case CLIENT_WAITING_FOR_GAME_START_STATE:
    gameConnect->setEnabled(false);
    gameDisconnect->setEnabled(true);
    gameStart->setEnabled(true);
    // TODO
    break;
    
  case CLIENT_GAME_RUNNING_STATE:
    /* TODO: all steps PRE_GAME -> GAME_RUNNING */
    // game is created, when game_init_info is received
    // GUI
    if (!_preferredSize.isEmpty()) resize(_preferredSize);
    else resize(sizeHint());

    _view->createViews();
    // Actions
    gameConnect->setEnabled(false);
    gameDisconnect->setEnabled(true);
    gameStart->setEnabled(false);
    // enable some actions by default (TODO: some of them should be enabled
    // or disabled in some situations)
    enable_actions(A_TRADE);
    // TODO: Status message ?
    break;
  case CLIENT_GAME_OVER_STATE:
    disable_actions(A_ROLL_DICE | A_DONE | A_PAY | A_BANKRUPTCY |
		    A_PROPERTY_BUY |
		    A_JAIL | A_PROPERTY | A_HOUSES | 
		    A_TRADE);
    break;
  } 

  slotViewToolBars();
  slotStatusMsg(i18n("Ready."));
}

//
// the routines
//   client_kill_server()
//   server_shutdown()
//   client_start_server()
// are taken from freeciv-2.0.1/client/connectdlg_common.c
//

#define WAIT_BETWEEN_TRIES 100000 /* usecs */
#define NUMBER_OF_TRIES 500

static pid_t server_pid = - 1;

static bool is_server_running()
{
  return (server_pid > 0);
}

static void client_kill_server()
{
  if (is_server_running()) {
    kill(server_pid, SIGTERM);
    waitpid(server_pid, NULL, WUNTRACED);
    server_pid = - 1;
  }
}

//
// This is called at program exit.
//
static void server_shutdown(void)
{
  client_kill_server();
}

bool KapitalistApp::client_start_server_and_connect(const char *name,
						    const int server_port)
{
  
#if !defined(HAVE_FORK)
  /* Can't do much without fork */
  return FALSE;
#else
  char errbuf[512];
  int connect_tries = 0;
  static bool initialized = FALSE;

  /* only one server (forked from this client) shall be running at a time */
  client_kill_server();
  
  if (!initialized) {
    atexit(server_shutdown);
    initialized = TRUE;
  }

  _view->appendChatMsg("Starting server...");

  server_pid = fork();
  
  if (server_pid == 0) {
    int fd, argc = 0;
    const int max_nargs = 8;
    char *argv[max_nargs + 1], port_buf[32], logfile[50];
    /* inside the child */

    pid_t child_pid = getpid();

    /* Set up the command-line parameters. */
    snprintf(port_buf, sizeof(port_buf), "%d", server_port);
    snprintf(logfile, sizeof(logfile), "/tmp/capitalist.%d.log", child_pid);
    argv[argc++] = "capitalist";
    argv[argc++] = "-p";
    argv[argc++] = port_buf;
    if (logfile) {
      argv[argc++] = "--debug";
      argv[argc++] = "2";
      argv[argc++] = "--log";
      argv[argc++] = logfile;
    }
    argv[argc++] = "--firsthack";
    argv[argc] = NULL;
    assert(argc <= max_nargs);

    /* avoid terminal spam, but still make server output available */ 
    fclose(stdout);
    fclose(stderr);

    /* include the port to avoid duplication */
    if (logfile) {
      fd = open(logfile, O_WRONLY | O_CREAT);

      if (fd != 1) {
        dup2(fd, 1);
      }
      if (fd != 2) {
        dup2(fd, 2);
      }
      fchmod(1, 0644);
    }

    /* If it's still attatched to our terminal, things get messed up, 
      but capitalist server needs *something* */ 
    fclose(stdin);
    fd = open("/dev/null", O_RDONLY);
    if (fd != 0) {
      dup2(fd, 0);
    }

    /* these won't return on success */ 
    execvp("./capitalist", argv);
    execvp("capitalist", argv);
    execvp("/tmp/captest/bin/capitalist", argv);
    
    /* This line is only reached if capitalist server cannot be started, 
     * so we kill the forked process.
     * Calling exit here is dangerous due to X11 problems (async replies) */ 
    _exit(1);
  } 

  bool server_unreachable=false;

  /* a reasonable number of tries */ 
  while (connect_to_server(name, "localhost", server_port, 
			   errbuf, sizeof(errbuf)) == -1) {
    usleep(WAIT_BETWEEN_TRIES);
    if (waitpid(server_pid, NULL, WNOHANG) != 0) {
      server_unreachable = true;
      break;
    }
    if (connect_tries++ > NUMBER_OF_TRIES) {
      break;
    }

  }

//   /* weird, but could happen, if server doesn't support new startup stuff
//    * capabilities won't help us here... */ 
//   if (!_conn.used) {
  if (server_unreachable || connect_tries > NUMBER_OF_TRIES) {
    /* possible that server is still running. kill it */ 
    client_kill_server();

    _view->appendChatMsg("Couldn't connect to the server.");
    _view->appendChatMsg("We probably couldn't start it from here.");
    _view->appendChatMsg("You'll have to start one manually. Sorry...");
    return FALSE;
  }

  _view->appendChatMsg("Success starting server.");
  
  return TRUE;

#endif
 
}

int KapitalistApp::connect_to_server(const char *name, const char *hostname,
				     const int port,
				     char *errbuf, size_t errbufsize)
{
  /* use name to find TCPIP address of server */
  struct sockaddr_in src;
  struct hostent *ph;
  long address;
  struct packet_req_join_game req;

  if(isdigit((size_t)*hostname)) {
    if((address = inet_addr(hostname)) == -1) {
//        strcpy(errbuf, "Invalid hostname");
      strncpy(errbuf, "ERROR: Invalid hostname", errbufsize);
      return -1;
    }
    src.sin_addr.s_addr = address;
    src.sin_family = AF_INET;
  }
  else if ((ph = gethostbyname(hostname)) == NULL) {
//      strcpy(errbuf, "Failed looking up host");
      strncpy(errbuf, "ERROR: Failed looking up host", errbufsize);
    return -1;
  }
  else {
    src.sin_family = ph->h_addrtype;
    memcpy((char *) &src.sin_addr, ph->h_addr, ph->h_length);
  }
  
  src.sin_port = htons(port);
  
  /* ignore broken pipes */
  ::signal (SIGPIPE, SIG_IGN);
  
  if((_conn.sock = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
//      strcpy(errbuf, mystrerror(errno));
    snprintf(errbuf, errbufsize, "ERROR in creation of socket: %s",
	     mystrerror(errno));
    return -1;
  }
  
  if(::connect(_conn.sock, (struct sockaddr *) &src, sizeof (src)) < 0) {
//      strcpy(errbuf, mystrerror(errno));
    snprintf(errbuf, errbufsize, "ERROR in connect: %s", mystrerror(errno));
    ::close(_conn.sock);
    return -1;
  }

  _conn.buffer.ndata=0;

  /* hier ankoppeln an GUI */
/*    x_input_id=XtAppAddInput(app_context, _conn.sock,  */
/*  			   (XtPointer) XtInputReadMask, */
/*  			   (XtInputCallbackProc) get_net_input, NULL); */
  /* 
   * TODO: GUI socket handling should be is initialized here, but I moved this
   * to gui files/classes
   */
     
  
  /* now send join_request package */

  strcpy(req.name, name);
  req.major_version=MAJOR_VERSION;
  req.minor_version=MINOR_VERSION;
  req.patch_version=PATCH_VERSION;

  send_packet_req_join_game(&_conn, &req);
  
  return 0;
}

void KapitalistApp::disconnect_from_server(void)
{
  ::close(_conn.sock);
  set_client_state(CLIENT_BOOT_STATE);
}  

//  void KapitalistApp::get_net_input(int *fid)
//  {
//    /* TODO: fid sollte == _conn.sock sein, sonst funktioniert disconnect
//       nicht ! */
//    if(read_socket_data(*fid, &_conn.buffer)>0) {
//      int type;
//      void *packet;

//      while((packet= get_packet_from_connection(&_conn, &type))) {

//        // handle chat and error messages immediately
//        if ( (type == PACKET_CHAT_MSG) ||
//  	   (type == PACKET_JOIN_GAME_REPLY) ) {
//  	handle_packet_input((char *)packet, type);
//        } else {  // put all other packets into the queue
//  	queue_packet_input(packet, type);
//        }
//      }
//    }
//    else {
//      freelog(LOG_NORMAL, "lost connection to server");
//      /* TODO: disconnect_from_server instead */
//  /*      close(*fid); */
//  /*      set_client_state(CLIENT_BOOT_STATE); */
//      disconnect_from_server();
//      _view->appendChatMsg("Lost connection to server!");
//    }
//  }



// needed ???????
//  void KapitalistApp::close_server_connection(void);
/*  { */
/*    close(_conn.sock); */
/*  } */


void KapitalistApp::animationStarted()
{ 
  _animationInProgress = true; 
}

void KapitalistApp::animationFinished()
{ 
  _animationInProgress = false;
  slotProcessPacketQueue(); // trigger queue processing again
}

void KapitalistApp::enable_actions(int flags, bool enable)
{
  if (flags & A_ROLL_DICE)    { actionRollDice->setEnabled(enable); }
  if (flags & A_DONE)         { actionDone->setEnabled(enable); }
  if (flags & A_PROPERTY_BUY) { actionPropertyBuy->setEnabled(enable); }
  if (flags & A_PAY)         { actionPay->setEnabled(enable); }
  if (flags & A_BANKRUPTCY)   { actionBankruptcy->setEnabled(enable); }

  if (flags & A_JAIL) { actionJail->setEnabled(enable); }
//    if (flags & A_PROPERTY) { actionProperty->setEnabled(enable); }
//    if (flags & A_HOUSES) { actionHouses->setEnabled(enable); }
  if (flags & A_TRADE) { actionTrade->setEnabled(enable); }

}

void KapitalistApp::disable_actions(int flags)
{
  enable_actions(flags, false);
}


/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KapitalistApp::slotGameNewWindow()
{
  slotStatusMsg(i18n("Opening a new application window..."));
	
  KapitalistApp *new_window= new KapitalistApp();
  new_window->show();

  slotStatusMsg(i18n("Ready."));
}

void KapitalistApp::slotGameConnect()
{
  slotStatusMsg(i18n("Connecting to Capitalist server..."));

  // open connect dialog

  if (_connectDlg == 0) _connectDlg = new ConnectDialog(this);

  // first connect to server (modal!)
  if (_connectDlg->exec()) { 
    // sending was successful, but we aren't connected yet, because
    // there could have been an error message from the server !
    set_client_state(CLIENT_WAITING_FOR_SERVER_REPLY_STATE);
  } else { // only possibility is, that the user has clicked "Cancel"
    // do nothing
//      slotGameQuit();
  }

  slotStatusMsg(i18n("Ready."));
}


void KapitalistApp::slotGameDisconnect()
{
  slotStatusMsg(i18n("Closing connection to Capitalist server ..."));
	
  disconnect_from_server();
  _view->appendChatMsg("closed connection to server");

  slotStatusMsg(i18n("Ready."));
}

void KapitalistApp::slotGameStart()
{
  slotStatusMsg(i18n("Starting Game on Capitalist server..."));

  // send a chat packet containing command "/start"
  struct packet_generic_message pack;
	
  strcpy(pack.message, "Xstart");
  pack.message[0] = SERVER_COMMAND_PREFIX;
  send_packet_generic_message(_view->getApp()->conn(),
			      PACKET_CHAT_MSG, &pack);

  slotStatusMsg(i18n("Ready."));
}

void KapitalistApp::slotGameClose()
{
  slotStatusMsg(i18n("Closing client..."));
	
  if (!close()) {
    // closing was unsuccessful
    slotStatusMsg(i18n("Ready."));
  }
}


void KapitalistApp::slotGameQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  saveOptions();
  // close the first window, the list makes the next one the first
  // again.  This ensures that queryClose() is called on each window
  // to ask for closing
  KMainWindow* w;
//    bool successful = true;
  if(memberList)
  {
    QPtrListIterator<KMainWindow> it(*KMainWindow::memberList);
    for (it.toFirst(); it.current(); ++it) {
      w = it.current();
      // only close the window if the closeEvent is accepted. If the
      // user presses Cancel on the saveModified() dialog, the window
      // and the application stay open.
      if(!w->close())
//  	successful = false;
	break;
    }
  }	
  
//    if (!successful) slotStatusMsg(i18n("Ready."));
}

void KapitalistApp::slotActionRollDice()
{ 
  slotRollDice(); 
}
void KapitalistApp::slotActionDone()
{
  slotDone();
}
void KapitalistApp::slotActionPropertyBuy()
{ 
  slotPropertyBuy(_view->getGame()->myPlr()->location()); 
}
void KapitalistApp::slotActionPay()
{
  slotPayDebts();
}
void KapitalistApp::slotActionBankruptcy()
{
  slotBankruptcy();
}



void KapitalistApp::slotActionJail()
{
  JailDialog dlg(this, this);

  QObject::connect(&dlg, SIGNAL(user1Clicked()), this, SLOT(slotJailCard()));
  QObject::connect(&dlg, SIGNAL(user2Clicked()), this, SLOT(slotJailPay()));

  dlg.exec();
//    if (dlg.exec()) { 
//      slotStatusMsg(i18n("JailDialog successful."));
//    } else {
//      slotStatusMsg(i18n("JailDialog unsuccessful."));
//    }
  
//    KMessageBox::questionYesNo(this, "Hi guys");

}

void KapitalistApp::slotActionHouses(int monopid)
{
  assert(monopid >= 0 && monopid < BOARD_N_MONOPS);
  HousesDialog *dlg = new HousesDialog(this, this, monopid);

  dlg->show();
//    if (!_housesDlg[monopid])
//      _housesDlg[monopid] = new HousesDialog(this, this, monopid);
  
//    _housesDlg[monopid]->show();
}

//  void KapitalistApp::slotActionProperty(){}
//  void KapitalistApp::slotActionHouses(){}


void KapitalistApp::slotActionTrade()
{
  if (!_tradeDlg)
    _tradeDlg = new TradeDialog(this, this);
  else {
    _tradeDlg->show();
    _tradeDlg->updateHoldings();
  }
}



void KapitalistApp::slotViewToolBars()
{
  slotStatusMsg(i18n("Toggling toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!viewToolBars->isChecked())
  {
    toolBar("gameGeneral")->hide();
    toolBar("gameServer")->hide();
    toolBar("actionTurn")->hide();
    toolBar("actionDialogs")->hide();
  }
  else
  {
    toolBar("gameGeneral")->hide();
    toolBar("gameServer")->hide();
    toolBar("actionTurn")->hide();
    toolBar("actionDialogs")->hide();
    if (_client_state >= CLIENT_GAME_RUNNING_STATE) {
      toolBar("actionDialogs")->show();
      toolBar("actionTurn")->show();
    }
    toolBar("gameServer")->show();
    toolBar("gameGeneral")->show();
  }		

  slotStatusMsg(i18n("Ready."));
}

void KapitalistApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}

void KapitalistApp::slotSettingsConfigure()
{
  if (!_configDlg)
    _configDlg = new ConfigDialog(this, this);

  _configDlg->show();
}



void KapitalistApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}

void KapitalistApp::slotNetInput()
{
  int sock = _conn.sock;
//    get_net_input(&sock);
  if(read_socket_data(sock, &_conn.buffer)>0) {
    int type;
    void *packet;

    while((packet= get_packet_from_connection(&_conn, &type))) {

      // handle chat and error messages immediately
      if ( (type == PACKET_CHAT_MSG) ||
	   (type == PACKET_JOIN_GAME_REPLY) ) {
	handle_packet_input((char *)packet, type);
      } else {  // put all other packets into the queue
	queue_packet_input(packet, type);
      }
    }
  }
  else {
    freelog(LOG_NORMAL, "lost connection to server");
    disconnect_from_server();
    _view->appendChatMsg("Lost connection to server!");
  }

}


void KapitalistApp::slotRollDice(int dicesum)
{
  struct packet_generic_integer pack;

  if (dicesum >= 2 && dicesum <= 12) {
    pack.value = dicesum;
  } else {
    pack.value = -1;
  }

  send_packet_generic_integer(_view->getApp()->conn(),
			      PACKET_DICE_ROLL_REQUEST, &pack);

}

void KapitalistApp::slotDone()
{
  struct packet_generic_integer pack;

  pack.value = _view->getGame()->myPID();

  send_packet_generic_integer(_view->getApp()->conn(),
			      PACKET_TURN_DONE, &pack);

}

void KapitalistApp::slotPayDebts()
{
  struct packet_generic_integer pack;

  pack.value = _view->getGame()->myPID();

  send_packet_generic_integer(_view->getApp()->conn(),
			      PACKET_PAY_REQUEST, &pack);

}

void KapitalistApp::slotBankruptcy()
{
  struct packet_generic_integer pack;

  pack.value = _view->getGame()->myPID();

  send_packet_generic_integer(_view->getApp()->conn(),
			      PACKET_DECLARE_BANKRUPT, &pack);
}


void KapitalistApp::slotPropertyBuy(int property)
{
  struct packet_generic_integer pack;

  if (property != -1) {
    pack.value = _view->getGame()->myPlr()->location();
  } else { // cheating
    pack.value = property;
  }

  send_packet_generic_integer(_view->getApp()->conn(),
			      PACKET_PROPERTY_BUY, &pack);

}


void KapitalistApp::slotPropertyMortgage(int property)
{
  struct packet_generic_integer pack;

  pack.value = property;

  send_packet_generic_integer(_view->getApp()->conn(),
			      PACKET_PROPERTY_BUY, &pack);

}
void KapitalistApp::slotPropertyUnmortgage(int property)
{
  struct packet_generic_integer pack;

  pack.value = property;

  send_packet_generic_integer(_view->getApp()->conn(),
			      PACKET_PROPERTY_BUY, &pack);
}

//  void KapitalistApp::slotHousesBuy(int monopid,
//  				  int nhouses[MAX_STREETS_IN_MONOP])
//  {
//    int i;
//    struct packet_house_request pack;

//    pack.monopid = monopid;
//    for (i=0; i<MAX_STREETS_IN_MONOP; ++i)
//      pack.nhouses[i] = nhouses[i];

//    send_packet_house_request(_view->getApp()->conn(),
//  			    PACKET_HOUSES_BUY, &pack);
//  }

//  void KapitalistApp::slotHousesSell(int monopid,
//  				   int nhouses[MAX_STREETS_IN_MONOP])
//  {
//    int i;
//    struct packet_house_request pack;
  
//    pack.monopid = monopid;
//    for (i=0; i<MAX_STREETS_IN_MONOP; ++i)
//      pack.nhouses[i] = nhouses[i];
  
//    send_packet_house_request(_view->getApp()->conn(),
//  			    PACKET_HOUSES_SELL, &pack);
//  }

void KapitalistApp::slotJailPay()
{
  struct packet_generic_integer pack;

  pack.value = E_LEAVE_JAIL_PAY_FINE;

  send_packet_generic_integer(_view->getApp()->conn(),
			      PACKET_JAIL_FREE_REQUEST, &pack);

}

void KapitalistApp::slotJailCard()
{
  struct packet_generic_integer pack;

  pack.value = E_LEAVE_JAIL_USE_CARD;

  send_packet_generic_integer(_view->getApp()->conn(),
			      PACKET_JAIL_FREE_REQUEST, &pack);

}

//  void KapitalistApp::updateHousesDialog(struct packet_monop_info *pack) const
//  {
//    int monopid = pack->monopid;
//    assert(monopid>=0 && monopid < BOARD_N_MONOPS);
//    if (_housesDlg[monopid]) _housesDlg[monopid]->update_status();
//  }

