// -*- C++ -*-
/***************************************************************************
                          kapview.h  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KAPVIEW_H
#define KAPVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

//
// Qt
//

#include <qcolor.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include <qtimer.h>
#include <qlabel.h>

class QLayout;
class QBoxLayout;
class QVBox;
class QGridLayout;

class QTabWidget;

class QCanvas;
class QCanvasPixmapArray;

#include <kglobal.h>
#include <kiconloader.h>

//
// App
//

#include "kapitalist.h"

#include "global.h"
#include "packets.h"

//
// Game
//

#include "board.h"

class KapGame;

//
// View
//

class BoardView;
class CenterView;
class LeftView;

class InputLine;
class UpDownFrame;
class OutputWindow;

class PlayerView;

class GameInfo;
class MyPlrInfo;
class MyHoldingsInfo;
class PlayerInfo;

class ThemeFieldEntry;
class ThemeMoneyEntry;
class ThemeDiceAnimEntry;
class ThemeData;


enum game_messages {
  MSG_INFO = 0,
  MSG_COMMAND,
  MSG_QUERY,
  MSG_LAST
};

enum game_buttons {
  BUT_NO = 0,
  BUT_BUY,
  BUT_LAST,
  BUT_UNSET_ALL
};

/***********************************/
/*  Locales                        */
/***********************************/

struct monop_locale_struct {
  char color[MONOP_MAX_LENGTH_COLOR];
  char color_short;
};

struct field_locale_struct {
  char name[FIELD_MAX_LENGTH_NAME];
};

struct card_locale_struct {
  char msg[MSG_SIZE];
};


/**
 * The KapView class provides the view widget for the KapitalistApp
 * instance.  The View instance inherits QWidget as a base class and
 * represents the view object of a KTMainWindow. As KapView is
 * part of the game-view model, it needs a reference to the game
 * object connected with it by the KapitalistApp class to manipulate and
 * display the game structure provided by the KapGame class.
 * 	
 * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
 * @version KDevelop version 0.4 code generation */
class KapView : public QWidget
{
  Q_OBJECT
public:
  /**
   * Constructor for the main view 
   */
  KapView(QWidget *parent = 0, const char *name=0);
  /**
   * Destructor for the main view
   */
  ~KapView();

  /**
   * TODO
   */
  KapitalistApp *getApp() const;

  /**
   * returns a pointer to the game connected to the view
   * instance. Mind that this method requires a KapitalistApp instance
   * as a parent widget to get to the window game pointer by calling
   * the KapitalistApp::getGame() method.
   *
   * @see KapitalistApp#getGame
   */
  KapGame * const getGame() const { return getApp()->game(); }

  PlayerView * const myPlrView() const;

  PlayerView * const plr (int id) const;

  PlayerView * const currPlr() const;

  QColor centerColor() const { return _centerColor; }

  QColor leftColor() const { return _leftColor; }
  
//    gGame * const game() const { return _game; }

  BoardView * const board() const { return _board; }

  QPixmap getPixmap(const QString &name) const;
  QPixmap getPixmap(const int fieldID) const;
  QPixmap getDiePixmap(const int i, const int j) const;
  QBitmap getBitmap(const QString &name) const;
  QCanvasPixmapArray *dice() const;

  void add_left(QWidget *, int stretch = 0);
  void add_center(QWidget *, int stretch = 0);
  void add_center(QLayout *, int stretch = 0);

  void createViews();
  void createEmptyViews();
//    void closeViews();

  void changeTheme(const QString& namesFname=QString::null,
		   const QString& boardFname=QString::null,
		   const QString& diceFname=QString::null);

  void appendChatMsg(const char *msg) const;
  void appendGameMsg(const char *msg) const;
  void appendCurrOut(const char *msg) const;

  QString replace_locales(const QString &text) const;

  /*
   * packet handler (GUI side)
   */

  void handle_your_turn(struct packet_generic_integer *packet);
  void handle_dice_rolled(struct packet_dice_rolled *packet);
  
  void handle_card_drawn(struct packet_generic_values *);
  
  void handle_trade_info(struct packet_trade *packet);
  
  void handle_field_info(struct packet_field_info *packet);
  void handle_monop_info(struct packet_monop_info *packet);
  
  void handle_game_init_info(struct packet_game_init_info *packet);
  void handle_game_info(struct packet_game_info *packet);
  
  void handle_player_info(struct packet_player_info *packet);

  struct monop_locale_struct monop_locale(int monopID);

//    void locales_eliminate_hyphens();
  const ThemeFieldEntry *field_locale(int fieldID) const;
//    ThemeFieldEntry *field_locale_raw(int fieldID);

  const ThemeMoneyEntry *money_locale() const;

  QString money_locale(const int money) const;

  const ThemeDiceAnimEntry *diceAnim() const;

  struct card_locale_struct cchest_card_locale(int cchestID);

  struct card_locale_struct chance_card_locale(int chanceID);
  
public slots:

  void slotIdleCycle();
  void slotFinishIdleCycle();
//      const QColor infoColor = QColor(240, 240, 160);

signals:

  void idleCycleFinished();

protected:


  // WARNING: do not rely on the existence of the frames, layouts
  // etc. even if they have nonzero pointer values !!
  // only explicitly never destroyed objects should be dereferenced in
  // methods (except the are created there)

  // TODO: make this a little more failsafe

  // top layout
  QGridLayout *_top;

  // layout for left side of main view
  LeftView *_leftFrm;
  QBoxLayout *_left;
  QColor _leftColor;

  // right hand side: board frame
  BoardView *_board;

  // center of the board (contains game info + output window + inputline)
  CenterView *_centerFrm;
  QBoxLayout *_center;
  QColor _centerColor;
  QLabel *_logoLab;

  // layouts in center
//    QBoxLayout *_gameInfos;
//    QGridLayout *_centerTop;
  QBoxLayout *_centerTop;
  QGridLayout *_centerMid;
  QBoxLayout *_centerBot;

  QTabWidget *_textIOTab;

  // central canvas (viewed by game info) + elements on it
  QCanvas *_centerCanv;
  GameInfo *_gameinfo;

  // widgets in center
  QFrame *_chatIOFrm;
  QBoxLayout *_chatIO;
  UpDownFrame *_chatHistory;
  InputLine *_chatIn;
  OutputWindow *_chatOut;

  QFrame *_gameIOFrm;
  QBoxLayout *_gameIO;
  UpDownFrame *_gameCmdHistory;
  InputLine *_gameCmdIn;
  OutputWindow *_gameOut;


  // player tokens
  PlayerView *_plr[GAME_MAX_PLAYERS];

  // infos on the left
  MyPlrInfo *_myinfo;
  MyHoldingsInfo *_myhold;
  PlayerInfo *_plrinfo;

  QTimer _idleTimer;
  bool _finishIdleCycle;

  // Theme related stuff
  ThemeData *_theme;

};

#endif // KAPVIEW_H

