// -*- C++ -*-

/***************************************************************************
                          boardview.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// System
//

#include <assert.h>

//
// Qt
//

#include <qlayout.h>

//
// Game
//

#include "kapgame.h"
#include "board.h"
#include "field.h"
#include "property.h"
#include "street.h"

//
// View
//

#include "kapview.h"
#include "boardview.h"
#include "fieldview.h"
#include "propertyview.h"
#include "streetview.h"
#include "jailfieldview.h"



BoardView::BoardView(KapView *view, QWidget *parent)
  : QFrame(parent), 
    _view(view), 
    _top(0),
    _center(0)
{
  int i;
  
//    setFrameStyle( QFrame::Panel | QFrame::Raised);
//    setLineWidth(4);
//    setMidLineWidth(3);
//    setMargin(4);

  for (i=0; i<BOARD_N_FIELDS; ++i)
    _field[i] = 0;

  _top = new QGridLayout(this, 12,12);
//    _center = new QVBoxLayout();

  _top->setColStretch(0, 5);
  _top->setColStretch(10, 5);
  _top->setRowStretch(0, 5);
  _top->setRowStretch(10, 5);
  _top->setRowStretch(11, 0);
  _top->addRowSpacing(11, 0);
  for (i=1; i<10; ++i) {
    _top->setColStretch(i, 2);
    _top->setRowStretch(i, 2);
  }
//    _top->setRowStretch(3,5);

  add_empty_fields();

  _fieldFont = font();
  _fieldFont.setPointSize(8);

  _top->activate();

  show();
}

BoardView::~BoardView()
{
  // all widgets have this as parent, deletion not necessary
//    int i;

//    for (i=0; i<BOARD_N_FIELDS; ++i) {
//      if (_field[i]) delete _field[i];
//    }
}


void BoardView::add_empty_fields()
{
  int i;

  for (i=0; i<BOARD_N_FIELDS; ++i) {
    if (_field[i]) delete _field[i];

    _field[i] = new EmptyFieldView(_view, this, (int) get_orientation(i));
    _top->addWidget(_field[i], getX(i), getY(i));
    _field[i]->show();
  }

}


void BoardView::init_board(Board */*board*/)
{

  int i;

  hide();

  if (_top) delete _top; _top = 0;
  
  _top = new QGridLayout(this, 12, 12);

  _top->setColStretch(0, 5);
  _top->setColStretch(10, 5);
  _top->setRowStretch(0, 5);
  _top->setRowStretch(10, 5);
  _top->setRowStretch(11, 0);
  _top->addRowSpacing(11, 0);
  _top->setColStretch(11, 0);
  _top->addColSpacing(11, 0);
  for (i=1; i<10; ++i) {
    _top->setColStretch(i, 3);
    _top->setRowStretch(i, 3);
  }

  for (i=0; i<BOARD_N_FIELDS; ++i) {

    if (_field[i]) delete _field[i]; _field[i] = 0;

    // find out, what type of field the new one should be
    Field *f = _view->getGame()->board()->field(i);
    Property *p = dynamic_cast<Property *>(f);
    Street *s = dynamic_cast<Street *>(f);
    
    if (s) {	// its a street
      _field[i] = new StreetView(_view, this, (int) get_orientation(i), s);
    } else if (p) {	// its another property
      _field[i] = new PropertyView(_view, this, (int) get_orientation(i), p);
    } else if (i == JUST_VISITING) {
      _field[i] = new JailFieldView(_view, this, (int) get_orientation(i), f);
    } else {	// its a normal field
      _field[i] = new FieldView(_view, this, (int) get_orientation(i), f);
    }      
    _field[i]->update_status();
    _field[i]->activate_layouts();
    _field[i]->show();
    _top->addWidget(_field[i], getX(i), getY(i));
  }

  _top->addMultiCellWidget(_center, 1,9,1,9);

  _top->activate();

  // init the player position in all fields
  // note: this can only be done after _top->activate(), because it uses
  // width() and height() of the fields
  for (i=0; i<BOARD_N_FIELDS; ++i)
    _field[i]->init_plr_positions();

  show();
}

void BoardView::delete_board()
{
  int i;

  hide();

  if (_top) delete _top;

  _top = new QGridLayout(this, 12,12);
//    _center = new QVBoxLayout();

  _top->setColStretch(0, 5);
  _top->setColStretch(10, 5);
  _top->setRowStretch(0, 5);
  _top->setRowStretch(10, 5);
  _top->setRowStretch(11, 0);
  _top->addRowSpacing(11, 0);
  for (i=1; i<10; ++i) {
    _top->setColStretch(i, 2);
    _top->setRowStretch(i, 2);
  }

  add_empty_fields();

  _top->activate();

  show();

}

void BoardView::add_center(QWidget *wdg)
{
  _center = wdg;
  _top->addMultiCellWidget(wdg, 1,9,1,9);
}


enum BoardView::orientation BoardView::get_orientation(const int loc)
{
  enum orientation o;

  switch (loc) {
  case 0: 
    return BOTRIGHT;
  case 10:
    return BOTLEFT;
  case 20:
    return TOPLEFT;
  case 30:
    return TOPRIGHT;
    
  default:	// other cases -> no square fields
    {
      switch (loc / 10) {
      case 0: return BOTTOM;
      case 1: return LEFT;
      case 2: return TOP;
      case 3: return RIGHT;
      default:
	assert(0);  // BUG!!
      }
    }
  }

  return o;

}

int BoardView::getX(int loc)
{
  switch (loc) {
  case 0: case 10:
    return 10;
  case 20: case 30:
    return 0;

  default:	// other cases -> no square fields
    
    switch (loc / 10) {
    case 0:	// BOTTOM
      return 10;
    case 1:	// LEFT
      return 20-loc;
    case 2:	// TOP
      return 0;
    case 3:	// RIGHT
      return loc-30;
      
    default:
      assert(0); // impossible case -> bug
    }
    
  }      
}

int BoardView::getY(int loc)
{
    switch (loc) {
    case 0: case 10:
      return 10-loc;
    case 20: case 30:
      return loc-20;

    default:	// other cases -> no square fields

      switch (loc / 10) {
      case 0:	// BOTTOM
	return 10-loc;
      case 1:	// LEFT
	return 0;
      case 2:	// TOP
	return loc-20;
      case 3:	// RIGHT
	return 10;

      default:
	assert(0); // impossible case -> bug
      }
      
    }      
}

void BoardView::rereadThemeData()
{
  int i;
  // let all the fields reread the theme data
  for (i=0; i<BOARD_N_FIELDS; ++i)
    _field[i]->rereadThemeData();
  
}

void BoardView::resizeEvent( QResizeEvent * ) {

//   std::cout << "resize board: " << height() << width() << std::endl;

  // take the smaller extent as a criterion to scale fonts
  int min_extent = (height()<width()) ? height() : width();

  if (min_extent < 600) {
    _fieldFont.setPointSize(4);
  } else if (min_extent < 700) {
    _fieldFont.setPointSize(6);
  } else if (min_extent < 800) {
    _fieldFont.setPointSize(8);
  } else {
    _fieldFont.setPointSize(10);
  }

  for (int i=0; i<BOARD_N_FIELDS; ++i)
    _field[i]->resize_elements();

}
