// -*- C++ -*-

/***************************************************************************
                          boardview.h  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BOARDVIEW_H
#define BOARDVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

//
// System
//

#include <assert.h>

//
// Qt
//

#include <qframe.h>
#include <qsize.h>

class QBoxLayout;
class QGridLayout;

//
// App
//

#include "global.h"
#include "board.h"

//
// View
//

class KapView;
class FieldView;


/**
 * TODO
 */
class BoardView : public QFrame {
  Q_OBJECT
  
public:
  enum orientation 
    { TOP, BOTTOM, LEFT, RIGHT, TOPLEFT, TOPRIGHT, BOTLEFT, BOTRIGHT };

public:

  BoardView(KapView *view, QWidget *parent);
  ~BoardView();

  FieldView * const field(const int loc) const
    { 
      assert (loc>=0 && loc <= LAST_LOC);
      if (loc == JAIL) return _field[JUST_VISITING];
      else return _field[loc]; 
    }
  const QFont *fieldFont() const 
    { return &_fieldFont; }

  void add_empty_fields();

  void init_board(Board *);
  void delete_board();

  void add_center(QWidget *);

  QWidget *center()
    { return _center; }

  enum orientation get_orientation(const int loc);

  int getX(int location);
  int getY(int location);

  void rereadThemeData();

  virtual void resizeEvent( QResizeEvent *);

private:

  KapView *_view;

  QGridLayout *_top;
  QWidget *_center;

  FieldView *_field[BOARD_N_FIELDS];

  QFont _fieldFont;
};

#endif // BOARDVIEW_H
