// -*- C++ -*-

/***************************************************************************
                          dieview.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// System
//

#include <cstdlib>
#include <ctime>
#include <cmath>

//
// Qt
//

#include <qcanvas.h>
#include <qpoint.h>
#include <qimage.h>
#include <qptrlist.h>

//
// Game
//

//
// View
//

#include "kapview.h"
#include "dieview.h"
#include "gameinfo.h"
#include "themedata.h"

DieView::DieView(KapView *view, QCanvas *canvas, GameInfo *gameinfo)
  : QCanvasSprite(0, canvas), _view(view), _canvas(canvas), 
    _gameinfo(gameinfo), _value(1)
{
//   int i,j;

// //    QPixmap die = _view->getDiePixmap(0,0);
//   QList<QPixmap> list;
//   QList<QPoint> plist;
//   i=0;
//   QPixmap *die;
//   for (j=0; j<4; ++j) {
//     die = new QPixmap(_view->getDiePixmap(j,i));
//     list.append(die);
//     plist.append(new QPoint(die->width()/2,die->height()/2));
//   }

// //    list.append(&die);
// //    plist.append(new QPoint(0,0));

// //    setSequence( new QCanvasPixmapArray(list, plist) );
//   _nframes = 33;
//   _frameIndex[0] = 0;
//   _frameIndex[1] = 22;
//   _frameIndex[2] = 0; 
//   _frameIndex[3] = 31;
//   _frameIndex[4] = 11;
//   _frameIndex[5] = 20; 
//   _frameIndex[6] = 9;
//   // TODO: test existence of die files
//   setSequence( new QCanvasPixmapArray("die.%1.png", _nframes) );

  _nframes = _view->diceAnim()->nframes;
  _frameIndex[0] = 0; // not used
  for (int i=1; i<=6; ++i)
    _frameIndex[i] = _view->diceAnim()->frameindex[i];
  setSequence( _view->dice());
  setAnimated(true);
  
  _airResistance = 0.01;
  _bumpResistance = 0.03;
  
//    for(i=0; i<6; ++i) {
//      for (j=0; j<4; ++j) {
//        _die[i][j] = _view->getDiePixmap(i,j);
//      }
//    }

//    _mask = _view->getBitmap("die-mask");

  srand(time(NULL));

}

DieView::~DieView()
{
}

bool DieView::hit( const QPoint &p ) const
{
  // this returns false if a transparent pixel on the image is hit
  // , otherwise true.
  int ix = p.x() - int( x() - width()/2 );
  int iy = p.y() - int( y() - height()/2 );
  if ( !image()->convertToImage().valid( ix, iy ) )
    return false;
  QRgb pixel = image()->convertToImage().pixel( ix, iy );
  return qAlpha( pixel ) != 0;
}


void DieView::advance( int phase )
{
  static int frame = 0;
  static int changeframe = 100*rand();

  double xVSign = (xVelocity()>=0)? 1.0 : -1.0;
  double yVSign = (yVelocity()>=0)? 1.0 : -1.0;

  int cntframes = int(8.0 / (xVSign*xVelocity()));
  if (cntframes < 1) cntframes = 1;
  if (changeframe++ % cntframes == 0) frame++;

  if ( phase == 0 ) { // stuff to be done before we move
    // apply resistance to slow down the die
//      setXVelocity( xVelocity() - xVSign*_airResistance );
//      setYVelocity( yVelocity() - yVSign*_airResistance );

    setXVelocity( - sqrt((double)(x()-width()/2) / 10) );
    setYVelocity( - xVelocity() / 4.5 );
//      _airResistance += 0.001;

    // right border hit check
    if ( !canvas()->onCanvas( int(x()+width()/2+xVelocity()), 1 ) ) {
      _moveX = false;
      move( canvas()->width() - width()/2, y(), frame%_nframes);
      setXVelocity( ( xVelocity() - xVSign*_bumpResistance ) * -1 );
    }
    // left border hit check
    else if ( !canvas()->onCanvas( int(x()-width()/2+xVelocity()), 1 ) ) {
      _moveX = false;
      move( 0 + width()/2, y(), frame%_nframes);
      setXVelocity( (xVelocity() - xVSign*_bumpResistance) * -1 );
    }

    // lower border hit check
    if ( !canvas()->onCanvas( 1, int(y()+height()/2+yVelocity()) ) ) {
      _moveY = false;
      move( x(), canvas()->height() - height()/2, frame%_nframes);
      setYVelocity( (yVelocity() - yVSign*_bumpResistance) * -1 );
    }
    // upper border hit check
    else if ( !canvas()->onCanvas( 1, int(y()-height()/2+yVelocity()) ) ) {
      _moveY = false;
      move( x(), 0 + height()/2, frame%_nframes);
      setYVelocity( (yVelocity() - yVSign*_bumpResistance) * -1 );
    }
  } else {    // time to move
    if ( _moveX && _moveY )
      move( x() + xVelocity(), y() + yVelocity(), frame % _nframes);
    else if ( _moveX )
      move( x() + xVelocity(), y(), frame % _nframes);
    else if ( _moveY )
      move( x(), y() + yVelocity(), frame % _nframes);

    _moveX = _moveY = true;
  }

  if (xVSign*xVelocity() < 0.1) { 
    setAnimated(false);
    // reset resistances
    _airResistance = 0.01;
    // set the right visible value
    switch(_value) {
    case 1: setFrame(_frameIndex[1]); break;
    case 2: setFrame(_frameIndex[2]); break;
    case 3: setFrame(_frameIndex[3]); break;
    case 4: setFrame(_frameIndex[4]); break;
    case 5: setFrame(_frameIndex[5]); break;
    case 6: setFrame(_frameIndex[6]); break;
    }
    _gameinfo->die_stopped(this);
  }
  
}

