// -*- C++ -*-

/***************************************************************************
                          fieldview.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// System
//

#include <assert.h>
#include <stdlib.h>
#include <iostream>

//
// Qt
//

#include <qlayout.h>
#include <qframe.h>
#include <qregexp.h>
#include <qpixmap.h>

//
// Game
//

#include "board.h"
#include "field.h"

//
// View
//

#include "kapview.h"
#include "boardview.h"
#include "fieldview.h"
#include "playerview.h"
#include "themedata.h"


QColor FieldView::_baseColor = QColor(210, 255, 215);


FieldView::FieldView(KapView *view, QWidget *parent, const int orientation,
		     const Field *field, bool add_base_widgets) 
  : QFrame (parent),
    _view(view),
    _field(field),
    _orientation(orientation),
    _titleLab(0),
    _icon(0)
{
  int i;
  int loc = field->location();

  setFrameStyle (Box | Plain);
  setBackgroundColor (_baseColor);

  switch (orientation) {
  case BoardView::TOP: 
//      setMinimumSize(DEFAULT_WIDTH, DEFAULT_HEIGHT);
    _top = new QBoxLayout(this, QBoxLayout::Down, 1);
    break;
  case BoardView::BOTTOM:
//      setMinimumSize(DEFAULT_WIDTH, DEFAULT_HEIGHT);
    _top = new QBoxLayout(this, QBoxLayout::Up, 1);
    break;
  case BoardView::LEFT:
//      setMinimumSize(DEFAULT_HEIGHT, DEFAULT_WIDTH);
    _top = new QBoxLayout(this, QBoxLayout::LeftToRight, 1);
    break;
  case BoardView::RIGHT:
//      setMinimumSize(DEFAULT_HEIGHT, DEFAULT_WIDTH);
    _top = new QBoxLayout(this, QBoxLayout::RightToLeft, 1);
    break;
  case BoardView::TOPLEFT: case BoardView::TOPRIGHT:
//      setMinimumSize(DEFAULT_HEIGHT, DEFAULT_HEIGHT);
    _top = new QBoxLayout(this, QBoxLayout::Down, 1);
    break;
  case BoardView::BOTLEFT: case BoardView::BOTRIGHT:
//      setMinimumSize(DEFAULT_HEIGHT, DEFAULT_HEIGHT);
    _top = new QBoxLayout(this, QBoxLayout::Up, 1);
    break;
  default:
    assert(0);	// not possible -> bug
  }

  switch (loc) {
  case GO:
    set_icon("start", this);
    break;
  case FREE_PARKING:
    set_title(_view->field_locale(loc)->hyphname);
    set_icon("car", this);
    break;
  case GOTO_JAIL:
    set_title(_view->field_locale(loc)->hyphname);
    set_icon("police", this);
    break;
  case LUXURY_TAX:
    set_title(_view->field_locale(loc)->hyphname);
    set_icon("luxtax", this);
    break;
  case 17:
    // workaround (fields 17 and 33 (comchests) were too small)
    set_title(_view->field_locale(loc)->hyphname);
    _top->addStrut(10);
    break;
  default:
    set_title(_view->field_locale(loc)->hyphname);
  }

  switch (_field->type()) {
  case CHANCE:
    set_icon("chance", this);
    break;
  case COMCHEST:
    set_icon("comchest", this);
  default:
    break;
  }

//    eliminate_hyphens(_view->field_locale(loc)->name);

  if (add_base_widgets) add_all_widgets();

  for (i=0; i<GAME_MAX_PLAYERS; ++i)
    _plr[i] = 0;

}


/* protected !! */

FieldView::FieldView (KapView *view, QWidget *parent, const int orientation)
  : QFrame (parent),
    _view(view),
    _field(0),
    _orientation(orientation),
    _titleLab(0),
    _icon(0)
{
}

FieldView::~FieldView()
{
}


QSize FieldView::sizeHint() const
{ 
  switch (_orientation) {

  case BoardView::TOP: case BoardView::BOTTOM:
    return QSize(DEFAULT_WIDTH, DEFAULT_HEIGHT); 
  case BoardView::LEFT: case BoardView::RIGHT:
    return QSize(DEFAULT_HEIGHT, DEFAULT_WIDTH); 
  case BoardView::TOPLEFT: case BoardView::TOPRIGHT:
  case BoardView::BOTLEFT: case BoardView::BOTRIGHT:
    return QSize(DEFAULT_HEIGHT , DEFAULT_HEIGHT);

  default:
    assert(0);	// impossible case
  }
}

void FieldView::resizeEvent( QResizeEvent *)
{
  int i;
  if (_field) {
//      cout << "w = " << width() << " h = " << height() 
//  	 << " in " << location() <<endl;
    init_plr_positions();
    for (i=0; i<GAME_MAX_PLAYERS; ++i) {
      if (_plr[i])
	_plr[i]->move(mapToParent(QPoint(0,0)) + _plrPos[i]);   
    }
  }

  resize_elements();

}


void FieldView::set_title(const char *rawtitle)
{
  _rawTitle = rawtitle;
  _title = rawtitle;
  _fullTitle = rawtitle;
  _title.replace( QRegExp("-") , "-\n" );
  _title.replace( QRegExp(" ") , "\n" );
  // underscore for spaces that should not be hyphenated
  _title.replace( QRegExp("_"), " " );	
  _fullTitle.replace( QRegExp("-") , "" );
  _fullTitle.replace( QRegExp("_"), " " );	

  QFont titlefont = font();
  titlefont.setPointSize(10);

  _titleLab = new QLabel(this);
//    _titleLab->setFont(QFont ("helvet", 10));
  _titleLab->setFont(titlefont);
  _titleLab->setAlignment(AlignCenter);
  _titleLab->setText(_title);
  _titleLab->setBackgroundColor(_baseColor);
//    _titleLab->setMinimumSize( _titleLab->sizeHint() );
}


void FieldView::set_icon(const char */*filename*/, QWidget *use_parent)
{
//    QPixmap p = _view->getPixmap(filename);
  assert(_field);
  QPixmap p = _view->getPixmap(_field->location());
  _icon = new QLabel(use_parent);
  _icon->setAlignment(AlignCenter);
  _icon->setBackgroundColor(_baseColor);
  _icon->setPixmap(p);
}

void FieldView::set_icon(QWidget *use_parent, const int fieldID)
{
  int fid = fieldID;

  if (fieldID < -1 || fieldID > LAST_LOC-1) fid = -1;
  if (fieldID == -1) { // take location of field as default
    assert(_field);
    fid = _field->location();
  } 

  QPixmap p = _view->getPixmap(fid);
  _icon = new QLabel(use_parent);
  _icon->setAlignment(AlignCenter);
  _icon->setBackgroundColor(_baseColor);
  _icon->setPixmap(p);
}

void FieldView::change_title(const QString &rawtitle)
{
  if (!_titleLab) return; // do nothing, if there is no title
  _rawTitle = rawtitle;
  _title = rawtitle;
  _fullTitle = rawtitle;
  _title.replace( QRegExp("-") , "-\n" );
  _title.replace( QRegExp(" ") , "\n" );
  // underscore for spaces that should not be hyphenated
  _title.replace( QRegExp("_"), " " );	
  _fullTitle.replace( QRegExp("-") , "" );
  _fullTitle.replace( QRegExp("_"), " " );

  _titleLab->setText(_title);
}

void FieldView::change_icon(const int fieldID)
{
  if (!_icon) return; // do nothing, if there is no icon label

  int fid = fieldID;
  if (fieldID < -1 || fieldID > LAST_LOC-1) fid = -1;
  if (fieldID == -1) { // take location of field as default
    assert(_field);
    fid = _field->location();
  } 

  QPixmap p = _view->getPixmap(fid);
  _icon->setPixmap(p);
  _icon->show();
}

void FieldView::add_all_widgets()
{
  int iconstretch, titlestretch;

  switch (_field->location()) {
  case GOTO_JAIL:
    iconstretch = 5;
    titlestretch = 3;
    break;
  case FREE_PARKING:
    iconstretch = 7;
    titlestretch = 3;
    break;
  default:
    iconstretch = 5;
    titlestretch = 7;
  }

  if ( _field->type() == CHANCE && 
       ( _orientation == BoardView::TOP ||
	 _orientation == BoardView::BOTTOM ) ) {
    // question mark is a little higher
    iconstretch = 7;
    titlestretch = 5;
  }

  if (_titleLab) _top->addWidget(_titleLab, titlestretch);
  if (_icon) _top->addWidget(_icon, iconstretch);
      
}

void FieldView::activate_layouts()
{
  _top->activate();
}


void FieldView::init_plr_positions()
{
  assert(GAME_MAX_PLAYERS >= 4);
  assert(GAME_MAX_PLAYERS <= 8);

  //	0	4	2
  //	7		6
  //	3	5	1

  int w = width();
  int h = height();
  int bx = w/10; 
  int by = h/10;


  int xl, xr, yt, yb;

  //  xl = x_left, xr = x_right
  //  yt = y_top, yb = y_bottom

  xl = bx;
  yt = by;
  xr = w-bx-20;
  yb = h-by-20;

  switch (_orientation) {
  case BoardView::TOP: 
    h = (int) (0.66 * h);
    yb = h-by-20;
    break;
  case BoardView::BOTTOM: 
    h = (int) (0.66 * h);
    yt = by + (height()-h);
    break;
  case BoardView::LEFT: 
    w = (int) (0.66 * w);
    xr = w-bx-20;
    break;
  case BoardView::RIGHT: 
    w = (int) (0.66 * w);
    xl = bx + (width() - w);
    break;
  }
  


//    cout << "init_plr_positions() in " << location() << endl;
  
  _plrPos[0] = QPoint(xl,yt);
  _plrPos[1] = QPoint(xr, yb);
  _plrPos[2] = QPoint(xr, yt);
  _plrPos[3] = QPoint(xl, yb);

  if (GAME_MAX_PLAYERS < 5) return;
  _plrPos[4] = _plrPos[0] + 0.5 * (_plrPos[2] - _plrPos[0]);
  if (GAME_MAX_PLAYERS < 6) return;
  _plrPos[5] = _plrPos[3] + 0.5 * (_plrPos[1] - _plrPos[3]);
  if (GAME_MAX_PLAYERS < 7) return;
  _plrPos[6] = _plrPos[4] + 0.5 * (_plrPos[1] - _plrPos[4]);
  if (GAME_MAX_PLAYERS < 8) return;
  _plrPos[7] = _plrPos[0] + 0.5 * (_plrPos[5] - _plrPos[0]);

}


void FieldView::update_status()
{
  // there is nothing to do with a normal field
}

void FieldView::resize_elements()
{
  if (_titleLab) resize_title();
}

void FieldView::resize_title()
{
  int wtrigger, htrigger;

  QFont titlefont = _titleLab->font();

  titlefont.setPointSize(_view->board()->fieldFont()->pointSize());

  _titleLab->setFont(titlefont);
}


void FieldView::rereadThemeData()
{
  if (_field) {
    change_title(_view->field_locale(_field->location())->hyphname);
    change_icon(_field->location());
  }
}


/*
 * player movement
 */

void FieldView::enter(PlayerView *plr)
{
  int i;
//    FieldView *f;

  // this doesn't work because plr->parent() is now board
  // so call "leave" manually everywhere enter is called 
  // (i.e. in PlayerView::enter() )
//    if (plr->old_loc() != )
//    if ((f = dynamic_cast<FieldView *>(plr->parent()))) { // leave old field
//      f->leave(plr);
//    }

  // find appropriate position for this player (i.e. next best free position)
  for (i=0; i<GAME_MAX_PLAYERS; ++i) {
    if (_plr[i] == 0) break;	// found empty position
  }

  assert(i<GAME_MAX_PLAYERS);	// found a free position

  // register it, put it at appropriate position and show it
  _plr[i] = plr;
//    plr->recreate( this, 0, _plrPos[i], TRUE );
  plr->move(mapToParent(QPoint(0,0)) + _plrPos[i]);   
  plr->show();
}

void FieldView::leave(PlayerView *plr)
{
  int i;
  // find my position in player array
  for (i=0; i<GAME_MAX_PLAYERS; ++i) {
    if (plr == _plr[i]) break;
  }

  // was player on that field ? (TODO: no assertion, but warning ?)

  assert(i<GAME_MAX_PLAYERS);

  // player token disappears
  plr->hide();
//    plr->recreate(parentWidget(), 0, QPoint(0,0));
  //clear player reference in field
  _plr[i] = 0;

}




/*
 * empty Field Widget, contains nothing
 */

EmptyFieldView::EmptyFieldView(KapView *view, QWidget *parent,
			       const int orientation)
  : FieldView(view, parent, orientation)
{
  setFrameStyle (Box | Plain);
  setBackgroundColor (_baseColor);

  switch (orientation) {
  case BoardView::TOP: 
    _top = new QBoxLayout(this, QBoxLayout::Down, 1);
    break;
  case BoardView::BOTTOM:
    _top = new QBoxLayout(this, QBoxLayout::Up, 1);
    break;
  case BoardView::LEFT:
    _top = new QBoxLayout(this, QBoxLayout::LeftToRight, 1);
    break;
  case BoardView::RIGHT:
    _top = new QBoxLayout(this, QBoxLayout::RightToLeft, 1);
    break;
  case BoardView::TOPLEFT: case BoardView::TOPRIGHT:
    _top = new QBoxLayout(this, QBoxLayout::Down, 1);
    break;
  case BoardView::BOTLEFT: case BoardView::BOTRIGHT:
    _top = new QBoxLayout(this, QBoxLayout::Up, 1);
    break;
  default:
    assert(0);	// not possible -> bug
  }

  QFrame *colFrm;
  QColor randCol;
  int r,g,b;


  // generate a random color
  r = 100 + (int) (155.0 * rand() / (RAND_MAX + 1.0));
  g = 100 + (int) (155.0 * rand() / (RAND_MAX + 1.0));
  b = 100 + (int) (155.0 * rand() / (RAND_MAX + 1.0));

  randCol = QColor(r,g,b);

  switch(orientation) {
  case BoardView::TOP:
  case BoardView::BOTTOM:
  case BoardView::LEFT:
  case BoardView::RIGHT:
    colFrm = new QFrame(this);
    colFrm->setBackgroundColor( randCol );
    _top->addStretch(3);
    _top->addWidget(colFrm, 2);
    break;
  default:
    // do nothing
    break;
  }

  _top->activate();
  setMinimumSize( sizeHint() );
}

EmptyFieldView::~EmptyFieldView()
{
}

QSize EmptyFieldView::sizeHint() const
{ 
  int w,h;
  
  w = 38;
  h = 60;
  switch (_orientation) {

  case BoardView::TOP: case BoardView::BOTTOM:
    return QSize(w,h); 
  case BoardView::LEFT: case BoardView::RIGHT:
    return QSize(h,w); 
  case BoardView::TOPLEFT: case BoardView::TOPRIGHT:
  case BoardView::BOTLEFT: case BoardView::BOTRIGHT:
    return QSize(h,h);

  default:
    assert(0);	// impossible case
  }

}

