// -*- C++ -*-

/***************************************************************************
                          fieldview.h  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FIELDVIEW_H
#define FIELDVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

//
// System
//

#include <assert.h>

//
// Qt
//

#include <qframe.h>
#include <qcolor.h>
#include <qstring.h>
#include <qpoint.h>

class QLabel;
class QBoxLayout;

class QPopupMenu;

//
// App
//

#include "global.h"

//
// Game
//

class Field;

//
// View
//

class KapView;
class PlayerView;


/**
 * TODO
 */
class FieldView : public QFrame {
  Q_OBJECT

public:
  enum { DEFAULT_WIDTH = 55, DEFAULT_HEIGHT = 80 };

public:
  static QColor baseColor()
    { return _baseColor; }

public:
  FieldView (KapView *view, QWidget *parent, int orientation,
	     const Field *field, bool add_base_widgets = true);
  
  virtual ~FieldView();

  virtual QSize sizeHint() const;
  virtual void resizeEvent( QResizeEvent *);

//    const int location() const
//      { 
//        assert(_field);
//        return _field->location(); 
//      }

  virtual const QString title() const { return _title; }
  virtual const QString fullTitle() const { return _fullTitle; }
  virtual const QString rawTitle() const { return _rawTitle; }

  virtual void set_title(const char *rawtitle);
  // TODO: set_icon(filename) deprecated, use set_icon(fieldID);
  virtual void set_icon(const char *filename, QWidget *use_parent);
  virtual void set_icon(QWidget *use_parent, const int fieldID=-1);

  virtual void change_title(const QString &rawtitle);
  virtual void change_icon(const int fieldID=-1);

  virtual void add_all_widgets();
  virtual void activate_layouts();

  virtual void init_plr_positions();

  virtual void update_status();

  virtual void resize_elements();
  virtual void resize_title();

  virtual void rereadThemeData();

  virtual void enter(PlayerView *);
  virtual void leave(PlayerView *);

protected:

  KapView *_view;

  const Field *_field;	// data base

  int _orientation;
  static QColor _baseColor;

  QString _title;
  QString _fullTitle;
  QString _rawTitle;

  QLabel *_titleLab;
  
  QLabel *_icon; // for all fields with icon

  PlayerView *_plr[GAME_MAX_PLAYERS];
  QPoint _plrPos[GAME_MAX_PLAYERS];

  QBoxLayout *_top;
  
protected:
  
  FieldView (KapView *view, QWidget *parent, const int orientation);

};    


class EmptyFieldView : public FieldView {
  Q_OBJECT

public:
  EmptyFieldView(KapView *view, QWidget *parent, int orientation);
  ~EmptyFieldView();

  const int location() const;	// undefined !

  virtual QSize sizeHint() const;

//      { return -1; }
  
};


#endif // FIELDVIEW_H



