// -*- C++ -*-
/***************************************************************************
                          housesdialog.cpp  -  description
                             -------------------
    begin                : Mon Jul  9 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// System

#include <assert.h>
#include <math.h>

//
// Qt
//

#include <kdialogbase.h>
#include <klocale.h>
#include <qlayout.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qslider.h>

//
// App
//

#include "kapitalist.h"
#include "global.h"

#include "housesdialog.h"

//
// Game
//

#include "kapgame.h"
#include "monop.h"

//
// View
//

#include "kapview.h"
#include "themedata.h"


/**
 *
 */
//  UrlDlg::UrlDlg( QWidget *parent, const QString& caption,
//                   const QString& urltext)
//     : KDialogBase( parent, "urldialog", true, caption, Ok|Cancel, Ok, true )
//   {
//     QWidget *page = new QWidget( this ); 
//     setMainWidget(page);
//     QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );
 
//     QLabel *label = new QLabel( caption, page, "caption" );
//     topLayout->addWidget( label );
 
//     lineedit = new QLineEdit( urltext, page, "lineedit" );
//     lineedit->setMinimumWidth(fontMetrics().maxWidth()*20);
//     topLayout->addWidget( lineedit );

//     topLayout->addStretch(10); 
//   }


HousesDialog::HousesDialog(KapitalistApp *app, QWidget *parent, int monopid)
  : KDialogBase(parent, "housesdialog", false, "Houses", Apply|User1|Close, 
		Close, true, QString(i18n("&Reset")) ),
    _app(app)
{
  int i, row;

  // TODO: there are only 5 slots to connect with!
  assert(MAX_STREETS_IN_MONOP <= 5);

  QWidget *page = new QWidget( this ); 
  setMainWidget(page);

  // TODO: should be tested in Board::monop(int) !
  assert(monopid>=0 && monopid <=BOARD_N_MONOPS);

  _monop = app->game()->board()->monop(monopid);
  int nstreets = _monop->nstreets();
  for (i=0; i<MAX_STREETS_IN_MONOP; ++i) {
    _newhouses[i] = 0;
  }

  _pmHouse = _app->view()->getPixmap("house");
  _pmHotel = _app->view()->getPixmap("hotel");
  _pmHouseAdd = _app->view()->getPixmap("houseadd");
  _pmHotelAdd = _app->view()->getPixmap("hoteladd");
  _pmHouseDel = _app->view()->getPixmap("housedel");
  _pmHotelDel = _app->view()->getPixmap("hoteldel");

  _top = new QGridLayout( page, nstreets+2,4, spacingHint() );
  _top->addRowSpacing(nstreets+1, 50);

  _top->setColStretch(0, 3);
  _top->setColStretch(1, 2);
  _top->setColStretch(2, 2);
  _top->setColStretch(3, 2);
  
  for(row=0; row<nstreets; ++row) {
    _housesRow[row] = new QGridLayout(0, 1, STREET_MAX_HOUSES+2, 0, -1 );
    for (i=0; i<STREET_MAX_HOUSES+1; ++i) {
      _housesRow[row]->addColSpacing(i, _pmHotel.width());
      _housesRow[row]->setColStretch(i, 0);
    }
    _housesRow[row]->setColStretch(STREET_MAX_HOUSES+1, 1);
    _top->addLayout(_housesRow[row], row+1,1);
    _top->addRowSpacing(row+1, _pmHotel.height() + 10);
  }

  QLabel *label 
    = new QLabel( i18n("You can buy/sell houses by adjusting the sliders\n"
		       "on the right side and apply the changes\n"
		       "with the Apply button at the bottom"),
		  page, "jdlab" );
  label->setAlignment(AlignCenter);
  label->setMinimumSize(label->sizeHint());
  _top->addMultiCellWidget( label, 0,0, 0,3 );

  for (i=0; i<nstreets; ++i) {
    _streetLab[i] = new QLabel(page);
    int streetloc = _monop->street(i)->location();
    _streetLab[i]->setText(_app->view()->field_locale(streetloc)->name);
    _top->addWidget(_streetLab[i], i+1,0);
    // Slider inc
    _nHousesSlider[i] = new QSlider(0,STREET_MAX_HOUSES+1, 1,
				    _monop->street(i)->nhouses(),
				    QSlider::Horizontal, page);
    _nHousesSlider[i]->setFixedSize(_nHousesSlider[i]->sizeHint());
    _top->addWidget(_nHousesSlider[i], i+1, 2, Qt::AlignRight);
  }
  
  for (row=0; row<nstreets; ++row) {
    for (i=0; i<STREET_MAX_HOUSES; ++i) {
      _house[row][i] = new QLabel(page);
      _house[row][i]->setBackgroundColor(backgroundColor());
      _house[row][i]->setAlignment(AlignCenter);
      _house[row][i]->setPixmap (_pmHouse);
      _house[row][i]->hide();
      _houseadd[row][i] = new QLabel(page);
      _houseadd[row][i]->setBackgroundColor(backgroundColor());
      _houseadd[row][i]->setAlignment(AlignCenter);
      _houseadd[row][i]->setPixmap (_pmHouseAdd);
      _houseadd[row][i]->hide();
      _housedel[row][i] = new QLabel(page);
      _housedel[row][i]->setBackgroundColor(backgroundColor());
      _housedel[row][i]->setAlignment(AlignCenter);
      _housedel[row][i]->setPixmap (_pmHouseDel);
      _housedel[row][i]->hide();
      _housesRow[row]->addWidget(_house[row][i], 0,i);
      _housesRow[row]->addWidget(_houseadd[row][i], 0,i);
      _housesRow[row]->addWidget(_housedel[row][i], 0,i);
    }

    _hotel[row] = new QLabel(page);
    _hotel[row]->setBackgroundColor (backgroundColor());
    _hotel[row]->setAlignment(AlignCenter);
    _hotel[row]->setPixmap(_pmHotel);
    _hotel[row]->hide();
    _house[row][STREET_MAX_HOUSES] = _hotel[row];
    _hoteladd[row] = new QLabel(page);
    _hoteladd[row]->setBackgroundColor (backgroundColor());
    _hoteladd[row]->setAlignment(AlignCenter);
    _hoteladd[row]->setPixmap(_pmHotelAdd);
    _hoteladd[row]->hide();
    _houseadd[row][STREET_MAX_HOUSES] = _hoteladd[row];
    _hoteldel[row] = new QLabel(page);
    _hoteldel[row]->setBackgroundColor (backgroundColor());
    _hoteldel[row]->setAlignment(AlignCenter);
    _hoteldel[row]->setPixmap(_pmHotelDel);
    _hoteldel[row]->hide();
    _housedel[row][STREET_MAX_HOUSES] = _hoteldel[row];
//      _housesRow[row]->addMultiCellWidget(_hotel[row], 0,0, 1,2);
    _housesRow[row]->addWidget(_hotel[row], 0, STREET_MAX_HOUSES);
    _housesRow[row]->addWidget(_hoteladd[row], 0, STREET_MAX_HOUSES);
    _housesRow[row]->addWidget(_hoteldel[row], 0, STREET_MAX_HOUSES);
    _housesRow[row]->activate();

    // money
    _moneyLab[row] = new QLabel(page);
    _moneyLab[row]->setText("- $12345678");
    _top->addWidget( _moneyLab[row], row+1, 3 );

  }

  _moneySumDesc = new QLabel(page);
  _top->addWidget( _moneySumDesc, row+1, 1, AlignLeft );
  _moneySumLab = new QLabel(page);
  _top->addWidget( _moneySumLab, row+1, 2, AlignCenter ); 

  _top->activate();

  // TODO: remove this hack!
  if (nstreets>=1)
    QObject::connect( _nHousesSlider[0], SIGNAL(valueChanged(int)), 
		      this, SLOT(slotNumHouses0(int)) );
  if (nstreets>=2)
    QObject::connect( _nHousesSlider[1], SIGNAL(valueChanged(int)), 
		      this, SLOT(slotNumHouses1(int)) );
  if (nstreets>=3)
    QObject::connect( _nHousesSlider[2], SIGNAL(valueChanged(int)), 
		      this, SLOT(slotNumHouses2(int)) );
  if (nstreets>=4) 
    QObject::connect( _nHousesSlider[3], SIGNAL(valueChanged(int)), 
		      this, SLOT(slotNumHouses3(int)) );
  if (nstreets>=5) 
    QObject::connect( _nHousesSlider[4], SIGNAL(valueChanged(int)), 
		      this, SLOT(slotNumHouses4(int)) );

  QObject::connect(this, SIGNAL(user1Clicked()), this, SLOT(slotReset()));


  for (i=0; i<_monop->nstreets(); ++i) {
    QObject::connect( _monop->street(i), SIGNAL(updated()),
		      this, SLOT(slotReset()) );
  }

  show();

  slotReset();

}

HousesDialog::~HousesDialog()
{
}


void HousesDialog::update_status(int only_row)
{
  assert(only_row < MAX_STREETS_IN_MONOP);

  int row;

  for (row=0; row<_monop->nstreets(); ++row) {
    if (only_row >= 0) row = only_row;

    _nHousesSlider[row]->setValue(_newhouses[row]);

    if (only_row >= 0) break;
  }
  // other updates (houseviews and money) are triggered by setValue of sliders

}

void HousesDialog::update_houseviews(int only_row)
{
  int i, row;
  int nstreets = _monop->nstreets();
  int nhouses, nhouses_old;
  // set houses/hotel appropriate
  for (row=0; row<nstreets; ++row) {
    if (only_row >=0) row = only_row;
    nhouses_old = _monop->street(row)->nhouses();
    nhouses = _newhouses[row]; 
    // hotel is included in _house array as last element!!
    if (nhouses >= nhouses_old) { // add some houses
      for (i=0; i<nhouses_old; ++i) { // show house
	_house[row][i]->show();
	_houseadd[row][i]->hide();
	_housedel[row][i]->hide();
      }
      for (i=nhouses_old; i<nhouses; ++i) { // show houseadd
	_house[row][i]->hide();
	_houseadd[row][i]->show();
	_housedel[row][i]->hide();
      }
      for (i=nhouses; i<STREET_MAX_HOUSES+1; ++i) { // hide all
	_house[row][i]->hide();
	_houseadd[row][i]->hide();
	_housedel[row][i]->hide();
      }	
    } else { // remove some houses
      for (i=0; i<nhouses; ++i) {
	_house[row][i]->show();
	_houseadd[row][i]->hide();
	_housedel[row][i]->hide();
      }
      for (i=nhouses; i<nhouses_old; ++i) {
	_house[row][i]->hide();
	_houseadd[row][i]->hide();
	_housedel[row][i]->show();
      }
      for (i=nhouses_old; i<STREET_MAX_HOUSES+1; ++i) { // hide all
	_house[row][i]->hide();
	_houseadd[row][i]->hide();
	_housedel[row][i]->hide();
      }	
    }
    if (only_row >=0) break;
  }
}

void HousesDialog::update_money(int only_row)
{
  int row;
  int nstreets = _monop->nstreets();

  int diffhouses, houseprice;
  QString moneyPrefix;

  for (row=0; row<nstreets; ++row) {
    if (only_row >=0) row = only_row;

    diffhouses = ( _newhouses[row] - _monop->street(row)->nhouses() );
    houseprice = _monop->street(row)->house_price();
    _money[row] = ( (diffhouses>0) ? diffhouses * houseprice  // buy houses
		    : diffhouses * houseprice / 2 ); // sell houses (halfprice)
    moneyPrefix = ( (_money[row]>=0) ? QString("- $") : QString("+ $") );
    _moneyLab[row]->setText( moneyPrefix
			     + QString().setNum(abs(_money[row])) );

    if (only_row >=0) break;
  }

  // total amount of money
  
  for (_moneySum = 0, row=0; row<nstreets; ++row)
    _moneySum += _money[row];

  moneyPrefix = "$";
  _moneySumDesc->setText( (_moneySum >= 0) ? QString("Total cost:") 
			  : QString("You get:") );
  _moneySumLab->setText( moneyPrefix + QString().setNum(abs(_moneySum)) );
  
}


void HousesDialog::slotApply()
{
  int i;
  struct packet_house_request pack;
  pack.monopid = _monop->id();
  for (i=0; i<_monop->nstreets(); ++i)
    pack.nhouses[i] = (_newhouses[i] - _monop->street(i)->nhouses());

  send_packet_house_request(_app->conn(), PACKET_HOUSES_BUY, &pack);
}

void HousesDialog::slotReset()
{
  int i;

  for (i=0; i<_monop->nstreets(); ++i) {
    _newhouses[i] = _monop->street(i)->nhouses();
  }

  update_status();
  // is not automatically done, because _newhouses==oldhouses
  update_houseviews();
  update_money();
}

void HousesDialog::slotClose()
{
  // TODO: reset to actual situation
  emit reject();
  delayedDestruct();
}

void HousesDialog::slotNumHouses0(int nhouses)
{
  slotNumHouses(0, nhouses);
}

void HousesDialog::slotNumHouses1(int nhouses)
{
  slotNumHouses(1, nhouses);
}

void HousesDialog::slotNumHouses2(int nhouses)
{
  slotNumHouses(2, nhouses);
}

void HousesDialog::slotNumHouses3(int nhouses)
{
  slotNumHouses(3, nhouses);
}

void HousesDialog::slotNumHouses4(int nhouses)
{
  slotNumHouses(4, nhouses);
}

void HousesDialog::slotNumHouses(int row, int nhouses)
{
  if (nhouses != _newhouses[row]) {
    _newhouses[row] = nhouses;
    update_houseviews(row);
    update_money(row);
  }
}
