// -*- C++ -*-

/***************************************************************************
                          jailfieldview.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// System
//

#include <assert.h>

//
// Qt
//

#include <qlayout.h>
#include <qframe.h>
#include <qregexp.h>

//
// Game
//

#include "board.h"
#include "field.h"

//
// View
//

#include "kapview.h"
#include "jailfieldview.h"
#include "boardview.h"
#include "fieldview.h"
#include "playerview.h"



JailFieldView::JailFieldView (KapView *view, QWidget *parent,
			      const int orientation, const Field *field) 
  : FieldView(view, parent, orientation, field, true),
    _jailLab(NULL)
{
  int i;
  int loc = field->location();

  // There is only *one* Jailfield: JUST_VISITING
  // This includes (as a QLabel) the field JAIL
  assert(loc == JUST_VISITING);

  // layout for jail
  _jail = new QHBoxLayout();

//    set_title(_view->field_locale(loc)->hyphname);
//    set_jail_title(_view->field_locale(JAIL)->hyphname);
  set_icon(this, JAIL);

//    eliminate_hyphens(_view->field_locale(loc)->hyphname);
//    eliminate_hyphens(_view->field_locale(JAIL)->hyphname);


  add_all_widgets();

  for (i=0; i<GAME_MAX_PLAYERS; ++i) {
    _plr[i] = NULL;
    _jailPlr[i] = NULL;
  }

}


JailFieldView::~JailFieldView()
{
}

// we must reimplement this, because of plr->move
void JailFieldView::resizeEvent( QResizeEvent * )
{
  int i;
  if (_field) {
//      cout << "w = " << width() << " h = " << height() 
//  	 << " in " << location() <<endl;
    init_plr_positions();
    for (i=0; i<GAME_MAX_PLAYERS; ++i) {
      if (_plr[i]) {	// there is a player at position i on JUST_VISITING
	assert(_plr[i]->location() == JUST_VISITING); 
	_plr[i]->move(mapToParent(QPoint(0,0)) + _plrPos[i]);   
      }
      if (_jailPlr[i]) {	// player at position i on JAIL
	assert(_jailPlr[i]->location() == JAIL);
	_jailPlr[i]->move(mapToParent(QPoint(0,0)) + _jailPlrPos[i]);   
      }
    }
  }
  resize_elements();
}

  // this is no longer needed, because we can underscore spaces to avoid
  // them to be converted into newlines
//  void JailFieldView::set_title(const char *title)
//  {
//    // _title was already generated and set in base class FieldView
//    // we only change attributes of the label

//    // undo replacement of spaces into newlines
//    _title.replace( QRegExp("\n") , " ");
//    _titleLab->setText(_title);
//  }

void JailFieldView::set_jail_title(const char *title)
{
  _jailTitle = title;
  _jailTitle.replace( QRegExp("-") , "-\n");
  _jailTitle.replace( QRegExp(" ") , "\n");
  
  _jailLab = new QLabel(this);
  _jailLab->setFont (QFont ("helvet", 10));
  _jailLab->setAlignment(AlignCenter);
  _jailLab->setFrameStyle( Panel | Raised );
  _jailLab->setText(_jailTitle);
  _jailLab->setBackgroundColor(_baseColor);
  _jailLab->show();
}

void JailFieldView::change_icon(const int /*fieldID*/)
{
  if (!_icon) return; // do nothing, if there is no icon label

  // ignore fieldID argument and set icon on jail field
  int fid = JAIL;

  QPixmap p = _view->getPixmap(fid);
  _icon->setPixmap(p);
}


void JailFieldView::add_all_widgets()
{
  _top->addLayout(_jail, 21);	// 21 = 3 * 7 (stretch of "JUST_VISITING")
  _jail->addStretch(1);
//    _jail->addWidget(_jailLab, 3);
  _jail->addWidget(_icon, 3, AlignCenter);
}

void JailFieldView::activate_layouts()
{
//    _jail->activate();
  _top->activate();
}


void JailFieldView::init_plr_positions()
{
  assert(GAME_MAX_PLAYERS >= 4);
  assert(GAME_MAX_PLAYERS <= 8);

  // we have 2 fields in one: JUST_VISITING and JAIL, so we must define
  // two player position arrays (_plrPos and _jailPlrPos)

  int w = width();
  int h = height();
  int bx = w/25; 
  int by = h/25;
  
  //  xl = x_left, xr = x_right
  //  yt = y_top, yb = y_bottom

  int xl, xr, yt, yb;

  xl = bx;
  yt = by;
  xr = w-bx-20;
  yb = h-by-20;

  // _plrPos: (for JUST_VISITING)
  //	2
  //	4
  //	0
  //	6
  //		7	1	5	3
  // -> define positions 2, 3 and 6, 7, other lie between them
  
  _plrPos[2] = QPoint(xl,yt);
  _plrPos[3] = QPoint(xr, yb);
  _plrPos[6] = QPoint(xl, yb-10);
  _plrPos[7] = QPoint(xl+10, yb);

  if (GAME_MAX_PLAYERS < 5) return;
  _plrPos[0] = _plrPos[2] + (_plrPos[6] - _plrPos[2]) * 2 / 3;
  if (GAME_MAX_PLAYERS < 6) return;
  _plrPos[1] = _plrPos[7] + (_plrPos[3] - _plrPos[7]) * 1 / 3;
  if (GAME_MAX_PLAYERS < 7) return;
  _plrPos[4] = _plrPos[2] + (_plrPos[6] - _plrPos[2]) * 1 / 3;
  if (GAME_MAX_PLAYERS < 8) return;
  _plrPos[5] = _plrPos[7] + (_plrPos[3] - _plrPos[7]) * 2 / 3;


  // _jailPlrPos: (for JAIL)
  //	0	4	2
  //	7		6
  //	3	5	1

  bx = w/15;
  by = h/15;
  int xoff = w/4;	// offset in x-direction
  int yoff = h/4;	// offset in y-direction
  xl = xoff + bx;
  yt = by;
  xr = w-bx-20;
  yb = h-yoff-by-20;

  _jailPlrPos[0] = QPoint(xl,yt);
  _jailPlrPos[1] = QPoint(xr, yb);
  _jailPlrPos[2] = QPoint(xr, yt);
  _jailPlrPos[3] = QPoint(xl, yb);

  if (GAME_MAX_PLAYERS < 5) return;
  _jailPlrPos[4] = _jailPlrPos[0] + 0.5 * (_jailPlrPos[2] - _jailPlrPos[0]);
  if (GAME_MAX_PLAYERS < 6) return;
  _jailPlrPos[5] = _jailPlrPos[3] + 0.5 * (_jailPlrPos[1] - _jailPlrPos[3]);
  if (GAME_MAX_PLAYERS < 7) return;
  _jailPlrPos[6] = _jailPlrPos[4] + 0.5 * (_jailPlrPos[1] - _jailPlrPos[4]);
  if (GAME_MAX_PLAYERS < 8) return;
  _jailPlrPos[7] = _jailPlrPos[0] + 0.5 * (_jailPlrPos[5] - _jailPlrPos[0]);


}


//  void JailFieldView::update_status()
//  {
//    // there is nothing to do here
//  }


/*
 * player movement
 */

// TODO: check plrloc==JAIL || plrloc==JUST_VISITING !
void JailFieldView::enter(PlayerView *plr)
{
  int i;

  // find appropriate position for this player (i.e. next best free position)
  if (plr->location() == JAIL) {

    for (i=0; i<GAME_MAX_PLAYERS; ++i)
      if (_jailPlr[i] == NULL) break; // found empty position in jail
    
    assert(i<GAME_MAX_PLAYERS);	// found a free position

    // register it and put it at appropriate position
    _jailPlr[i] = plr;
    plr->move(mapToParent(QPoint(0,0)) + _jailPlrPos[i]);   

  } else {	// plr is on JUST_VISITING

    // same procedure as in JAIL

    for (i=0; i<GAME_MAX_PLAYERS; ++i)
      if (_plr[i] == NULL) break; // found empty position on JUST_VISITING

    assert(i<GAME_MAX_PLAYERS);

    _plr[i] = plr;
    plr->move(mapToParent(QPoint(0,0)) + _plrPos[i]);   

  }

  plr->show();
}

void JailFieldView::leave(PlayerView *plr)
{
  int i;

  if (plr->old_loc() == JAIL) {

    // find my position in player array
    for (i=0; i<GAME_MAX_PLAYERS; ++i)
      if (plr == _jailPlr[i]) break;

    // was player on that field ? (TODO: no assertion, but warning ?)
    assert(i<GAME_MAX_PLAYERS);

    // clear player reference in field and hide token
    _jailPlr[i] = NULL;
    plr->hide();

  } else {	// plr is on JUST_VISITING

    // find my position in player array
    for (i=0; i<GAME_MAX_PLAYERS; ++i) {
      if (plr == _plr[i]) break;
    }

    // was player on that field ? (TODO: no assertion, but warning ?)
    assert(i<GAME_MAX_PLAYERS);

    // clear player reference in field and hide token
    _plr[i] = NULL;
    plr->hide();

  }

}

