// -*- C++ -*-

/***************************************************************************
                          myholdingsinfo.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// Qt
//

#include <qlayout.h>
#include <qframe.h>

//
// Game
//

#include "kapgame.h"
#include "player.h"
#include "monop.h"

//
// View
//

#include "kapview.h"
#include "playerview.h"
#include "monopview.h"
#include "myholdingsinfo.h"



MyHoldingsInfo::MyHoldingsInfo(KapView *view, QWidget *parent)
  : QFrame(parent), _view(view)
{
//    setAlignment(AlignLeft);
//    setTitle("Your Holdings");
  setFrameStyle( Box | Plain );
  setLineWidth(2);
//    setMargin(10);
//    setBackgroundColor(_view->leftColor().light(150));
  setBackgroundColor(Qt::white);


  _top = new QGridLayout( this, 3,2, 5);
  
  _top->setColStretch(0,1);
  _top->setColStretch(1,1);
  
  _topLab = new QLabel(this);
  QFont topLabFont = _topLab->font();
  topLabFont.setBold(true);
  _topLab->setFont(topLabFont);
  _topLab->setText("Your Holdings");
  _topLab->setBackgroundColor(backgroundColor());
  _topLab->show();

  _top->addMultiCellWidget(_topLab, 0,0, 0,1, Qt::AlignLeft);

  _streetFrm = new StreetInfoFrame(_view, this);
  _streetFrm->setBackgroundColor(backgroundColor());
  _top->addMultiCellWidget(_streetFrm, 1,1, 0,1, Qt::AlignCenter);

  _otherFrm = new OtherInfoFrame(_view, this);
  _otherFrm->setBackgroundColor(backgroundColor());
  _top->addWidget(_otherFrm, 2,0, Qt::AlignLeft);

  _jcardFrm = new JailCardInfoFrame(_view, this);
  _jcardFrm->setBackgroundColor(backgroundColor());
  _top->addWidget(_jcardFrm, 2,1, Qt::AlignRight);

  _top->activate();

  show();
}

MyHoldingsInfo::~MyHoldingsInfo()
{
}

void MyHoldingsInfo::init()
{
  _streetFrm->init_icons();
}

void MyHoldingsInfo::update() 
{
  // TODO: highlight the owned streets
  _streetFrm->update();
  _otherFrm->update();
  _jcardFrm->update();
}  






StreetInfoFrame::StreetInfoFrame(KapView *view, QWidget *parent)
  : QFrame(parent), _view(view)
{

  int i, j;

//    setBackgroundColor(mainWin->leftColor());
//    setFrameStyle( Panel | Raised );

  // initalize all icons with NULL
  for (i=0; i<BOARD_N_MONOPS; ++i)
    for (j=0; j<MAX_STREETS_IN_MONOP; ++j)
      _streetIcon[i][j] = NULL;

  // geometry is managed in init_icons

  show();
}
 
StreetInfoFrame::~StreetInfoFrame()
{
}

void StreetInfoFrame::resizeEvent( QResizeEvent * )
{
  // TODO: resize icons here
//    int i,j;
//    int x,y;
//    int dist = 5; // distance between two overlapping cards
  
//  //    Monop *monop = _view->getGame()->board()->monop(_monopid);

//    // a little border
//    x = 5; y = 5;
  
//    for (i=0; i<BOARD_N_MONOPS; ++i) {
//      for (j=0; j<MAX_STREETS_IN_MONOP; ++j) {
//        if (_streetIcon[i][j]) {
//  //  	_streetIcon[i][j]->move(mapToParent(QPoint(x,y)));
//  	_streetIcon[i][j]->show();
//        }
//        x += dist;
//        y += dist;
//      }
//      x += StreetIcon::DEFAULT_WIDTH + 10;
//      y = 5;
//    }

//    // total width and height of frame
//    int w_tot = width();
//    int h_tot = height();


//    // width and height of each card
//    int w = w_tot / BOARD_N_MONOPS / 2;
//    int h = h_tot - MAX_STREETS_IN_MONOP * dist;
  
}


void StreetInfoFrame::init_icons()
{
  int i,j;
  Monop *monop;

  int x,y;
  int dist = 5; // distance between two overlapping cards
  
  // a little border
  x = 4; y = 4;
  
  for (i=0; i<BOARD_N_MONOPS; ++i) {
    for (j=0; j<MAX_STREETS_IN_MONOP; ++j) {
      if (_streetIcon[i][j]) {
      }
    }
  }
  // create new icons
  for (i=0; i<BOARD_N_MONOPS; ++i) {
    monop = _view->getGame()->board()->monop(i);
    for (j=0; j<monop->nstreets(); ++j) {
      _streetIcon[i][j] = new StreetIcon(_view, this, i);
      _streetIcon[i][j]->move( QPoint(x,y) );
      _streetIcon[i][j]->set_highlighted(true);
      _streetIcon[i][j]->show();
      x += dist;
      y += dist;
    }
    x += StreetIcon::DEFAULT_WIDTH;
    y = 4;
  }

  adjustSize();
  setMinimumSize( size() );
}

void StreetInfoFrame::update()
{
  int i,j;
  Monop *monop;

  for (i=0; i<BOARD_N_MONOPS; ++i) {
    monop = _view->getGame()->board()->monop(i);
    for (j=0; j<monop->nstreets(); ++j) {
      assert(_streetIcon[i][j]);
      // if it's my street, highlight it, else normal color
      if (monop->street(j)->owner() == _view->getGame()->myPlr()) {
	_streetIcon[i][j]->set_highlighted(true);
	if (monop->street(j)->is_mortgaged())
	  _streetIcon[i][j]->set_mortgaged(true);
	else
	  _streetIcon[i][j]->set_mortgaged(false);
      } else	// street not mine
	_streetIcon[i][j]->set_highlighted(false);
    }
  }
}



const QColor StreetIcon::normal_color = gray;
const QColor StreetIcon::highlight_color = white;
const QColor StreetIcon::mortgaged_color = QColor(255,235,235);


StreetIcon::StreetIcon(KapView *view, QWidget *parent, int monopid)
  : QFrame(parent), _view(view),
    _monopid(monopid),
    _is_highlighted(false),
    _is_mortgaged(false)
{
  assert(monopid >= 0 && monopid < BOARD_N_MONOPS);
  _monop_color = monop_color[monopid];

  setFixedSize(DEFAULT_WIDTH, DEFAULT_HEIGHT);
//    setFrameStyle (Panel | Sunken);
  setFrameStyle( Box | Plain );
  setLineWidth(1);
  setBackgroundColor(highlight_color);

  _innerFrm = new QFrame(this);
  _innerFrm->setFixedSize(DEFAULT_WIDTH-2, DEFAULT_HEIGHT/3);
  _innerFrm->setFrameStyle( NoFrame );
//    _innerFrm->setLineWidth(1);
  _innerFrm->move(QPoint(1,1));
  _innerFrm->show();
  set_highlighted(false);

  // is showed when geometry is fixed
}


StreetIcon::~StreetIcon()
{
}

void StreetIcon::set_highlighted(bool b)
{
  if (b == _is_highlighted) return; // status didn't change

  _is_highlighted = b;

  if (b) {
    _innerFrm->setBackgroundColor(_monop_color);
    setBackgroundColor(highlight_color);
  }
  else {
    _innerFrm->setBackgroundColor(normal_color);
    setBackgroundColor(normal_color.light(130));
  }
}

void StreetIcon::set_mortgaged(bool b)
{

  if (b == _is_mortgaged) return; // status didn't change

  _is_mortgaged = b;

  if (b) {
    _innerFrm->setBackgroundColor(mortgaged_color);
    setBackgroundColor(mortgaged_color);
  }
  else {
    _innerFrm->setBackgroundColor(_monop_color);
    setBackgroundColor(highlight_color);
  }
}


/***************************************/

OtherInfoFrame::OtherInfoFrame(KapView *view, QWidget *parent)
  : QFrame(parent), _view(view)
{

  int i;

  _top = new QHBoxLayout(this, 2);
  
//    setBackgroundColor(mainWin->leftColor());
//    setFrameStyle( Panel | Raised );

  for (i=0; i<N_RR; ++i) {
    // TODO: move all pixmap stuff into gGame (centralization)
    // or new class "Pixmap/Image-Server"
    _otherIcon[i] = new OtherIcon(_view, this, 
				  _view->getPixmap("rrowned"),
				  _view->getPixmap("rrshape"),
				  _view->getPixmap("rrmortg"));
    _otherIcon[i]->setBackgroundColor(Qt::white);
    _top->addWidget(_otherIcon[i]);
  }
  // "continue" loop
  assert (i==4);	// for now we must be sure to have exactly 4 railroads

  _top->addSpacing(10);
  _otherIcon[i] = new OtherIcon(_view, this, 
				_view->getPixmap("electricowned"),
				_view->getPixmap("electricshape"),
				_view->getPixmap("electricmortg"));
  _otherIcon[i]->setBackgroundColor(Qt::white);
  _top->addWidget(_otherIcon[i++]);
  _otherIcon[i] = new OtherIcon(_view, this,
				_view->getPixmap("waterowned"),
				_view->getPixmap("watershape"),
				_view->getPixmap("watermortg"));
  _otherIcon[i]->setBackgroundColor(Qt::white);
  _top->addWidget(_otherIcon[i++]);

  assert(i==N_OTHERS);	// and be sure to have 2 utils

  _top->activate();

  show();
}
 
OtherInfoFrame::~OtherInfoFrame()
{
}


void OtherInfoFrame::init_icons()
{
  // just made in constructor

//    int i,j;
//    Monop *monop;

//    int x,y;
//    int dist = 5; // distance between two overlapping cards
  
//    // a little border
//    x = 4; y = 4;
  
//    for (i=0; i<BOARD_N_MONOPS; ++i) {
//      for (j=0; j<MAX_STREETS_IN_MONOP; ++j) {
//        if (_streetIcon[i][j]) {
//        }
//      }
//    }
//    // create new icons
//    for (i=0; i<BOARD_N_MONOPS; ++i) {
//      monop = _view->getGame()->board()->monop(i);
//      for (j=0; j<monop->nstreets(); ++j) {
//        _OtherIcon[i][j] = new OtherIcon(_view, this, i);
//        _OtherIcon[i][j]->move( QPoint(x,y) );
//        _OtherIcon[i][j]->set_highlighted(true);
//        _OtherIcon[i][j]->show();
//        x += dist;
//        y += dist;
//      }
//      x += OtherIcon::DEFAULT_WIDTH;
//      y = 4;
//    }

//    adjustSize();
//    setMinimumSize( size() );
}

void OtherInfoFrame::update()
{
  int i;
  int index[N_OTHERS] = { RRSOUTH, RRWEST, RRNORTH, RRCENTRAL,
			  UELECTRIC, UWATER };

  assert(_view->getGame()->myPlr());
//    int nrailroads = _view->getGame()->myPlr()->nrailroads();

  for (i=0; i< N_OTHERS; ++i) {
    if (_view->getGame()->myPlr()->holdings(index[i]) != NULL)	{ 
      // we own it
      _otherIcon[i]->set_highlighted(true);

      Property *p 
	= dynamic_cast<Property *>(_view->getGame()->board()->field(index[i]));
      assert(p);
      if (p->is_mortgaged())
	_otherIcon[i]->set_mortgaged(true);
      else
	_otherIcon[i]->set_mortgaged(false);

    } else {	// its not ours
      _otherIcon[i]->set_highlighted(false);
      _otherIcon[i]->set_mortgaged(false);
    }
  }
  
//    for (i=0; i<N_RR; ++i) {
//      assert(_otherIcon[i]);
//      if (i<nrailroads)
//        _otherIcon[i]->set_highlighted(true);
//      else
//        _otherIcon[i]->set_highlighted(false);
//    }

//    assert(i==4);	// be sure to have 4 railroads

//    // utilities
//    if (_view->getGame()->myPlr()->holdings(UELECTRIC) != NULL) 
//      // we own electric co.
//      _otherIcon[i]->set_highlighted(true);
//      if (_view->getGame()->board()->field(UELECTRIC)->is_mortgaged())
//        _otherIcon[i]->set_mortgaged(true);
//      else
//        _otherIcon[i]->set_mortgaged(false);
//    } else {	// its not ours
//      _otherIcon[i]->set_highlighted(false);
//      _otherIcon[i]->set_mortgaged(false);
//    }

//    i++;
//    if (_view->getGame()->myPlr()->holdings(UWATER) != NULL)
//      // we own water works
//      _otherIcon[i]->set_highlighted(true);
//    else
//      _otherIcon[i]->set_highlighted(false);
}



OtherIcon::OtherIcon(KapView *view, QWidget *parent,
		       QPixmap highPix, QPixmap normPix, QPixmap mortgPix)
  : QLabel(parent), _view(view),
    _highPix(highPix),
    _normPix(normPix),
    _mortgPix(mortgPix),
    _is_highlighted(false),
    _is_mortgaged(false)
{
//    setBackgroundColor(mainWin->leftColor());
  setPixmap(normPix);
  setMinimumSize( sizeHint() );
  
  show();
}

OtherIcon::~OtherIcon()
{
}

void OtherIcon::set_highlighted(bool b)
{

  if (b == _is_highlighted) return; // status didn't change

  _is_highlighted = b;

  if (b) setPixmap(_highPix);
  else setPixmap(_normPix);

}

void OtherIcon::set_mortgaged(bool b)
{

  if (b == _is_mortgaged) return; // status didn't change

  _is_mortgaged = b;

  if (b) setPixmap(_mortgPix);
  else setPixmap(_highPix);

}




/***************************************/

JailCardInfoFrame::JailCardInfoFrame(KapView *view, QWidget *parent)
  : QFrame(parent), _view(view)
{

  int i;

  _top = new QHBoxLayout(this, 2);
  
//    setBackgroundColor(mainWin->leftColor());
//    setFrameStyle( Panel | Raised );

  for (i=0; i<N_JAILCARDS; ++i) {
    // TODO: move all pixmap stuff into gGame (centralization)
    // or new class "Pixmap/Image-Server"
    _jailcardIcon[i] = new JailCardIcon(_view, this, 
					_view->getPixmap("jcardowned"),
					_view->getPixmap("jcardshape"));
    _jailcardIcon[i]->setBackgroundColor(Qt::white);
    _top->addWidget(_jailcardIcon[i]);
  }

  _top->addStretch(1);
  _top->activate();

  show();
}
 
JailCardInfoFrame::~JailCardInfoFrame()
{
}


void JailCardInfoFrame::init_icons()
{
  // just made in constructor
}

void JailCardInfoFrame::update()
{
  int i;

  assert(_view->getGame()->myPlr());
  
  int ncards = _view->getGame()->myPlr()->ncards();
  assert(ncards<=N_JAILCARDS);

  for (i=0; i<ncards; ++i) {
    _jailcardIcon[i]->set_highlighted(true);
  } 
  for (;i<N_JAILCARDS;++i) {
    _jailcardIcon[i]->set_highlighted(false);
  }
}



JailCardIcon::JailCardIcon(KapView *view, QWidget *parent, 
			     QPixmap highPix, QPixmap normPix)
  : QLabel(parent), _view(view),
    _highPix(highPix),
    _normPix(normPix),
    _is_highlighted(false)
{
//    setBackgroundColor(mainWin->leftColor());
  setPixmap(normPix);
  setMinimumSize( sizeHint() );
  
  show();
}

JailCardIcon::~JailCardIcon()
{
}

void JailCardIcon::set_highlighted(bool b)
{

  if (b == _is_highlighted) return; // status didn't change

  _is_highlighted = b;

  if (b) setPixmap(_highPix);
  else setPixmap(_normPix);
}

