// -*- C++ -*-

/***************************************************************************
                          myholdingsinfo.h  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MYHOLDINGSINFO_H
#define MYHOLDINGSINFO_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

//
// Qt
//

#include <qframe.h>
#include <qlabel.h>
#include <qcolor.h>
#include <qpixmap.h>

class QGridLayout;
class QBoxLayout;

//
// Game
//

#include "board.h"

//
// View
//

class KapView;
class StreetInfoFrame;
class StreetIcon;
class OtherInfoFrame;
class OtherIcon;
class JailCardInfoFrame;
class JailCardIcon;


/**
 * TODO
 */
class MyHoldingsInfo : public QFrame {
  Q_OBJECT

public:

  MyHoldingsInfo(KapView *view, QWidget *parent);
  virtual ~MyHoldingsInfo();

  void init();
  void update();

protected:

  KapView *_view;

  QGridLayout *_top;

  QLabel *_topLab;
  StreetInfoFrame *_streetFrm;
  OtherInfoFrame *_otherFrm;
  JailCardInfoFrame *_jcardFrm;

};


/**
 * TODO
 */
class StreetInfoFrame : public QFrame {
  Q_OBJECT

public:
  StreetInfoFrame(KapView *view, QWidget *parent);
  virtual ~StreetInfoFrame();

  virtual void resizeEvent( QResizeEvent * );

  void init_icons();
  void update();

protected:

  KapView *_view;

  StreetIcon *_streetIcon[BOARD_N_MONOPS][MAX_STREETS_IN_MONOP];

};


/**
 * TODO
 */
class StreetIcon : public QFrame {
  Q_OBJECT

public:
  enum { DEFAULT_WIDTH  = 12, DEFAULT_HEIGHT = 18 };
  static const QColor normal_color;
  static const QColor highlight_color;
  static const QColor mortgaged_color;

public:

  StreetIcon(KapView *view, QWidget *parent, int monopid);
  virtual ~StreetIcon();

  void set_highlighted(bool);
  void set_mortgaged(bool);

protected:
  
  KapView *_view;

  int _monopid;
  QColor _monop_color;

  bool _is_highlighted;
  bool _is_mortgaged;

  QFrame *_innerFrm;

};


/**
 * railroads and utilities
 */
class OtherInfoFrame : public QFrame {
  Q_OBJECT

public:
  enum { N_RR = 4, N_UTILS = 2, N_OTHERS = 6 };	// 4 railroads and 2 utilities

public:
  OtherInfoFrame(KapView *view, QWidget *parent);
  virtual ~OtherInfoFrame();

//    virtual void resizeEvent( QResizeEvent * );

  void init_icons();
  void update();

protected:

  KapView *_view;

  QBoxLayout *_top;

  OtherIcon *_otherIcon[N_OTHERS];

};


/**
 * TODO
 */
class OtherIcon : public QLabel {
  Q_OBJECT

public:
  OtherIcon(KapView *view, QWidget *parent, 
	     QPixmap highlightedPixmap, QPixmap normalPixmap,
	     QPixmap mortgagedPixmap);
  virtual ~OtherIcon();
  
  void set_highlighted(bool);
  void set_mortgaged(bool);

protected:

  KapView *_view;

  QPixmap _highPix;
  QPixmap _normPix;
  QPixmap _mortgPix;

  bool _is_highlighted;
  bool _is_mortgaged;

};



/**
 * jail free cards
 */
class JailCardInfoFrame : public QFrame {
  Q_OBJECT

public:
  enum { N_JAILCARDS = 2 };

public:
  JailCardInfoFrame(KapView *view, QWidget *parent);
  virtual ~JailCardInfoFrame();

//    virtual void resizeEvent( QResizeEvent * );

  void init_icons();
  void update();

protected:

  KapView *_view;

  QBoxLayout *_top;

  JailCardIcon *_jailcardIcon[N_JAILCARDS];

};


/**
 *
 */
class JailCardIcon : public QLabel {
  Q_OBJECT

public:
  JailCardIcon(KapView *view, QWidget *parent, 
		QPixmap highlightedPixmap, QPixmap normalPixmap);
  virtual ~JailCardIcon();
  
  void set_highlighted(bool);

protected:

  KapView *_view;

  QPixmap _highPix;
  QPixmap _normPix;

  bool _is_highlighted;

};

#endif // MYHOLDINGSINFO_H




