// -*- C++ -*-

/***************************************************************************
                          myplrinfo.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// System
//

#include <cmath>

//
// Qt
//

#include <qframe.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qcolor.h>

//
// Game
//

#include "kapgame.h"
#include "player.h"

//
// View
//

#include "kapview.h"
#include "playerview.h"
#include "myplrinfo.h"

#include "themedata.h"


MyPlrInfo::MyPlrInfo(KapView *view, QWidget *parent)
  : QFrame(parent), _view(view)
{
//    setAlignment(AlignLeft);
//    setTitle("Your Player");
  setFrameStyle( Box | Plain );
  setLineWidth(2);
  setMargin(2);
//    setBackgroundColor(_view->leftColor().light(150));
  setBackgroundColor(Qt::white);

  _top = new QGridLayout( this, 2,3, 5);

  _top->setColStretch(0,1);
  _top->setColStretch(1,0);
  _top->setColStretch(2,1);

  _topLab = new QLabel(this);
  QFont topLabFont = _topLab->font();
  topLabFont.setBold(true);
  _topLab->setFont(topLabFont);
  _topLab->setText("Your Player");
  _topLab->setBackgroundColor(backgroundColor());
  _topLab->show();

  _top->addMultiCellWidget(_topLab, 0,0, 0,1, Qt::AlignLeft);

//    _nameDesc = new QLabel(this);
//    _nameDesc->setText("Name:");
//    _nameDesc->setAlignment(AlignRight);
//    _nameDesc->setMinimumSize( _nameDesc->sizeHint() );
//    _nameDesc->setBackgroundColor(backgroundColor());
//    _nameDesc->show();

//    _top->addWidget(_nameDesc, 1,0);

  _name = new QLabel(this);
  _name->setText("default");
  _name->setAlignment(AlignLeft);
  _name->setMinimumSize (_name->sizeHint() );
  _name->setBackgroundColor(backgroundColor());
  _name->show();

  _top->addWidget(_name, 1,0);

//    _tokenDesc = new QLabel(this);
//    _tokenDesc->setText("Token:");
//    _tokenDesc->setAlignment(AlignRight);
//    _tokenDesc->setMinimumSize( _tokenDesc->sizeHint() );
//    _tokenDesc->setBackgroundColor(backgroundColor());
//    _tokenDesc->show();

//    _top->addWidget(_tokenDesc, 2,0);

  // default token (gray), color is set in gGame::handle_game_info  
  _token = PlayerView::get_default_token(_view, this);
  _token->setAlignment(AlignCenter);
  _token->setFixedSize( _token->sizeHint() );
//    _iconLab->setFixedSize(QSize(20,20));
  _token->show();

  _top->setRowSpacing( 1, _token->height() );
  _top->setColSpacing( 1, _token->width() );
  _top->addWidget(_token, 1,1);

//    _budgetDesc = new QLabel(this);
//    _budgetDesc->setText("Budget:");
//    _budgetDesc->setAlignment(AlignRight);
//    _budgetDesc->setMinimumSize( _budgetDesc->sizeHint() );
//    _budgetDesc->setBackgroundColor(backgroundColor());
//    _budgetDesc->show();

//    _top->addWidget(_budgetDesc, 3,0);

  _budget = new QLabel(this);
  QFont budgetFont = _budget->font();
  budgetFont.setBold(true);
  _budget->setFont( budgetFont );
  _budget->setText("$123456 (-$123456)");
  _budget->setAlignment(AlignRight);
  _budget->setMinimumSize (_budget->sizeHint() );
  _budget->setBackgroundColor(backgroundColor());
//    _budget->setAutoResize(true);
  _budget->show();

  _top->addWidget(_budget, 1,2);

//    _debts = new QLabel(this);
//    QFont debtsFont = _debts->font();
//    debtsFont.setBold(true);
//    _debts->setFont( debtsFont );
//    _debts->setText("($123456)");
//    _debts->setAlignment(AlignCenter);
//  //    _debts->setMargin(1);
//    _debts->setMinimumSize ( _debts->sizeHint() );
//    _debts->setBackgroundColor(QColor(255, 150, 150));
//    _debts->setAutoResize(true);
//    _debts->show();

//    _top->addWidget(_debts, 1,3);

  _top->activate();

  show();
}

MyPlrInfo::~MyPlrInfo()
{
}

void MyPlrInfo::init()
{
  set_tokenColor(_view->myPlrView()->tokenColor());
  _name->setText(_view->getGame()->myPlr()->name());
}

void MyPlrInfo::update()
{
  assert(_view->getGame()->myPlr());

  int budget = _view->getGame()->myPlr()->budget();
  QString budstr = _view->money_locale(budget);

  int debts = _view->getGame()->myPlr()->debts_sum();
  QString debtstr = _view->money_locale(debts);

  if (debts > 0 ) {  // only show debts if there are some
    budstr.append( QString(" (-") + debtstr + QString(")") );
  }

  _budget->setText(budstr);

}

void MyPlrInfo::set_tokenColor(QColor c)
{
  _token->setBackgroundColor(c);
}

void MyPlrInfo::showToken()
{
  if (_token->isHidden()) _token->show();
}

void MyPlrInfo::hideToken()
{
  if (_token->isVisible()) _token->hide();
}
