// -*- C++ -*-

/***************************************************************************
                          playerinfo.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// System
//

#include <cmath>

//
// Qt
//

#include <qlayout.h>

//
// Game
//

#include "kapgame.h"
#include "player.h"


//
// View
//

#include "kapview.h"
#include "playerview.h"
#include "playerinfo.h"

#include "themedata.h"


PlayerInfo::PlayerInfo(KapView *view, QWidget *parent)
  : QFrame(parent), _view(view)
{
//    setTitle("Other Players");
  setFrameStyle( Box | Plain );
  setLineWidth(2);
  setMargin(2);
//    setBackgroundColor(_view->leftColor().light(150));
  setBackgroundColor(Qt::white);

  // init() creates all the elements (here we have not the necessary 
  // information (nplayers etc.)
}

PlayerInfo::~PlayerInfo()
{
}

void PlayerInfo::init()
{
  int i;
//    int myPID = _view->getGame()->myPID();

//    for (i=0; i<_view->getGame()->nplayers(); ++i) {

//      if (i==myPID) continue;
  
//      _token[i]->setBackgroundColor( _view->plr(i)->tokenColor() );
//      _name[i]->setText( _view->getGame()->player(i)->name() );
//    }

  _top = new QGridLayout( this, _view->getGame()->nplayers()+1 ,3, 5);

  _top->setColStretch(0,1);
  _top->setColStretch(1,0);
  _top->setColStretch(2,1);
  
  _topLab = new QLabel(this);
  QFont topLabFont = _topLab->font();
  topLabFont.setBold(true);
  _topLab->setFont(topLabFont);
  _topLab->setText("Other Players");
  _topLab->setBackgroundColor(backgroundColor());
  _topLab->show();

  _top->addMultiCellWidget(_topLab, 0,0, 0,1, Qt::AlignLeft);

//    _top->addRowSpacing(0,10);
//    _top->addRowSpacing(_view->getGame()->nplayers(),5);

  int myPID = _view->getGame()->myPID();

  for (i=0; i<_view->getGame()->nplayers(); ++i) {

    if (i==myPID) {
      _name[i] = 0;
      continue;
    }
    
    _name[i] = new QLabel(this);
    _name[i]->setText(_view->getGame()->player(i)->name());
    _name[i]->setAlignment(AlignLeft);
    _name[i]->setMinimumSize (_name[i]->sizeHint() );
    _name[i]->setBackgroundColor(backgroundColor());
    _name[i]->show();
    _top->addWidget(_name[i], i+1 - ( (i>myPID)?1:0 ) ,0);

    // default token (gray), color is set in gGame::handle_game_info  
    _token[i] = PlayerView::get_default_token(_view, this);
//      _token[i]->setMinimumSize( _token[i]->sizeHint() );
    _token[i]->setFixedSize(_token[i]->sizeHint());
    _token[i]->setBackgroundColor(_view->plr(i)->tokenColor());
    _token[i]->show();

    _top->setRowSpacing( i+1 - ( (i>myPID)?1:0 ), _token[i]->height() );
    _top->setColSpacing( 1, _token[i]->width() );
    _top->addWidget(_token[i], i+1 - ( (i>myPID)?1:0 ) ,1);

    _budget[i] = new QLabel(this);
//      _budget[i]->setFont( QFont("He", 14, QFont::Bold) );
//      QString budstr 
//        = ( QString("$") 
//  	  + QString().setNum(_view->getGame()->player(i)->budget())
//  	  + QString(" (-$")
//  	  + QString().setNum(_view->getGame()->player(i)->debts_sum())
//  	  + QString(")") );
//      _budget[i]->setText(budstr);
    _budget[i]->setText("$123456 (-$123456)");
    _budget[i]->setAlignment(AlignRight);
    _budget[i]->setMinimumSize (_budget[i]->sizeHint() );
    _budget[i]->setBackgroundColor(backgroundColor());
    _budget[i]->show();
    
    _top->addWidget(_budget[i], i+1 - ( (i>myPID)?1:0 ) ,2);
    
  }

    
  _top->activate();

  show();
}

void PlayerInfo::update()
{
  int i;

  for (i=0; i<_view->getGame()->nplayers(); ++i) {
    
    if (i==_view->getGame()->myPID()) {
      continue;
    }
    
    // name and token don't change, so update only budget

    int budget = _view->getGame()->player(i)->budget();
    QString budstr = _view->money_locale(budget);

    int debts = _view->getGame()->player(i)->debts_sum();
    QString debtstr = _view->money_locale(debts);

    if (debts > 0 ) {  // only show debts if there are some
      budstr.append( QString(" (-") + debtstr + QString(")") );
    }

    _budget[i]->setText(budstr);

  }
}


void PlayerInfo::showToken(int plrid)
{
  assert(!_view->getGame()->nobody(plrid));
  if (_token[plrid]->isHidden()) _token[plrid]->show();
}

void PlayerInfo::hideToken(int plrid)
{
  assert(!_view->getGame()->nobody(plrid));
  if (_token[plrid]->isVisible()) _token[plrid]->hide();
}
