// -*- C++ -*-
/***************************************************************************
                          propertydeed.cpp  -  description
                             -------------------
    begin                : Mon Jul 13 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// System
//

#include <assert.h>

//
// Qt
//

#include <qlayout.h>
#include <qlabel.h>
#include <qfont.h>

//
// App
//

#include "kapitalist.h"

//
// Game
//

#include "property.h"

//
// View
//

#include "kapview.h"
#include "fieldview.h"
#include "propertyview.h"
#include "propertydeed.h"

/**
 *
 */
PropertyDeed::PropertyDeed(KapitalistApp *app, QWidget *parent, 
			   const PropertyView *propview, int variant)
  : Deed(app, parent, variant),
    _propview(propview)
{
  QString text;
  QFont font;

//    const Property *prop = _propview->property();

  switch(_variant) {
  case NORMAL: 
    
    _top = new QVBoxLayout(this, 1);
  
    _headFrm = new QFrame(this);
//      _headFrm->setBackgroundColor( _propview->color() );
    _headFrm->setPalette( QPalette( _propview->color() ) );
    _headFrm->setFrameStyle( NoFrame );
    //    _headFrm->setLineWidth(0);
    _detailsFrm = new QFrame(this);
    _detailsFrm->setFrameStyle( NoFrame );
    //    _detailsFrm->setLineWidth(0);
    _detailsFrm->setBackgroundColor( Qt::white );

    _head = new QGridLayout(_headFrm, 1,1, 1);
    _details = new QGridLayout(_detailsFrm, 1,1);

    _top->addWidget(_headFrm);
    _top->addWidget(_detailsFrm);

    _top->setStretchFactor(_headFrm, 1);
    _top->setStretchFactor(_detailsFrm, 3);
    
    lab = new QLabel(_headFrm);
    text = _propview->fullTitle();
    text.truncate(5);
    text.append("...");
    lab->setText(text);

    _head->addWidget(lab, 0,0, AlignCenter);

    _head->activate();
    _details->activate();
  
    _top->activate();
    break;

  case ELABORATE:
    _top = new QVBoxLayout(this, 1);
  
//     setMinimumSize( 250, 330 );
    _headFrm = new QFrame(this);
    _headFrm->setPalette( QPalette( _propview->color() ) );

    _top->addWidget(_headFrm);
      
    _head = new QGridLayout(_headFrm, 4,1, 1);
    _head->addRowSpacing(0,10);
    _head->addRowSpacing(3,10);

    lab = new QLabel(_headFrm);
    lab->setText("TITLE DEED");
    _head->addWidget(lab, 1,0, AlignCenter);
    lab = new QLabel(_headFrm);
    font = lab->font();
    font.setPointSize(font.pointSize()+4);
    font.setWeight(QFont::Bold);
    lab->setFont(font);
    text = _propview->fullTitle();
    lab->setText(text);

    _head->addWidget(lab, 2,0, AlignCenter);

    _head->activate();

    _detailsFrm = new QFrame(this);
    _detailsFrm->setBackgroundColor( Qt::white );
    _top->addWidget(_detailsFrm);

    _top->setStretchFactor(_headFrm, 1);
    _top->setStretchFactor(_detailsFrm, 3);

    _top->activate();
    break;

  default:
    assert(0);
  }
}


PropertyDeed::~PropertyDeed()
{
}

void PropertyDeed::updateStatus()
{
}

void PropertyDeed::addLine(int row, QString text1, QString text2, int money)
{
  QFont font;
  lab = new QLabel(_detailsFrm);
  lab->setBackgroundColor(_detailsFrm->backgroundColor());
  font = lab->font();
  font.setWeight(QFont::Bold);
  lab->setFont(font);
  lab->setText(text1);
  _details->addWidget(lab, row,1, AlignLeft);
  lab = new QLabel(_detailsFrm);
  lab->setBackgroundColor(_detailsFrm->backgroundColor());
  lab->setText(text2);
  _details->addWidget(lab, row,3, AlignLeft);
  lab = new QLabel(_detailsFrm);
  lab->setBackgroundColor(_detailsFrm->backgroundColor());
  lab->setText(_app->view()->money_locale(money));
  _details->addWidget(lab, row,5, AlignLeft);
  
}

void PropertyDeed::addLine(int row, QString text1, QString text2, QString rent)
{
  QFont font;
  lab = new QLabel(_detailsFrm);
  lab->setBackgroundColor(_detailsFrm->backgroundColor());
  font = lab->font();
  font.setWeight(QFont::Bold);
  lab->setFont(font);
  lab->setText(text1);
  _details->addWidget(lab, row,1, AlignLeft);
  lab = new QLabel(_detailsFrm);
  lab->setBackgroundColor(_detailsFrm->backgroundColor());
  lab->setText(text2);
  _details->addWidget(lab, row,3, AlignLeft);
  lab = new QLabel(_detailsFrm);
  lab->setBackgroundColor(_detailsFrm->backgroundColor());
  lab->setText(rent);
  _details->addWidget(lab, row,5, AlignLeft);
}
