// -*- C++ -*-

/***************************************************************************
                          propertyview.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// System
//

#include <assert.h>

//
// Qt / KDE
//

#include <qlayout.h>
#include <qframe.h>
#include <qregexp.h>
#include <qpixmap.h>

#include <kpopupmenu.h>

#include <klocale.h>

//
// Game
//

#include "board.h"
#include "property.h"
#include "railroad.h"
#include "utility.h"
#include "player.h"

//
// View
//

#include "boardview.h"
#include "kapview.h"
#include "playerview.h"
#include "propertyview.h"

#include "propinfodialog.h"
#include "propertydeed.h"
#include "railroaddeed.h"
#include "utilitydeed.h"

#include "themedata.h"


PropertyView::PropertyView (KapView *view, QWidget *parent,
			    const int orientation, 
			    const Property *prop, bool add_base_widgets) 
  : FieldView(view, parent, orientation, prop, false),
    _prop(prop), _color(Qt::white.dark(130))
{
  _sideFrm = new QFrame(this);
  _main = new QVBoxLayout();

  switch (_prop->location()) {
  case UELECTRIC:
    set_icon("electric", _sideFrm);
    break;
  case UWATER:
    set_icon("water", _sideFrm);
    break;
  }

  switch (_prop->type()) {
  case RAILROAD:

    switch (_orientation) {
    case BoardView::TOP: 
      set_icon("rrtop", _sideFrm);
      break;
    case BoardView::BOTTOM:
      set_icon("rrbottom", _sideFrm);
      break;
    case BoardView::LEFT:
      set_icon("rrleft", _sideFrm);
      break;
    case BoardView::RIGHT:
      set_icon("rrright", _sideFrm);
      break;

    default:
      assert(0);	// there are no properties on edge fields
    }

  default:
    ;	// no icon
  }

  _ownedSign = new QFrame(_sideFrm);
  _mortgSign = new QLabel(_sideFrm);
  _ownedSign->setFrameStyle( Box | Sunken ); 
//    _mortgSign->setMask(_view->getBitmap("mortgmask"));
  _mortgSign->setAlignment(AlignCenter);
//    _mortgSign->setMinimumSize( 10,10);
  

  switch (orientation) {
  case BoardView::TOP: 
  case BoardView::BOTTOM:
    _sideWidth = DEFAULT_WIDTH;
    _sideHeight = DEFAULT_HEIGHT/3;
    _sideFrm->setMinimumHeight(_sideHeight);
    _side = new QGridLayout(_sideFrm, 3, 5, 1);
    _top->addLayout(_main, 9);
    _top->addWidget(_sideFrm, 3);
    // setMinimumSize did't work with Streets (qt-2.2.4 bug??)
    _ownedSign->setFixedSize(15,6);
    break;
  case BoardView::LEFT:
  case BoardView::RIGHT:
    _sideWidth = DEFAULT_HEIGHT/3;
    _sideHeight = DEFAULT_WIDTH;
    _sideFrm->setMinimumWidth(_sideWidth);
    _side = new QGridLayout(_sideFrm, 5, 3, 1);
    _top->addLayout(_main, 9);
    _top->addWidget(_sideFrm, 3);
    // setMinimumSize did't work with Streets (qt-2.2.4 bug??)
    _ownedSign->setFixedSize(6,15);
    break;
  default:
    assert(0);	// not possible -> bug
  }

  switch (_orientation) {
  case BoardView::TOP: 
    _side->setRowStretch(0,2);
    _side->setRowStretch(1,1);
    _side->setRowStretch(2,0);
    _side->setColStretch(0,1);
    _side->setColStretch(1,1);
    _side->setColStretch(2,1);
    _side->setColStretch(3,1);
    _side->setColStretch(4,0);
//      _side->setRowStretch(0,0);
//      _side->setRowStretch(1,0);
//      _side->setRowStretch(2,1);
//      _side->setColStretch(0,0);
//      _side->setColStretch(1,0);
//      _side->setColStretch(2,0);
//      _side->setColStretch(3,0);
//      _side->setColStretch(4,1);
//      _side->addRowSpacing(0, 3*_sideHeight/4);
//      _side->addRowSpacing(1, 1*_sideHeight/4);
//      _side->addColSpacing(0, _sideWidth/4);
//      _side->addColSpacing(1, _sideWidth/4);
//      _side->addColSpacing(2, _sideWidth/4);
//      _side->addColSpacing(3, _sideWidth/4);
    _mortgSign->setPixmap(_view->getPixmap("mortgtop"));
    break;
  case BoardView::BOTTOM:
    _side->setRowStretch(0,1);
    _side->setRowStretch(1,2);
    _side->setRowStretch(2,0);
    _side->setColStretch(0,1);
    _side->setColStretch(1,1);
    _side->setColStretch(2,1);
    _side->setColStretch(3,1);
    _side->setColStretch(4,0);
    _mortgSign->setPixmap(_view->getPixmap("mortgtop"));
    break;
  case BoardView::LEFT: 
    _side->setColStretch(0,2);
    _side->setColStretch(1,1);
    _side->setColStretch(2,0);
    _side->setRowStretch(0,1);
    _side->setRowStretch(1,1);
    _side->setRowStretch(2,1);
    _side->setRowStretch(3,1);
    _side->setRowStretch(4,0);
    _mortgSign->setPixmap(_view->getPixmap("mortgleft"));
    break;
  case BoardView::RIGHT:
    _side->setColStretch(0,1);
    _side->setColStretch(1,2);
    _side->setColStretch(2,0);
    _side->setRowStretch(0,1);
    _side->setRowStretch(1,1);
    _side->setRowStretch(2,1);
    _side->setRowStretch(3,1);
    _side->setRowStretch(4,0);
    _mortgSign->setPixmap(_view->getPixmap("mortgright"));
    break;

  default:
    assert(0);
  }


  setup_price(_prop->price());

  if (add_base_widgets) add_all_widgets();


  _popup = new KPopupMenu(this);

  _popup->insertTitle(QString(_view->field_locale(_field->location())->name));
  _popup->insertItem(i18n("&Information"), this, SLOT(slotInfoAbout()), 
		      0, POPUPID_INFO );
//    _popup->setItemParameter( POPUPID_INFO, _field->id() );
  _popup->insertSeparator();
  _popup->insertItem( i18n("&Buy"), 
		      this, SLOT(slotBuy()), 
		      0, POPUPID_BUY );
  _popup->insertItem( i18n("&Mortgage"), 
		      this, SLOT(slotMortgage()), 
		      0, POPUPID_MORTGAGE );
//    _popup->setItemParameter( POPUPID_MORTGAGE, _field->id() );
  _popup->insertItem( i18n("&Unmortgage"), 
		      this, SLOT(slotUnmortgage()), 
		      0, POPUPID_UNMORTGAGE );
//    _popup->setItemParameter( POPUPID_UNMORTGAGE, _field->id() );
  
  _titleLab->installEventFilter( this );
  _priceLab->installEventFilter( this );
  _sideFrm->installEventFilter( this );

  QObject::connect(_popup, SIGNAL(aboutToShow()),
		   this, SLOT(updatePopupMenu()));
//    _titleLab->ignore();
//    _priceLab->ignore();
}


PropertyView::~PropertyView()
{
}

//  void PropertyView::resizeEvent( QResizeEvent *e)
//  {
//    int i;
//    if (_field) {
//      init_plr_positions();
//      for (i=0; i<GAME_MAX_PLAYERS; ++i) {
//        if (_plr[i])
//  	_plr[i]->move(mapToParent(QPoint(0,0)) + _plrPos[i]);   
//      }
//    }

//  }

bool PropertyView::eventFilter( QObject *o, QEvent *e )
{
  if ( e->type() == QEvent::MouseButtonPress ) {
    QMouseEvent *m = (QMouseEvent*)e;
    if (m->button() == RightButton) {
      QWidget *w = (QWidget *) o;
      // TODO: update popup before show
      _popup->popup( w->mapToGlobal(QPoint(0,0)) + m->pos() );
      return TRUE;                        // eat event
    }
  }
  return QWidget::eventFilter( o, e );    // standard event processing
}
  
void PropertyView::setup_price(const int price)
{
  QFont pricefont = font();
  pricefont.setPointSize(10);
  pricefont.setWeight(QFont::Bold);

  _price = _view->money_locale(price);

  _priceLab = new QLabel(this);
//    _priceLab->setFont (QFont ("helvet", 10, QFont::Bold));
  _priceLab->setFont(pricefont);
  _priceLab->setAlignment(AlignCenter);
  _priceLab->setText(_price);
  _priceLab->setBackgroundColor(_baseColor);
//    _priceLab->setMinimumSize( _priceLab->sizeHint() );
}

void PropertyView::change_price(const int newprice)
{
  _price = _view->money_locale(newprice);
  _priceLab->setText(_price);
}


void PropertyView::add_all_widgets()
{
  _main->addStretch(1);
  _main->addWidget(_titleLab, 10);
  _main->addWidget(_priceLab, 10);

  _sideFrm->setBackgroundColor(_baseColor);
  _mortgSign->setBackgroundColor(_baseColor);

  switch (_orientation) {
  case BoardView::TOP: 
    if (_icon) _side->addMultiCellWidget(_icon, 0,1,0,3);
    _side->addWidget(_ownedSign, 1,0);
//      _side->addWidget(_mortgSign, 1,3);
    _side->addMultiCellWidget(_mortgSign, 0,1,3,3);
    break;
  case BoardView::BOTTOM:
    if (_icon) _side->addMultiCellWidget(_icon, 0,1,0,3);
    _side->addWidget(_ownedSign, 0,0);
//      _side->addWidget(_mortgSign, 0,3);
    _side->addMultiCellWidget(_mortgSign, 0,1,3,3);
    break;
  case BoardView::LEFT: 
    if (_icon) _side->addMultiCellWidget(_icon, 0,3,0,1);
    _side->addWidget(_ownedSign, 0,1);
//      _side->addWidget(_mortgSign, 3,1);
    _side->addMultiCellWidget(_mortgSign, 3,3,0,1);
    break;
  case BoardView::RIGHT:
    if (_icon) _side->addMultiCellWidget(_icon, 0,3,0,1);
    _side->addWidget(_ownedSign, 0,0);
//      _side->addWidget(_mortgSign, 3,0);
    _side->addMultiCellWidget(_mortgSign, 3,3,0,1);
    break;

  default:
    assert(0);
  }

//    _ownedSign->hide();
//    _mortgSign->hide();
}


void PropertyView::activate_layouts()
{
  _side->activate();
  _top->activate();
//    _mortgSign->clearMask();
//    _mortgSign->setMask(_view->getBitmap("mortgmask"));
}


void PropertyView::init_plr_positions()
{
  assert(GAME_MAX_PLAYERS >= 4);
  assert(GAME_MAX_PLAYERS <= 8);

  //	0	4	2
  //	7		6
  //	3	5	1

  int w = width();
  int h = height();
  int off;
  int bx = w/10;	// border in x-direction
  int by = h/10;	// border in y-direction

  //  xl = x_left, xr = x_right
  //  yt = y_top, yb = y_bottom

  int xl, xr, yt, yb;

  switch (_orientation) {
    
  case BoardView::TOP:
    off = _sideFrm->height();
    xl = bx;
    yt = by;
    xr = w-bx-20;
    yb = h-off-by-20;
    break;
  case BoardView::BOTTOM:
    off = _sideFrm->height();
    xl = bx;
    yt = off+by;
    xr = w-bx-20;
    yb = h-by-20;
    break;
  case BoardView::LEFT:
    off = _sideFrm->width();
    xl = bx;
    yt = by;
    xr = w-off-bx-20;
    yb = h-by-20;
    break;
  case BoardView::RIGHT:
    off = _sideFrm->width();
    xl = off+bx;
    yt = by;
    xr = w-bx-20;
    yb = h-by-20;
    break;
  default:
    assert(0);	// bug

  }

//    if (location() == 3)
//      cout << "init_plr_positions() in " << location() << endl;
  
  _plrPos[0] = QPoint(xl, yt);
  _plrPos[1] = QPoint(xr, yb);
  _plrPos[2] = QPoint(xr, yt);
  _plrPos[3] = QPoint(xl, yb);

  if (GAME_MAX_PLAYERS < 5) return;
  _plrPos[4] = _plrPos[0] + 0.5 * (_plrPos[2] - _plrPos[0]);
  if (GAME_MAX_PLAYERS < 6) return;
  _plrPos[5] = _plrPos[3] + 0.5 * (_plrPos[1] - _plrPos[3]);
  if (GAME_MAX_PLAYERS < 7) return;
  _plrPos[6] = _plrPos[4] + 0.5 * (_plrPos[1] - _plrPos[4]);
  if (GAME_MAX_PLAYERS < 8) return;
  _plrPos[7] = _plrPos[0] + 0.5 * (_plrPos[5] - _plrPos[0]);

}


void PropertyView::update_status()
{
  // set mortgaged and owner sign appropriate

  bool mortgaged = _prop->is_mortgaged();
  Player *owner = _prop->owner();
  PlayerView *gowner = (owner) ? _view->plr(owner->id()) : 0;
  
  if (mortgaged) _mortgSign->show();
  else _mortgSign->hide();

  if (gowner) {	// it's owned
    _ownedSign->setBackgroundColor(gowner->tokenColor());
    _ownedSign->show();
  } else {
    _ownedSign->hide();
  }
}

void PropertyView::resize_elements()
{
  if (_titleLab) resize_title();
  if (_priceLab) resize_price();
}

void PropertyView::resize_price()
{
  int wtrigger, htrigger;

  QFont pricefont = _priceLab->font();

  pricefont.setPointSize(_view->board()->fieldFont()->pointSize());

  _priceLab->setFont(pricefont);
}

void PropertyView::rereadThemeData()
{
  FieldView::rereadThemeData();
  if (_prop) {
    change_price(_prop->price());
  }
}


PropertyDeed *PropertyView::getDeed(KapitalistApp *app, QWidget *parent,
				    const int variant) const
{
  const Railroad *rr = dynamic_cast<const Railroad *>(_prop);
  const Utility *util = dynamic_cast<const Utility *>(_prop);
  
  if (rr) {
    return new RailroadDeed(app, parent, this, variant);
  } else if (util) {
    return new UtilityDeed(app, parent, this, variant);
  } else {
    return new PropertyDeed(app, parent, this, variant);
  }
}

void PropertyView::slotInfoAbout()
{
  PropInfoDialog *dlg = new PropInfoDialog(_view->getApp(), _view->getApp(), 
					   this);
  dlg->show();
}

void PropertyView::slotBuy()
{
  _view->getApp()->slotPropertyBuy(_prop->location());
}

void PropertyView::slotMortgage()
{
  struct packet_generic_integer pack;

  pack.value = _field->location();
  send_packet_generic_integer(_view->getApp()->conn(),
			      PACKET_PROPERTY_MORTGAGE, &pack);
}

void PropertyView::slotUnmortgage()
{
  struct packet_generic_integer pack;

  pack.value = _field->location();
  send_packet_generic_integer(_view->getApp()->conn(),
			      PACKET_PROPERTY_UNMORTGAGE, &pack);
}

void PropertyView::updatePopupMenu()
{
  int actionflags = _prop->actions();
  // Buy
  if (actionflags & CAN_BUY) 
    _popup->setItemEnabled(POPUPID_BUY, true);
  else
    _popup->setItemEnabled(POPUPID_BUY, false);

  // Mortgage
  if (actionflags & CAN_MORTGAGE) 
    _popup->setItemEnabled(POPUPID_MORTGAGE, true);
  else
    _popup->setItemEnabled(POPUPID_MORTGAGE, false);
  // Unmortgage
  if (actionflags & CAN_UNMORTGAGE) 
    _popup->setItemEnabled(POPUPID_UNMORTGAGE, true);
  else 
    _popup->setItemEnabled(POPUPID_UNMORTGAGE, false);
}

void PropertyView::mousePressEvent( QMouseEvent *e )
{
  if ( e->button() == RightButton ) {
    // TODO: update popup before show
    _popup->popup( mapToGlobal(QPoint(0,0)) 
		   + QPoint( size().width(), size().height() ) );
  }
}
