// -*- C++ -*-
/***************************************************************************
                          streetdeed.cpp  -  description
                             -------------------
    begin                : Mon Jul 13 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// System
//

#include <cassert>

//
// Qt
//

#include <qlayout.h>
#include <qlabel.h>

//
// App
//

#include "kapitalist.h"

//
// Game
//

#include "street.h"

//
// View
//

#include "kapview.h"
#include "fieldview.h"
#include "streetview.h"
#include "streetdeed.h"

/**
 *
 */
StreetDeed::StreetDeed(KapitalistApp *app, QWidget *parent, 
		       const StreetView *streetview, int variant)
  : PropertyDeed(app, parent, streetview, variant),
    _streetview(streetview)
{
  QString text;
  int i;
  int nrows = STREET_MAX_HOUSES + 2  +  3 ;
  int nspacing = 4;

  const Street *street = _streetview->street();

  switch(_variant) {
  case NORMAL: 
    // everything done in PropertyDeed constructor
    break;

  case ELABORATE:

    _headFrm->setFrameStyle( NoFrame);
    //    _headFrm->setLineWidth(0);
    _detailsFrm->setFrameStyle( NoFrame );
    //    _detailsFrm->setLineWidth(0);

    
    _details = new QGridLayout(_detailsFrm, nrows+nspacing+2,9);

    _details->addColSpacing(0,10);
    _details->addColSpacing(2,10);
    _details->addColSpacing(4,10);
    _details->addColSpacing(6,10);
    _details->addRowSpacing(0,10);
    _details->setRowStretch(nrows+nspacing+1,1);

    _details->addWidget(new QLabel(_detailsFrm), 0,0, AlignLeft);

    addLine(1, "RENT:", "no buildings", street->rent_for(0));
    for (i=1; i<=STREET_MAX_HOUSES; ++i) {
      text = ( QString().setNum(i) + QString(" house") 
	       + ( (i>1) ? QString("s") : QString::null ) );
      addLine(i+1, "", text, street->rent_for(i));
    }
    addLine(i+1, "", "1 hotel", street->rent_for(i));
    i+=2;
//      i = STREET_MAX_HOUSES+3;
    _details->addRowSpacing(i++,20);
    addLine(i++, "MORTGAGE", "value", street->price() / 2);
//      addLine(i++, "UNMORTGAGE", "interest", street->price() / 2);
    _details->addRowSpacing(i++,10);
    addLine(i++, "HOUSES", "cost", street->house_price());
    addLine(i++, "HOTELS", "cost", street->house_price());
    _details->addRowSpacing(i++,20);
//      cout << "iend = " << i << endl;
    _details->activate();
    _top->activate();
     setMinimumSize(sizeHint());
    break;

  default:
    assert(0);
  }
}


StreetDeed::~StreetDeed()
{
}

//  void StreetDeed::updateStatus()
//  {
//  }

