// -*- C++ -*-

/***************************************************************************
                          streetview.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// System
//

#include <cassert>

//
// Qt / KDE
//

#include <qlayout.h>
#include <qframe.h>
#include <qregexp.h>

#include <klocale.h>
#include <kpopupmenu.h>

//
// App
//

class KapitalistApp;

//
// Game
//

#include "board.h"
#include "street.h"
#include "monop.h"

//
// View
//

#include "kapview.h"
#include "boardview.h"
#include "streetview.h"
#include "monopview.h"
#include "playerview.h"

#include "streetdeed.h"


StreetView::StreetView (KapView *view, QWidget *parent,
			const int orientation, const Street *street) 
  : PropertyView(view, parent, orientation, street, true),
    _street(street)
{
  int i;

  _color = monop_color[street->monop()->id()];
//    _bithouse = _view->getBitmap("hauskl");
//    _bithouse = _view->getBitmap("haus");
//    _pixhotel = _view->getPixmap("hotel");
//    assert(!_bithouse.isNull() && !_pixhouse.isNull()
//  	 && !_bithotel.isNull() && !_pixhotel.isNull());
  _pixhouse = _view->getPixmap("house");
  _pixhotel = _view->getPixmap("hotel");

  for (i=0; i<STREET_MAX_HOUSES; ++i) {
    _house[i] = new QLabel(_sideFrm);
    _house[i]->setBackgroundColor (_color);
    _house[i]->setAlignment(AlignCenter);
//      _house[i]->setMask (_bithouse);
    _house[i]->setPixmap (_pixhouse);
//      _house[i]->hide ();
  }
  _hotel = new QLabel(_sideFrm);
  _hotel->setBackgroundColor (_color);
  _hotel->setAlignment(AlignCenter);
//    _hotel->setMask(_bithotel);
  _hotel->setPixmap(_pixhotel);
//    _hotel->hide();

  add_all_widgets();

  _popup->insertSeparator();
  _popup->insertItem( i18n("Buy/Sell &Houses On Monopoly"),
		      this, SLOT(slotHouses()), 
		      0, POPUPID_HOUSES );
//    _popup->insertItem( i18n("&Sell Houses On Monopoly"),
//  		      this, SLOT(slotSellHouses()), 
//  		      0, POPUPID_HOUSES_SELL );

}


StreetView::~StreetView()
{
}

PropertyDeed *StreetView::getDeed(KapitalistApp *app, QWidget *parent,
				  const int variant) const
{
  return new StreetDeed(app, parent, this, variant);
}



void StreetView::add_all_widgets()
{
  int i;

  _sideFrm->setBackgroundColor(_color);
  _mortgSign->setBackgroundColor(_color);

  switch (_orientation) {
  case BoardView::TOP: 
    _side->addWidget(_ownedSign, 1,0);
    _side->addMultiCellWidget(_mortgSign, 0,1,3,3);
    for (i=0; i<STREET_MAX_HOUSES; ++i)
      _side->addWidget(_house[i], 0,i);
    _side->addMultiCellWidget(_hotel, 0,1, 1,2); 
    break;
  case BoardView::BOTTOM:
    _side->addWidget(_ownedSign, 0,0);
    _side->addMultiCellWidget(_mortgSign, 0,1,3,3);
    for (i=0; i<STREET_MAX_HOUSES; ++i)
      _side->addWidget(_house[i], 1,i);
    _side->addMultiCellWidget(_hotel, 0,1, 1,2); 
    break;
  case BoardView::LEFT: 
    _side->addWidget(_ownedSign, 0,1);
    _side->addMultiCellWidget(_mortgSign, 3,3,0,1);
    for (i=0; i<STREET_MAX_HOUSES; ++i)
      _side->addWidget(_house[i], i,0);
    _side->addMultiCellWidget(_hotel, 1,2, 0,1); 
    break;
  case BoardView::RIGHT:
    _side->addWidget(_ownedSign, 0,0, AlignLeft & AlignTop);
    _side->addMultiCellWidget(_mortgSign, 3,3,0,1);
    for (i=0; i<STREET_MAX_HOUSES; ++i)
      _side->addWidget(_house[i], i,1);
    _side->addMultiCellWidget(_hotel, 1,2, 0,1); 
    break;

  default:
    assert(0);
  }

}


void StreetView::activate_layouts()
{
  _side->activate();
  _top->activate();
}


void StreetView::update_status()
{
  int i;
  int nhouses = _street->nhouses();

  // set mortgaged and owner sign appropriate

  bool mortgaged = _street->is_mortgaged();
  Player *owner = _street->owner();
  PlayerView *gowner = (owner) ? _view->plr(owner->id()) : NULL;
  
  if (mortgaged) _mortgSign->show();
  else _mortgSign->hide();

  if (gowner) {	// it's owned
    _ownedSign->setBackgroundColor(gowner->tokenColor());
    _ownedSign->show();
  } else {
    _ownedSign->hide();
  }

  // set houses/hotel appropriate
  if (nhouses > STREET_MAX_HOUSES) {	// we have a hotel
    _hotel->show();
    for (i=0; i<STREET_MAX_HOUSES; ++i) _house[i]->hide();
    return;
  } else {	// we have houses (or nothing)
    _hotel->hide();
    for (i=0; i<STREET_MAX_HOUSES; ++i) {
      if (nhouses > i) _house[i]->show();
      else _house[i]->hide();
    }
  }
//     _house[0]->show();
//    for (i=0; i<STREET_MAX_HOUSES; ++i) _house[i]->show();
//    _hotel->show();
}

void StreetView::slotHouses()
{
  _view->getApp()->slotActionHouses(_street->monop()->id());
}

//  void StreetView::slotSellHouses()
//  {
//    cout << "slotSellHouses()" << endl;
//  }

void StreetView::updatePopupMenu()
{
  int actionflags = _prop->actions();
  // Buy
  if (actionflags & CAN_BUY) 
    _popup->setItemEnabled(POPUPID_BUY, true);
  else
    _popup->setItemEnabled(POPUPID_BUY, false);

  // Mortgage
  if (actionflags & CAN_MORTGAGE) 
    _popup->setItemEnabled(POPUPID_MORTGAGE, true);
  else 
    _popup->setItemEnabled(POPUPID_MORTGAGE, false);
  // Unmortgage
  if (actionflags & CAN_UNMORTGAGE) 
    _popup->setItemEnabled(POPUPID_UNMORTGAGE, true);
  else 
    _popup->setItemEnabled(POPUPID_UNMORTGAGE, false);

  // Houses
  if (actionflags & CAN_BUILD_HOUSES || actionflags & CAN_SELL_HOUSES) 
    _popup->setItemEnabled(POPUPID_HOUSES, true);
  else 
    _popup->setItemEnabled(POPUPID_HOUSES, false);

}
