// -*- C++ -*-
/***************************************************************************
                          themedata.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//
// System
//

#include <iostream>
#include <cassert>

using namespace std;

//
// Qt / KDE
//

#include <qregexp.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <qcanvas.h>

#include <kstddirs.h>
#include <klocale.h>

//
// App
//

#include "global.h"

//
// Game
//

//
// View
//

#include "themedata.h"


ThemeMonopEntry::ThemeMonopEntry()
{
  color[0] = '\0';
  color_short = 'U';
}

ThemeFieldEntry::ThemeFieldEntry()
{
  name[0] = '\0';
  hyphname[0] = '\0';
}

ThemeCardEntry::ThemeCardEntry()
{
  msg[0] = '\0';
}

ThemeMoneyEntry::ThemeMoneyEntry()
{
  factor = 1.0;
  prefix = "$";
}


bool ThemeData::readThemeGeneralEntry(const QString &filename,
				      ThemeGeneralEntry &genEntry)
{
  QFile fFile( filename );

  if (!fFile.open( IO_ReadOnly )) {
    return false;
  }

  // parse config file
  int groupFlag = G_UNKNOWN;
  QString key, val;
  int fieldindex = 0;  // unused here
  bool foundGeneral=false, foundName=false, foundDesc=false;
  

  // read the file line by line with a QTextStream

  QTextStream t(&fFile);

  while ( !t.eof() ) {
    QString line = t.readLine();

    if ( !parseLine(line, groupFlag, key, val, fieldindex) ) continue;

    // if a group header was parsed, no key/value pair is available,
    // so skip key/value check.
    if (key==QString::null) {
      // if the group is 'general' we want to remember this
      if (groupFlag==G_GENERAL) foundGeneral=true;
      continue;
    }

    switch (groupFlag) {
    case G_GENERAL: 
      if (key == "name") {
	genEntry.name = val; // do nothing with it
	foundName = true;
      } else if (key == "description") {
	genEntry.description = val;  // do nothing with it
	foundDesc = true;
      } else {
	continue; // ignore any other keys
      }
      break;
    }
    
//     if (!parseLine(line, 
//     // strip whitespace at start and end of line
//     line = line.stripWhiteSpace();

//     // ignore comments and empty lines
//     if (line.startsWith("#") || line.length()==0)
//       continue;

//     // process group lines

//     QString currentGroup;

//     if ( line.startsWith("[") && line.endsWith("]") ) { // group starts

//       currentGroup = line.remove('[').remove(']');
//       cout << "group: " << currentGroup << endl;

//       // test if group matches "general"
//       if (currentGroup != "general") {
// 	ignoreData = true;
// 	continue;
//       } else {
// 	// group "general" found
// 	ignoreData = false;
// 	foundGeneral = true;
//       }
//       continue;

//     }

//     // process data within groups

//     // ignore data if requested before
//     if (ignoreData) continue;

//     // ignore lines without equal sign
//     if (!line.find('=')) continue;

//     QString key = line.section('=', 0, 0);
//     QString val = line.section('=', 1, -1);

//     key.stripWhiteSpace();
//     val.stripWhiteSpace();
    
//     cout << "key/value pair found: " << key << ": " << val << endl;

//     if (key == "name") {
//       genEntry.name = val;
//       // do nothing with it
//     } else if (key == "description") {
//       genEntry.description = val;
//     } else {
//       continue; // ignore any other keys
//     }

  }

  fFile.close();

  // we require to find a group "general" with a name and a description
  if (!foundGeneral || !foundName || !foundDesc) {
    return false;
  }

//   cout << "read of theme file successful:" << endl;
//   cout << "  name = " << genEntry.name << endl;
//   cout << "  description = " << genEntry.description << endl;
  
  return true;

}


ThemeData::ThemeData(const QString namesFilename, const QString boardFilename,
		     const QString diceFilename) 
  : _namesFname(namesFilename), _boardFname(boardFilename),
    _diceFname(diceFilename), _useable(false),
    _dice(0)
{
  // take default filenames when argument are Null
  if (!_namesFname) {
    _namesFname
      = locate("appdata", "themes/names/names.us.atlantic_city.conf");
    if (!_namesFname) {
      cerr << "ThemeData::ThemeData: " 
	   << "ERROR in locating default names theme file" << endl;
      return;
    }
  }
  if (!_boardFname) {
    _boardFname
      = locate("appdata", "themes/board/board.cartoon.default.conf");
    if (!_boardFname) {
      cerr << "ThemeData::ThemeData: " 
	   << "ERROR in locating default board theme file" << endl;
      return;
    }
  }
  if (!_diceFname) {
    _diceFname
      = locate("appdata", "themes/dice/dice.default.conf");
    if (!_diceFname) {
      cerr << "ThemeData::ThemeData: " 
	   << "ERROR in locating default dice theme file" << endl;
      return;
    }
  }

  preloadPixmaps();

  if(!loadNames()) {  // error loading names theme file
    cerr << "ThemeData::ThemeData: "
	 << "ERROR in loading names theme file"
	 << _namesFname << endl;
    return;
  }

  if(!loadBoard()) {  // error loading board theme files
    cerr << "ThemeData::ThemeData: "
	 << "ERROR in loading board theme file"
	 << _boardFname << endl;
    return;
  }

  if (!loadDice()) {  // error loading dice theme files
    cerr << "ThemeData::ThemeData: "
	 << "ERROR in loading dice theme file"
	 << _diceFname << endl;
    return;
  }

//   cout << "*****themedata constructed" << endl;
}

ThemeData::~ThemeData()
{
//   if (_dice) delete _dice; // is done by QCanvasSprite
//   cout << "*****themedata destroyed" << endl;
}


const ThemeMonopEntry *ThemeData::monop(int monopID) const
{
  assert(monopID >= 0 && monopID < BOARD_N_MONOPS);
  return &_monop[monopID];
}

const ThemeFieldEntry *ThemeData::field(int fieldID) const
{
  assert(fieldID >= 0 && fieldID < LAST_LOC); // LAST_LOC includes JAIL !!
  return &_field[fieldID];
}

const ThemeCardEntry *ThemeData::chance(int chanceID) const
{
  assert(chanceID >= 0 && chanceID < GAME_N_CHANCES);
  return &_chance[chanceID];
}

const ThemeCardEntry *ThemeData::cchest(int cchestID) const
{
  assert(cchestID >= 0 && cchestID < GAME_N_COMCHESTS);
  return &_cchest[cchestID];
}

const ThemeMoneyEntry *ThemeData::money() const
{
  return &_money;
}

const ThemeDiceAnimEntry *ThemeData::diceAnim() const
{
  return &_diceAnim;
}


QPixmap ThemeData::getPixmap(const QString &name) const
{
  assert(!name.isNull());

  if (name == "logo") return _pmLogo;

  else if (name == "info") return _pmInfo;
  else if (name == "cmd") return _pmCmd;
  else if (name == "query") return _pmQuery;

  else if (name == "rrtop") return _pmRRtop;
  else if (name == "rrbottom") return _pmRRbottom;
  else if (name == "rrleft") return _pmRRleft;
  else if (name == "rrright") return _pmRRright;
  else if (name == "rrshape") return _pmRRshape;
  else if (name == "rrowned") return _pmRRowned;
  else if (name == "rrmortg") return _pmRRmortg;

  else if (name == "electric") return _pmElectric;
  else if (name == "electricshape") return _pmElectricShape;
  else if (name == "electricowned") return _pmElectricOwned;
  else if (name == "electricmortg") return _pmElectricMortg;

  else if (name == "water") return _pmWater;
  else if (name == "watershape") return _pmWaterShape;
  else if (name == "waterowned") return _pmWaterOwned;
  else if (name == "watermortg") return _pmWaterMortg;

  else if (name == "jcardshape") return _pmJcardShape;
  else if (name == "jcardowned") return _pmJcardOwned;

  else if (name == "start") return _pmStart;
  else if (name == "jail") return _pmJail;
  else if (name == "car") return _pmCar;
  else if (name == "police") return _pmPolice;

  else if (name == "chance") return _pmChance;
  else if (name == "comchest") return _pmComchest;
  else if (name == "luxtax") return _pmLuxtax;

  else if (name == "mortgtop") return _pmMortgtop;
  else if (name == "mortgleft") return _pmMortgleft;
  else if (name == "mortgright") return _pmMortgright;

  else if (name == "house") return _pmHouse;
  else if (name == "hotel") return _pmHotel;
  else if (name == "houseadd") return _pmHouseAdd;
  else if (name == "hoteladd") return _pmHotelAdd;
  else if (name == "housedel") return _pmHouseDel;
  else if (name == "hoteldel") return _pmHotelDel;

  else if (name == "plrsprite") return _pmPlrsprite;

  else if (name == "rightmondir") return _pmRightMonDir;
  else if (name == "leftmondir") return _pmLeftMonDir;

  else if (name == "leftarrow") return _pmLeftArrow;
  else if (name == "rightarrow") return _pmRightArrow;
  else if (name == "refresh") return _pmRefresh;

  else if (name == "trade-accept") return _pmTradeAccept;
  else if (name == "trade-reject") return _pmTradeReject;
  else if (name == "trade-offer") return _pmTradeOffer;
  else if (name == "trade-undecided") return _pmTradeUndecided;

  else return QPixmap(locate("data", "unknown.png"));
//    else assert(0); // impossible case

}

QPixmap ThemeData::getPixmap(const int fieldID) const
{
  if (fieldID < 0 || fieldID > LAST_LOC-1)
    return QPixmap(locate("data", "unknown.png"));
  else
    return _pixmap[fieldID].pixmap;
}


QBitmap ThemeData::getBitmap(const QString &name) const
{
  assert(!name.isNull());
  if (name == "die-mask") return _pmDiemask;
  else if (name == "plrspritemask") return _pmPlrspritemask;

  else return QBitmap(locate("data", "unknown.png"));
//    else assert(0);
}

QPixmap ThemeData::getDiePixmap(const int i, const int j) const
{
  assert(i>=0 && i<6);
  assert(j>=0 && j<4);
  return _pmDie[i][j];
}

QCanvasPixmapArray *ThemeData::dice() const
{
  assert(_dice);
  return _dice;
}

bool ThemeData::parseLine(const QString &line, int &groupFlag, 
			  QString &key, QString &val, int &fieldindex)
{
  // initialize key and val to null
  key = val = QString::null;

  // strip whitespace at start and end of line
  QString str = line.stripWhiteSpace();

  // ignore comments and empty lines
  if (str.startsWith("#") || str.length()==0)
    return false;

  // process group lines

  QString currentGroup;
  // regular expression for field description groups
  QRegExp fielddesc("field_[0-9]+");

  if ( str.startsWith("[") && str.endsWith("]") ) { // group starts

    currentGroup = str.remove('[').remove(']');

    // set groupFlag according to the matched group

    if (currentGroup == "general") {
      
      groupFlag = G_GENERAL;

    } else if (currentGroup.find(fielddesc) >= 0) {
      // group matches field description

      groupFlag = G_FIELDDESC;
      // extract field number
      QRegExp num("[0-9]+");
      int numpos = currentGroup.find(num);
      assert(numpos); // group matched fielddesc, there must be a number
      fieldindex = currentGroup.mid(numpos).toInt();
      if (fieldindex < 0 || fieldindex >= LAST_LOC) {
	// no valid field index, treat group as UNKNOWN
	groupFlag = G_UNKNOWN;
      }

    } else if (currentGroup == "money") {

      groupFlag = G_MONEY;

    } else if (currentGroup == "dice_animation") {

      groupFlag = G_DICE_ANIM;

    } else { // group doesn't match any valid group

      groupFlag = G_UNKNOWN;
      fprintf(stderr, "ERROR: '%s' doesn't match any valid group,"
	      " ... ignored\n", currentGroup.latin1());
      return false;
    }

    // return success if a valid group was found
    return true;

  }

  // process data within groups

  // ignore lines without equal sign
  if (!str.find('=')) {
    cerr << QString("ERROR: invalid line '%1', ... ignored").arg(str.latin1())
	 << endl;
    return false;
  }

  key = str.section('=', 0, 0);
  val = str.section('=', 1, -1);

  key = key.stripWhiteSpace();
  val = val.stripWhiteSpace();
    
//   cout << "key/value pair found: " << key << ": " << val << endl;

  return true; // appropriate arguments are set and can be used by calling code
}

bool ThemeData::loadNames()
{
  // parse names config file

  QFile fFile( _namesFname );

  if (!fFile.open( IO_ReadOnly )) {
    return false;
  }

//   cout << "... load names from file " << _namesFname << endl;

  int groupFlag = G_UNKNOWN;
  QString key, val;
  int fieldindex = 0;

  int i;

  // read the file line by line with a QTextStream

  QTextStream t(&fFile);

  while ( !t.eof() ) {
    QString line = t.readLine();
    
    // parse line, ignore invalid lines
    if ( !parseLine(line, groupFlag, key, val, fieldindex) ) continue;

    // if a group header was parsed, no key/value pair is available,
    // so skip key/value check.
    if (key==QString::null) continue;
      
    switch (groupFlag) {
    case G_GENERAL: 
      if (key == "name") {
	_general.name = val;
	// do nothing with it
      } else if (key == "description") {
	_general.description = val;
      } else {
	// ignore any other keys
      }
      break;
    case G_FIELDDESC:
      if (key == "type") {
	// do nothing with it
      } else if (key == "name") {
	snprintf(_field[fieldindex].name, FIELD_MAX_LENGTH_NAME, "%s", 
		 val.latin1()); 
      } else if (key == "hyphname") {
	snprintf(_field[fieldindex].hyphname, FIELD_MAX_LENGTH_NAME, "%s", 
		 val.latin1()); 
      } else {
	// ignore any other keys
      }
      break;
    case G_MONEY:
      if (key == "factor") {
	_money.factor = QString(val).toDouble();
// 	cout << "... money factor: " << _money.factor << endl;
      } else if (key == "prefix") {
	_money.prefix = val;
	_money.prefix.truncate(3); // take only first 3 letters
      } else {
	// ignore any other keys
      }
      break;
    }
    
  }

  fFile.close();
  
  for (i=0; i<LAST_LOC; ++i) {
    if (_field[i].name[0] == 0) 
      sprintf(_field[i].name, "<unknown>");
    if (_field[i].hyphname[0] == 0) 
      sprintf(_field[i].hyphname, "%s", _field[i].name);
  }

  if (!_money.factor>0.0) _money.factor = 1.0; 
  if (!_money.prefix) _money.prefix = "$";

//    cout << "result" << endl << "============================" << endl;
//    cout << "Name: " << _general.name << endl;
//    cout << "Description: " << _general.description << endl;
//    for(i=0; i<LAST_LOC; ++i) {
//      cout << "Field " << i << endl;
//      cout << "  Name    : " << _field[i].name << endl;
//      cout << "  Hyphname: " << _field[i].hyphname << endl;
//    }

  return true;
}


bool ThemeData::loadBoard()
{
  // parse board config file

  QFile fFile( _boardFname );
  if (!fFile.open( IO_ReadOnly )) {
    return false;
  }
  QDir dir = QFileInfo(fFile).dir();

  int groupFlag = G_UNKNOWN;
  QString key, val;
  int fieldindex = 0;

  // read the file line by line with a QTextStream

  QTextStream t(&fFile);

  while ( !t.eof() ) {
    QString line = t.readLine();
    
    // parse line, ignore invalid lines
    if ( !parseLine(line, groupFlag, key, val, fieldindex) ) continue;

    // if a group header was parsed, no key/value pair is available,
    // so skip key/value check.
    if (key==QString::null) continue;

    switch (groupFlag) {
    case G_GENERAL: 
      if (key == "name") {
	_general.name = val;
	// do nothing with it
      } else if (key == "description") {
	_general.description = val;
      } else if (key == "subdir") {
	_general.subdir = val;
	if (!dir.cd(val)) {
	  cerr << "ThemeData::loadBoard: "
	       << "ERROR: cannot find subdir " << val
	       << "... ignored" << endl;
	}
      } else {
	// ignore any other keys
      }
      break;
    case G_FIELDDESC:
      if (key == "type") {
	// do nothing with it
      } else if (key == "pixmap") {
// 	cout << "... load pixmap[" << fieldindex << "] " << val;
	_pixmap[fieldindex].pixmap.load( dir.filePath(val) );
	if (_pixmap[fieldindex].pixmap.isNull()) {
	  cerr << "ThemeData::loadBoard: "
	       << "ERROR: cannot load pixmap " << dir.filePath(val) << endl;
	} else {
// 	  cout << " ... successful" << endl;
	}
      } else {
	// ignore any other keys
      }
      break;
    }
    
  }

  fFile.close();
  
//    for (i=0; i<LAST_LOC; ++i) {
//      if (_field[i].name[0] == 0) 
//        sprintf(_field[i].name, "<unknown>");
//      if (_field[i].hyphname[0] == 0) 
//        sprintf(_field[i].hyphname, "%s", _field[i].name);
//    }

//   cout << "result" << endl << "============================" << endl;
//   cout << "Name: " << _general.name << endl;
//   cout << "Description: " << _general.description << endl;
//   cout << "Subdir: " << _general.subdir << endl;

  return true;
}

bool ThemeData::loadDice()
{
  // parse dice config file

  QFile fFile( _diceFname );
  if (!fFile.open( IO_ReadOnly )) {
    return false;
  }
  QDir dir = QFileInfo(fFile).dir();

  int groupFlag = G_UNKNOWN;
  QString key, val;
  int fieldindex = 0;

  // read the file line by line with a QTextStream

  QTextStream t(&fFile);

  while ( !t.eof() ) {
    QString line = t.readLine();
    
    // parse line, ignore invalid lines
    if ( !parseLine(line, groupFlag, key, val, fieldindex) ) continue;

    // if a group header was parsed, no key/value pair is available,
    // so skip key/value check.
    if (key==QString::null) continue;
    
    switch (groupFlag) {
    case G_GENERAL: 
      if (key == "name") {
	_general.name = val;
	// do nothing with it
      } else if (key == "description") {
	_general.description = val;
      } else if (key == "subdir") {
	_general.subdir = val;
	if (!dir.cd(val)) {
	  cerr << "ThemeData::loadDice: "
	       << "ERROR: cannot find subdir " << val
	       << "... ignored" << endl;
	}
      } else if (key == "filenamebase") {
	_general.filenamebase = val;
      } else if (key == "filetype") {
	_general.filetype = val;
      } else {
	// ignore any other keys
      }
      break;
    case G_DICE_ANIM:
      bool conversion_ok = true;
      if (key == "nframes") {
	_diceAnim.nframes = val.toInt(&conversion_ok);
      } else if (key == "frameindex1") {
	_diceAnim.frameindex[1] = val.toInt(&conversion_ok);
      } else if (key == "frameindex2") {
	_diceAnim.frameindex[2] = val.toInt(&conversion_ok);
      } else if (key == "frameindex3") {
	_diceAnim.frameindex[3] = val.toInt(&conversion_ok);
      } else if (key == "frameindex4") {
	_diceAnim.frameindex[4] = val.toInt(&conversion_ok);
      } else if (key == "frameindex5") {
	_diceAnim.frameindex[5] = val.toInt(&conversion_ok);
      } else if (key == "frameindex6") {
	_diceAnim.frameindex[6] = val.toInt(&conversion_ok);
      } else {
	// ignore any other keys
      }
      if (!conversion_ok) {
	cerr << "ThemeData::loadDice: "
	     << "ERROR: cannot convert string to integer: " << val
	     << "... ignored" << endl;
      }
      break;
    }
    
  }

  // load pixmaps for dice
  QString fnamepattern = dir.filePath(_general.filenamebase + ".%1." + 
				      _general.filetype);
  QCanvasPixmapArray *olddice = _dice;
  _dice = new QCanvasPixmapArray( fnamepattern, _diceAnim.nframes );
  if (!(*_dice)) {
    cerr << "ThemeData::loadDice: "
	 << "ERROR: cannot load dice pixmaps, fnamepattern: "
	 << fnamepattern << endl;
    _dice = olddice;
    return false;
  }

  // check dice animation variables
  
  if (_diceAnim.nframes < 1 || _diceAnim.nframes > 256) {
    cerr << "ThemeData::loadDice: "
	 << "ERROR: nframes out of range (assert 1<=nframes<=256)"
	 << "... set to 1" << endl;
    _diceAnim.nframes = 1;
  }
  for (int i=1; i<=6; ++i) {
    if (_diceAnim.frameindex[i] < 0 || 
	_diceAnim.frameindex[i] > _diceAnim.nframes-1) {
      cerr << "ThemeData::loadDice: "
	   << "ERROR: frameindex" << i << " out of range, "
	   << "(assert 0<=frameindex<nframes) "
	   << "... set to 0" << endl;
      _diceAnim.frameindex[i] = 0;
    }
  }

  fFile.close();
  
//    for (i=0; i<LAST_LOC; ++i) {
//      if (_field[i].name[0] == 0) 
//        sprintf(_field[i].name, "<unknown>");
//      if (_field[i].hyphname[0] == 0) 
//        sprintf(_field[i].hyphname, "%s", _field[i].name);
//    }

//   cout << "result" << endl << "============================" << endl;
//   cout << "name: " << _general.name << endl;
//   cout << "description: " << _general.description << endl;
//   cout << "subdir: " << _general.subdir << endl;
//   cout << "filename pattern: " << fnamepattern << endl;
//   cout << "nframes: " << _diceAnim.nframes << endl;

  return true;
}

//
// protected
//
void ThemeData::preloadPixmaps()
{
  // variables needed for dice
  int i,j;
  char filename[20];
  int index[6][4] = {  {2,3,4,5}, {1,3,4,6}, {1,2,5,6}, 
		     {1,2,5,6}, {1,3,4,6}, {2,3,4,5} };
  

  // logo pixmap
  _pmLogo.load(locate("appdata", "pics/kapitalist-logo.png"));

  // die pixmaps
  for(i=0; i<6; ++i) {
    for (j=0; j<4; ++j) {
      sprintf(filename, "pics/die-%d+%d.gif", i+1, index[i][j]);
      _pmDie[i][j].load(locate("appdata", filename));
    }
  }
  _pmDiemask.load(locate("appdata", "pics/die-mask.xpm"));
  
  // gameinfo pixmaps
  _pmInfo.load(locate("appdata", "pics/info.xpm"));
  _pmCmd.load(locate("appdata", "pics/cmd.xpm"));
  _pmQuery.load(locate("appdata", "pics/query.xpm"));
  
  // railroad pixmaps
  _pmRRtop.load(locate("appdata", "pics/rrtop.xpm"));
  _pmRRbottom.load(locate("appdata", "pics/rrbottom.xpm"));
  _pmRRleft.load(locate("appdata", "pics/rrleft.xpm"));
  _pmRRright.load(locate("appdata", "pics/rrright.xpm"));
  _pmRRshape.load(locate("appdata", "pics/rrshape.png"));
  _pmRRowned.load(locate("appdata", "pics/rrowned.png"));
  _pmRRmortg.load(locate("appdata", "pics/rrmortg.png"));

  // utility pixmaps
  _pmElectric.load(locate("appdata", "pics/electric.xpm"));
  _pmElectricShape.load(locate("appdata", "pics/electricshape.png"));
  _pmElectricOwned.load(locate("appdata", "pics/electricowned.png"));
  _pmElectricMortg.load(locate("appdata", "pics/electricmortg.png"));
  
  _pmWater.load(locate("appdata", "pics/water.xpm"));
  _pmWaterShape.load(locate("appdata", "pics/watershape.png"));
  _pmWaterOwned.load(locate("appdata", "pics/waterowned.png"));
  _pmWaterMortg.load(locate("appdata", "pics/watermortg.png"));
  
  // jailcard pixmaps
  _pmJcardShape.load(locate("appdata", "pics/jcardshape.png"));
  _pmJcardOwned.load(locate("appdata", "pics/jcardowned.png"));

  // corner fields pixmaps
  _pmStart.load(locate("appdata", "pics/start.xpm"));
  _pmJail.load(locate("appdata", "pics/jail.xpm"));
  _pmCar.load(locate("appdata", "pics/car.xpm"));
  _pmPolice.load(locate("appdata", "pics/police.xpm"));

  // special fields pixmaps (chance, comchest, luxtax)
  _pmChance.load(locate("appdata", "pics/chance.xpm"));
  _pmComchest.load(locate("appdata", "pics/comchest.xpm"));
  _pmLuxtax.load(locate("appdata", "pics/luxtax.xpm"));

  // property fields pixmaps
  _pmMortgtop.load(locate("appdata", "pics/mortgtop.xpm"));
  _pmMortgleft.load(locate("appdata", "pics/mortgleft.xpm"));
  _pmMortgright.load(locate("appdata", "pics/mortgright.xpm"));
  
  _pmHouse.load(locate("appdata", "pics/house.xpm"));
  _pmHotel.load(locate("appdata", "pics/hotel.xpm"));
  _pmHouseAdd.load(locate("appdata", "pics/houseadd.png"));
  _pmHotelAdd.load(locate("appdata", "pics/hoteladd.png"));
  _pmHouseDel.load(locate("appdata", "pics/housedel.png"));
  _pmHotelDel.load(locate("appdata", "pics/hoteldel.png"));
  
  // player sprite pixmap
  _pmPlrsprite.load(locate("appdata", "pics/plrsprite.xbm"));
  _pmPlrspritemask.load(locate("appdata", "pics/plrspritemask.xbm"));

  _pmRightMonDir.load(locate("appdata", "pics/rightMonDir.png"));
  _pmLeftMonDir.load(locate("appdata", "pics/leftMonDir.png"));
  
//    _pmRightArrow.load(locate("appdata", "pics/rightarrow.png"));
//    _pmLeftArrow.load(locate("appdata", "pics/leftarrow.png"));

  _pmRightArrow.load(locate("appdata", "pics/rightarrow.png"));
  _pmLeftArrow.load(locate("appdata", "pics/leftarrow.png"));
  _pmRefresh.load(locate("appdata", "pics/refresh.png"));

  _pmTradeAccept.load(locate("appdata", "pics/trade-accept.png"));
  _pmTradeReject.load(locate("appdata", "pics/trade-reject.png"));
  _pmTradeOffer.load(locate("appdata", "pics/trade-offer.png"));
  _pmTradeUndecided.load(locate("appdata", "pics/trade-undecided.png"));
  
  // TODO: use rotate for some pixmaps (esp. railroads)

//  	if (name.isNull())
//  		return 0;

//  	QPixmap *icon = new QPixmap(locate("data", "kmonop/pics/" + name));
//  	QWMatrix m;

//  	switch(_orientation)
//  	{
//  		case East:
//  			m.rotate(90);
//  			break;
//  		case West:
//  			m.rotate(-90);
//  			break;
//  		case South:
//  			m.rotate(180);
//  			break;
//  	}
//  	*icon = icon->xForm(m);
//  	return icon;


}
