// -*- C++ -*-
/***************************************************************************
                          themedata.h  -  description
                             -------------------
    begin                : Mon Nov 1 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef THEMEDATA_H
#define THEMEDATA_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

//
// Qt
//

#include <qpixmap.h>
#include <qbitmap.h>

class QCanvasPixmapArray;

//
// App
//

//
// Game
//

#include "board.h"

//
// View
//


struct ThemeGeneralEntry {
  QString name;
  QString description;
  QString subdir;
  QString filenamebase;
  QString filetype;
};

// TODO: replace char[] by QString

struct ThemeMonopEntry {
  ThemeMonopEntry();
  char color[MONOP_MAX_LENGTH_COLOR];
  char color_short;
};

struct ThemeFieldEntry {
  ThemeFieldEntry();
  char name[FIELD_MAX_LENGTH_NAME];
  char hyphname[FIELD_MAX_LENGTH_NAME];
};

struct ThemeCardEntry {
  ThemeCardEntry();
  char msg[MSG_SIZE];
};

struct ThemeMoneyEntry {
  ThemeMoneyEntry();
  double factor;
  QString prefix;
};

struct ThemePixmapEntry {
  QString filename;
  QPixmap pixmap;
};

struct ThemeDiceAnimEntry {
  int nframes;
  int frameindex[7];
};
  
//  static struct monop_locale_struct monop_locale[BOARD_N_MONOPS] = 
//  #include "monops.us.locale"
//  ;

//  static struct field_locale_struct field_locale_raw[LAST_LOC] =
//  #include "fields.us.locale"
//  ;				 

//  static struct field_locale_struct field_locale[LAST_LOC] =
//  #include "fields.us.locale"
//  ;				 

//  static struct card_locale_struct cchest_card_locale[GAME_N_COMCHESTS] =
//  #include "cchest_cards.us.locale"
//  ;				 
//  static struct card_locale_struct chance_card_locale[GAME_N_CHANCES] =
//  #include "chance_cards.us.locale"
//  ;				 



class ThemeData
{
public:
  
  static bool readThemeGeneralEntry(const QString &filename, 
				    ThemeGeneralEntry &genEntry);

public:
  ThemeData(const QString namesFilename=QString::null,
	    const QString boardFilename=QString::null,
	    const QString diceFilename=QString::null);
  virtual ~ThemeData();

  bool isUseable() { return _useable; }

  const ThemeMonopEntry *monop(int monopID) const;
  const ThemeFieldEntry *field(int fieldID) const;
  const ThemeCardEntry *chance(int chanceID) const;
  const ThemeCardEntry *cchest(int cchestID) const;
  
  const ThemeMoneyEntry *money() const;

  const ThemePixmapEntry *pixmap(int fieldID) const;

  const ThemeDiceAnimEntry *diceAnim() const;

  QPixmap getPixmap(const QString &name) const;
  QPixmap getPixmap(const int fieldID) const;

  QPixmap getDiePixmap(const int i, const int j) const;
  QBitmap getBitmap(const QString &name) const;

  QCanvasPixmapArray *dice() const;

protected:

  /*
   * groups for theme file parsing
   */
  enum {
    G_UNKNOWN=0,
    G_GENERAL=1,
    G_FIELDDESC=2,
    G_MONEY=4,
    G_DICE_ANIM=8
  };


  QString _namesFname;
  QString _boardFname;
  QString _diceFname;

  bool _useable;
  
  // TODO: _general is written partly by loadNames, loadBoard and loadDice
  //       and result is unpredictable
  ThemeGeneralEntry _general;

  ThemeMonopEntry _monop[BOARD_N_MONOPS];
  ThemeFieldEntry _field[LAST_LOC]; // JAIL included
  ThemeCardEntry _chance[GAME_N_CHANCES];
  ThemeCardEntry _cchest[GAME_N_COMCHESTS];

  ThemeMoneyEntry _money;

  ThemePixmapEntry _pixmap[LAST_LOC];

  ThemeDiceAnimEntry _diceAnim;

  // logo pixmap
  QPixmap _pmLogo;

  // die pixmaps
  QPixmap _pmDie[6][4];
  QBitmap _pmDiemask;

  // gameinfo pixmaps
  QPixmap _pmInfo;
  QPixmap _pmCmd;
  QPixmap _pmQuery;

  // railroad pixmaps
  QPixmap _pmRRtop;
  QPixmap _pmRRbottom;
  QPixmap _pmRRleft;
  QPixmap _pmRRright;
  QPixmap _pmRRshape;
  QPixmap _pmRRowned;
  QPixmap _pmRRmortg;

  // utility pixmaps
  QPixmap _pmElectric;
  QPixmap _pmElectricShape;
  QPixmap _pmElectricOwned;
  QPixmap _pmElectricMortg;

  QPixmap _pmWater;
  QPixmap _pmWaterShape;
  QPixmap _pmWaterOwned;
  QPixmap _pmWaterMortg;

  // jailcard pixmaps
  QPixmap _pmJcardShape;
  QPixmap _pmJcardOwned;

  // corner fields pixmaps
  QPixmap _pmStart;
  QPixmap _pmJail;
  QPixmap _pmCar;
  QPixmap _pmPolice;

  // special fields pixmaps (chance, comchest, luxtax)
  QPixmap _pmChance;
  QPixmap _pmComchest;
  QPixmap _pmLuxtax;

  // property fields pixmaps
  QPixmap _pmMortgtop;
  QPixmap _pmMortgleft;
  QPixmap _pmMortgright;

  QPixmap _pmHouse;
  QPixmap _pmHotel;
  QPixmap _pmHouseAdd;
  QPixmap _pmHotelAdd;
  QPixmap _pmHouseDel;
  QPixmap _pmHotelDel;

  // player sprite pixmap
  QPixmap _pmPlrsprite;
  QBitmap _pmPlrspritemask;

  QPixmap _pmRightMonDir;
  QPixmap _pmLeftMonDir;

  QPixmap _pmRightArrow;
  QPixmap _pmLeftArrow;
  QPixmap _pmRefresh;

  QPixmap _pmTradeAccept;
  QPixmap _pmTradeReject;
  QPixmap _pmTradeOffer;
  QPixmap _pmTradeUndecided;

  QCanvasPixmapArray *_dice;

protected:

  /**
   * load the theme from files, street names + monopolies
   */
  bool loadNames();
  bool loadBoard();
  bool loadDice();

  void preloadPixmaps();

  /**
   * parse one line in a configuration file (KDE style config syntax)
   * 
   * This method returns true, if a valid line was found, either a
   * group header line (syntax: "[somegroup]") or a key-value-line
   * (syntax: "key=value"). If a group header line was found, key and
   * val are set to QString::null.
   */
  static bool parseLine(const QString &line, int &groupFlag,
			QString &key, QString &val, int &fieldindex); 

};

#endif // THEMEDATA_H
