// -*- C++ -*-
/***************************************************************************
                          tradeview.cpp  -  description
                             -------------------
    begin                : Mon Jul  9 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// TODO: remove this
//  #include <iostream.h>

//
// Qt
//

#include <kdialogbase.h>
#include <klocale.h>
#include <knuminput.h>

#include <qlayout.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qtooltip.h>

#include <qpushbutton.h>

//
// App
//

#include "kapitalist.h"
#include "global.h"
#include "events.h"

//
// Game
//

#include "kapgame.h"
#include "property.h"
#include "trade.h"

//
// View
//

#include "kapview.h"
#include "kapitalist.h"
#include "propertydeed.h"
#include "themedata.h"

//  #include "tradedialog.h"
#include "tradeview.h"


TradeView::TradeView( KapitalistApp *app, QWidget *parent, Player *dest)
  : QFrame(parent), _app(app), _dest(dest),
    _giveMoney(true)
{
  int i, index;

  assert(_dest);
  assert(!_app->game()->nobody(_dest->id()));
  _me = _app->game()->myPlr();

  _trade = _me->get_trade_with(_dest);  // can be 0 !!

  // TODO: update trade after reconnecting

  _top = new QGridLayout( this, 6,5, 10 );

  _top->setRowStretch(1,1);
  _top->addRowSpacing(1,200);
  _top->addRowSpacing(4,20);

  _top->addColSpacing(0,150);
  _top->addColSpacing(2,150);
  _top->addColSpacing(4,150);
  _top->addColSpacing(1,10);
  _top->addColSpacing(3,10);
  _top->setColStretch(0,1);
  _top->setColStretch(2,1);
  _top->setColStretch(4,1);

  _myHeadLab = new QLabel( this );
  _myHeadLab->setText(i18n("Your properties"));
  _top->addWidget( _myHeadLab, 0,0, AlignCenter );

  _offerLab = new QLabel( this );
  _offerLab->setText(i18n("Offer"));
  _top->addWidget( _offerLab, 0,2, AlignCenter );

  _yourHeadLab = new QLabel( this );
  _yourHeadLab->setText(QString(_dest->name()) 
			+ i18n("'s properties"));
  _top->addWidget( _yourHeadLab, 0,4, AlignCenter );

  _myPropsBox = new QListBox( this );
  _myPropsBox->setRowMode( QListBox::Variable );
  _myPropsBox->setSelectionMode( QListBox::Multi );
  for (index=0, i=0; i<BOARD_N_FIELDS; ++i) {
    if (_me->holdings(i)) { // I own the property
      _myPropsBox->insertItem(QString(_app->view()->field_locale(i)->name));
      _myPropsIDByIndex[index++] = i;
    }
  }
  _top->addWidget( _myPropsBox, 1,0 );

  _offerBox = new QListBox( this );
  _offerBox->setRowMode( QListBox::Variable );
  _offerBox->setSelectionMode( QListBox::Multi );
//    for (i=0; i<BOARD_N_FIELDS; ++i) {
//      _offer[i] = -1;  // initialize all props not to be involved in offer
//    }
  _top->addWidget( _offerBox, 1,2 );

  _yourPropsBox = new QListBox( this );
  _yourPropsBox->setRowMode( QListBox::Variable );
  _yourPropsBox->setSelectionMode( QListBox::Multi );
  for (index=0, i=0; i<BOARD_N_FIELDS; ++i) {
    if (_dest->holdings(i)) { // you own the property
      _yourPropsBox->insertItem(QString(_app->view()->field_locale(i)->name));
      _yourPropsIDByIndex[index++] = i;
    }
  }
  _top->addWidget( _yourPropsBox, 1,4 );

  QVBox *myToOffer = new QVBox( this );
  _top->addWidget( myToOffer, 1,1 );

  _myAddBut = new QPushButton( myToOffer );
  _myAddBut->setPixmap( _app->view()->getPixmap("rightarrow"));
  _myDelBut = new QPushButton( myToOffer );
  _myDelBut->setPixmap( _app->view()->getPixmap("leftarrow"));
  _myRefreshBut = new QPushButton( myToOffer );
  _myRefreshBut->setPixmap( _app->view()->getPixmap("refresh"));

  QVBox *youToOffer = new QVBox( this );
  _top->addWidget( youToOffer, 1,3 );

  _yourAddBut = new QPushButton( youToOffer );
  _yourAddBut->setPixmap( _app->view()->getPixmap("leftarrow"));
  _yourDelBut = new QPushButton( youToOffer );
  _yourDelBut->setPixmap( _app->view()->getPixmap("rightarrow"));
  _yourRefreshBut = new QPushButton( youToOffer );
  _yourRefreshBut->setPixmap( _app->view()->getPixmap("refresh"));

  _myBudgetLab = new QLabel( this );
  _myBudgetLab->setText(i18n("Budget: $") 
			+ QString().setNum(_me->budget()));
  _top->addWidget( _myBudgetLab, 2,0, AlignCenter );

  _yourBudgetLab = new QLabel( this );
  _yourBudgetLab->setText(i18n("Budget: $") 
			  + QString().setNum(_dest->budget()));
  _top->addWidget( _yourBudgetLab, 2,4, AlignCenter );

  _moneyDirection = new QPushButton( this );
  _moneyDirection->setPixmap(_app->view()->getPixmap("rightmondir"));
  _top->addWidget( _moneyDirection, 2,2, AlignCenter );

  _moneyEdit = new KIntNumInput( this );
  _moneyEdit->setPrefix("$");
  _moneyEdit->setSuffix("  ");
  _moneyEdit->setRange(0, GAME_MAX_MONEY, 10, false);
  _top->addWidget( _moneyEdit, 3,2, AlignCenter );

  QHBox *but = new QHBox( this );
  _top->addMultiCellWidget( but, 5,5, 1,3, AlignCenter );

  _offerBut = new QPushButton( but );
  _offerBut->setText( i18n("Offer") );
  _acceptBut = new QPushButton( but );
  _acceptBut->setText( i18n("Accept") );
  _rejectBut = new QPushButton( but );
  _rejectBut->setText( i18n("Reject") );


  _myOfferLab = new QLabel(this);
  _myOfferLab->setPixmap(_app->view()->getPixmap("trade-offer"));
  _top->addWidget( _myOfferLab, 4,0, AlignCenter );

  _myAcceptLab = new QLabel(this);
  _myAcceptLab->setPixmap(_app->view()->getPixmap("trade-accept"));
  _top->addWidget( _myAcceptLab, 4,0, AlignCenter );

  _myRejectLab = new QLabel(this);
  _myRejectLab->setPixmap(_app->view()->getPixmap("trade-reject"));
  _top->addWidget( _myRejectLab, 4,0, AlignCenter );

  _myUndecidedLab = new QLabel(this);
  _myUndecidedLab->setPixmap(_app->view()->getPixmap("trade-undecided"));
  _top->addWidget( _myUndecidedLab, 4,0, AlignCenter );

  _yourOfferLab = new QLabel(this);
  _yourOfferLab->setPixmap(_app->view()->getPixmap("trade-offer"));
  _top->addWidget( _yourOfferLab, 4,4, AlignCenter );

  _yourAcceptLab = new QLabel(this);
  _yourAcceptLab->setPixmap(_app->view()->getPixmap("trade-accept"));
  _top->addWidget( _yourAcceptLab, 4,4, AlignCenter );

  _yourRejectLab = new QLabel(this);
  _yourRejectLab->setPixmap(_app->view()->getPixmap("trade-reject"));
  _top->addWidget( _yourRejectLab, 4,4, AlignCenter );

  _yourUndecidedLab = new QLabel(this);
  _yourUndecidedLab->setPixmap(_app->view()->getPixmap("trade-undecided"));
  _top->addWidget( _yourUndecidedLab, 4,4, AlignCenter );

  _top->activate();


  QToolTip::add(_myAddBut, "Add selected properties to trade offer" );
  QToolTip::add(_myDelBut, "Remove selected properties from trade offer");
  QToolTip::add(_myRefreshBut, "Refresh your holdings and budget on the left" );

  QToolTip::add(_yourAddBut, "Add selected properties to trade offer" );
  QToolTip::add(_yourDelBut, "Remove selected properties from trade offer");
  QToolTip::add(_yourRefreshBut, "Refresh " + QString(_dest->name()) +
		"'s holdings and budget on the right" );
  
  QToolTip::add(_moneyDirection, "Choose if you want to give or to get money");

  QToolTip::add(_myAcceptLab, "You accepted the trade offer");
  QToolTip::add(_myRejectLab, "You rejected the trade offer");
  QToolTip::add(_myOfferLab, "Your trade offer is displayed");
  QToolTip::add(_myUndecidedLab, "You haven't decided yet");

  QToolTip::add(_yourAcceptLab, QString(_dest->name()) + 
		" accepted the trade offer");
  QToolTip::add(_yourRejectLab, QString(_dest->name()) +
		" rejected the trade offer");
  QToolTip::add(_yourOfferLab, QString(_dest->name()) +
		"'s trade offer is displayed");
  QToolTip::add(_yourUndecidedLab, QString(_dest->name()) +
		" hasn't decided yet");
  

  QObject::connect(_myAddBut, SIGNAL(clicked()), 
		   this, SLOT(slotMyAddBut()));
  QObject::connect(_myDelBut, SIGNAL(clicked()), 
  		   this, SLOT(slotMyDelBut()));
  QObject::connect(_myRefreshBut, SIGNAL(clicked()), 
  		   this, SLOT(slotMyRefreshBut()));

  QObject::connect(_yourAddBut, SIGNAL(clicked()), 
		   this, SLOT(slotYourAddBut()));
  QObject::connect(_yourDelBut, SIGNAL(clicked()), 
  		   this, SLOT(slotYourDelBut()));
  QObject::connect(_yourRefreshBut, SIGNAL(clicked()), 
  		   this, SLOT(slotYourRefreshBut()));

  QObject::connect(_moneyDirection, SIGNAL(clicked()),
		   this, SLOT(slotToggleMoneyDirection()));
  QObject::connect(_moneyEdit, SIGNAL(valueChanged(int)),
		   this, SLOT(slotMoneyChanged(int)));

//    QObject::connect(_myPropsBox, SIGNAL(selected(int)),
//  		   this, SLOT(slotTmp(int)));

  QObject::connect(_offerBut, SIGNAL(clicked()),
		   this, SLOT(slotOffer()));
  QObject::connect(_acceptBut, SIGNAL(clicked()),
		   this, SLOT(slotAccept()));
  QObject::connect(_rejectBut, SIGNAL(clicked()),
		   this, SLOT(slotReject()));

  slotEnableButtons(T_BUT_NONE);
  slotShowMyIcon(T_ICO_UNDECIDED);
}

TradeView::~TradeView()
{
}


void TradeView::slotOffer()
{
  struct packet_trade pack;

  unsigned int i;
  int mine;
  int yours;
  Trade *trade;
  bool new_trade;
  int prop[BOARD_N_FIELDS];

  trade = _me->get_trade_with(_dest);
  new_trade = (trade)? false:true;

  for (i=0; i<BOARD_N_FIELDS; ++i)
    prop[i] = T_NOTRADE;

  for (i=0; i<_offerBox->count(); ++i) {
    prop[_offerPropsIDByIndex[i]] = 1;
  }
  
  pack.tradeid = 0;	// not used
  pack.event = (new_trade)? E_TRADE_OFFER : E_TRADE_REPLY;

  pack.srcid = -1;	// not used on server side
  pack.destid = _dest->id();

  pack.offer_from = T_NOTRADE;	// not used!!!

  pack.money = _moneyEdit->value() * ((_giveMoney)? 1:-1);


  // find out, who is dest and who is src of trade and change
  // sign of money accordingly
  // TODO: remove this pain in the ass (here and in chatline.cpp)
  if (new_trade) {	// there is no trade yet
    mine = T_SRC;
    yours = T_DEST;
  } else {
    mine = (trade->get_src() == _me)? T_SRC : T_DEST;
    yours = (trade->get_src() == _me)? T_DEST : T_SRC;
  }

  // change sign of money, if we are dest
  if (mine == T_DEST) {	// I am dest
    pack.money = -pack.money;
  }
	
  for (i=0; i<BOARD_N_FIELDS; ++i)
    pack.prop[i] = prop[i];

  pack.cards = 0;			// not used yet

  send_packet_trade(_app->conn(), &pack);

}

void TradeView::slotAccept()
{
  struct packet_trade pack;
  pack.event = E_TRADE_AGREE;
  pack.destid = _dest->id();

  send_packet_trade(_app->conn(), &pack);
}

void TradeView::slotReject()
{
  struct packet_trade pack;
  pack.event = E_TRADE_REJECT;
  pack.destid = _dest->id();

  send_packet_trade(_app->conn(), &pack);
}

void TradeView::update(struct packet_trade *pack)
{
  assert(pack);

  int event = pack->event;
  int myPID = _app->game()->myPID();

  bool meSrc = (pack->srcid == myPID);
  bool meDest = !meSrc;
  bool myOffer = ( (pack->offer_from==T_SRC && meSrc) ||
		   (pack->offer_from==T_DEST && meDest) );
  bool meActor = 
    ( ( (event==E_TRADE_OFFER || event==E_TRADE_REPLY) && myOffer ) ||
      ( (event==E_TRADE_AGREE || event==E_TRADE_REJECT) && !myOffer ) ||
      ( (event==E_TRADE_FINISHED) && !myOffer ) );
  
  _trade = _me->get_trade_with(_dest);	// this is always up to date

  slotDisableButtons(T_BUT_ALL);
  
  switch (event) {
  case E_TRADE_OFFER:
  case E_TRADE_REPLY:
    assert(_trade);
    // update _offerBox and money
    // enable/disable buttons and show/hide icons
    updateOffer(pack);
    if (meActor) {
      slotEnableButtons(T_BUT_REJECT);
      slotShowMyIcon(T_ICO_OFFER);
      slotShowYourIcon(T_ICO_UNDECIDED);
    } else {
      slotEnableButtons(T_BUT_REJECT | T_BUT_AGREE);
      slotShowMyIcon(T_ICO_UNDECIDED);
      slotShowYourIcon(T_ICO_OFFER);
    }
    // TODO: icons
    break;
  case E_TRADE_AGREE:
    // not used by server
    break;
  case E_TRADE_REJECT:
//      updateOffer(pack);
    if (meActor) slotShowMyIcon(T_ICO_REJECT);
    else slotShowYourIcon(T_ICO_REJECT);
    break;
  case E_TRADE_FINISHED:
    updateOffer(pack);
    updateMyHoldings();  // TODO: this is too early here because
    updateYourHoldings(); // player_info hasn't arrived yet !!
    if (meActor) slotShowMyIcon(T_ICO_ACCEPT);
    else slotShowYourIcon(T_ICO_ACCEPT);
    break;
  case E_NOEVENT:
    break;
  default:
    assert(0);
  }

}

void TradeView::updateOffer(struct packet_trade *pack)
{
  int i, index;
  int money;

  _offerBox->clear();
  for (index=0, i=0; i<BOARD_N_FIELDS; ++i) {
    if (_me->holdings(i) && pack->prop[i] != T_NOTRADE) {
      // I own the property and it's involved in the trade
      _offerBox->insertItem(QString("Give: ") 
			    + QString(_app->view()->field_locale(i)->name));
      _offerPropsIDByIndex[index++] = i;
    }
    if (_dest->holdings(i) && pack->prop[i] != T_NOTRADE) {
      // dest owns the property and it's involved in the trade
      _offerBox->insertItem(QString("Get: ") 
			    + QString(_app->view()->field_locale(i)->name));
      _offerPropsIDByIndex[index++] = i;
    }
  }

  // update money

  if (_me->id() == pack->srcid) // I'm source
    money = pack->money;
  else  // I'm dest
    money = - pack->money;

  // if money direction is wrong, toggle it
  if ( (_giveMoney && money<=0) ||
       (!_giveMoney && money>=0) )
    slotToggleMoneyDirection();
  
  _moneyEdit->setValue(( (money>=0) ? money : - money ));
  
}

void TradeView::updateMyHoldings()
{
//    cout << "updateMyHoldings()" << endl;
  updateHoldings(_me);
}

void TradeView::updateYourHoldings()
{
//    cout << "updateYourHoldings()" << endl;
  updateHoldings(_dest);
}

void TradeView::updateHoldings(const Player *plr)
{
  int i, index;
  
  assert(plr == _me || plr == _dest);
  bool my = (plr == _me);

  QListBox *box = ( my ? _myPropsBox : _yourPropsBox );
  QLabel *budget = ( my ? _myBudgetLab : _yourBudgetLab );
  int *IDByIndex = ( my ? _myPropsIDByIndex : _yourPropsIDByIndex );

  box->clear();
  for (index=0, i=0; i<BOARD_N_FIELDS; ++i) {
    if (plr->holdings(i)) { // I/You own the property
      box->insertItem(QString(_app->view()->field_locale(i)->name));
      IDByIndex[index++] = i;
    }
  }
  
  budget->setText(i18n("Budget: $") + QString().setNum(plr->budget()));

}

//
// protected slots
//

void TradeView::slotMyAddBut()
{
  unsigned int i,j;
  int propid;
  bool cont = false;

  _offerBox->clearSelection();  // newly added props will be selected

  // copy selected items in myPropsBox into offerBox

  for (i=0; i<_myPropsBox->count(); ++i) {

    cont=false;
    propid = _myPropsIDByIndex[i];
    
    // test if prop is selected and if it isn't already included in offer
    if (!_myPropsBox->isSelected(i)) cont = true;  // item is not selected
    for (j=0; j<_offerBox->count(); ++j) {	// propid already in offer
//        cout << "  " << _offerPropsIDByIndex[j] << endl;
      if (_offerPropsIDByIndex[j] == propid) cont = true;
    }
    if (cont) continue;

    QListBoxText *item = new QListBoxText(QString("Give: ") 
					  + _myPropsBox->item(i)->text());
    _offerBox->insertItem(item);
    _offerBox->setSelected(item, true);
    _offerPropsIDByIndex[_offerBox->count()-1] = propid;
//      cout << "added " << _myPropsIDByIndex[i] << " to offer" << endl;
    // something changed, so enable offer button again, etc.
    slotEnableButtons(T_BUT_OFFER | T_BUT_REJECT);
    slotDisableButtons(T_BUT_AGREE);
    slotShowMyIcon(T_ICO_UNDECIDED);
  }

  _myPropsBox->clearSelection();
}

void TradeView::slotYourAddBut()
{
  unsigned int i,j;
  int propid;
  bool cont = false;

  _offerBox->clearSelection();  // newly added props will be selected

  // copy selected items in yourPropsBox into offerBox

  for (i=0; i<_yourPropsBox->count(); ++i) {

    cont=false;
    propid = _yourPropsIDByIndex[i];
    
    // test if prop is selected and if it isn't already included in offer
    if (!_yourPropsBox->isSelected(i)) cont = true;  // item is not selected
    for (j=0; j<_offerBox->count(); ++j) {	// propid already in offer
//        cout << "  " << _offerPropsIDByIndex[j] << endl;
      if (_offerPropsIDByIndex[j] == propid) cont = true;
    }
    if (cont) continue;

    QListBoxText *item = new QListBoxText(QString("Get: ") 
					  + _yourPropsBox->item(i)->text());
    _offerBox->insertItem(item);
    _offerBox->setSelected(item, true);
    _offerPropsIDByIndex[_offerBox->count()-1] = propid;
//      cout << "added " << _yourPropsIDByIndex[i] << " to offer" << endl;
    // something changed, so enable offer button again, etc.
    slotEnableButtons(T_BUT_OFFER | T_BUT_REJECT);
    slotDisableButtons(T_BUT_AGREE);
    slotShowYourIcon(T_ICO_UNDECIDED);
  }

  _yourPropsBox->clearSelection();
}


void TradeView::slotMyDelBut()
{
  unsigned int i,j;
  int propid;

  // remove selected items in offerBox, if they belong to us
  for (i=0;; ++i) {
    
    if (i >= _offerBox->count()) break;

    propid = _offerPropsIDByIndex[i];
    
    // test if prop is selected and is mine, if not skip this
    if (!_offerBox->isSelected(i) || !_me->holdings(propid) ) continue;

    _offerBox->removeItem(i);
//      cout << "removed " << _offerPropsIDByIndex[i] << " from offer" << endl;
    // shift IDByIndex 
    for (j=i; j<_offerBox->count(); ++j)
      _offerPropsIDByIndex[j] = _offerPropsIDByIndex[j+1];

    // something changed, so enable offer button again etc.
    slotEnableButtons(T_BUT_OFFER | T_BUT_REJECT);
    slotDisableButtons(T_BUT_AGREE);
    slotShowMyIcon(T_ICO_UNDECIDED);

    // we have removed one item, so i should be unchanged
    i--;
  }

  for (i=0; i<_offerBox->count(); ++i) {
//      cout << " index " << i << " -> propid " << _offerPropsIDByIndex[i] << endl;
  }
}


void TradeView::slotYourDelBut()
{
  unsigned int i,j;
  int propid;

  // remove selected items in offerBox, if they belong to us
  for (i=0;; ++i) {
    
    if (i >= _offerBox->count()) break;

    propid = _offerPropsIDByIndex[i];
    
    // test if prop is selected and is mine, if not skip this
    if (!_offerBox->isSelected(i) || !_dest->holdings(propid) ) continue;

    _offerBox->removeItem(i);
//      cout << "removed " << _offerPropsIDByIndex[i] << " from offer" << endl;
    // shift IDByIndex 
    for (j=i; j<_offerBox->count(); ++j)
      _offerPropsIDByIndex[j] = _offerPropsIDByIndex[j+1];

    // something changed, so enable offer button again etc.
    slotEnableButtons(T_BUT_OFFER | T_BUT_REJECT);
    slotDisableButtons(T_BUT_AGREE);
    slotShowYourIcon(T_ICO_UNDECIDED);

    // we have removed one item, so i should be unchanged
    i--;
  }

//    for (i=0; i<_offerBox->count(); ++i) {
//      cout << " index " << i << " -> propid " << _offerPropsIDByIndex[i] << endl;
//    }
}


void TradeView::slotMyRefreshBut()
{
  updateMyHoldings();
}

void TradeView::slotYourRefreshBut()
{
  updateYourHoldings();
}


void TradeView::slotToggleMoneyDirection()
{
  // toggle the state of _giveMoney
  _giveMoney = !_giveMoney;
  if (_giveMoney)
    _moneyDirection->setPixmap(_app->view()->getPixmap("rightmondir"));
  else
    _moneyDirection->setPixmap(_app->view()->getPixmap("leftmondir"));

  // something changed, so enable offer button again, etc.
  slotEnableButtons(T_BUT_OFFER | T_BUT_REJECT);
  slotDisableButtons(T_BUT_AGREE);
  slotShowMyIcon(T_ICO_UNDECIDED);
}

void TradeView::slotMoneyChanged(int)
{
  // something changed, so enable offer button again, etc.
  slotEnableButtons(T_BUT_OFFER | T_BUT_REJECT);
  slotDisableButtons(T_BUT_AGREE);
  slotShowMyIcon(T_ICO_UNDECIDED);
}


void TradeView::slotEnableButtons(int buttonsflag)
{
  if (buttonsflag & T_BUT_AGREE) _acceptBut->setEnabled(true);
  if (buttonsflag & T_BUT_REJECT) _rejectBut->setEnabled(true);
  if (buttonsflag & (T_BUT_OFFER | T_BUT_REPLY)) _offerBut->setEnabled(true);
}

void TradeView::slotDisableButtons(int buttonsflag)
{
  if (buttonsflag & T_BUT_AGREE) _acceptBut->setEnabled(false);
  if (buttonsflag & T_BUT_REJECT) _rejectBut->setEnabled(false);
  if (buttonsflag & (T_BUT_OFFER | T_BUT_REPLY)) _offerBut->setEnabled(false);
}

void TradeView::slotShowMyIcon(int iconflag)
{
  _myOfferLab->hide();
  _myAcceptLab->hide();
  _myRejectLab->hide();
  _myUndecidedLab->hide();
  if (iconflag & (T_ICO_OFFER | T_ICO_REPLY)) _myOfferLab->show();
  if (iconflag & T_ICO_ACCEPT) _myAcceptLab->show();
  if (iconflag & T_ICO_REJECT) _myRejectLab->show();
  if (iconflag & T_ICO_UNDECIDED) _myUndecidedLab->show();
}

void TradeView::slotShowYourIcon(int iconflag)
{
  _yourOfferLab->hide();
  _yourAcceptLab->hide();
  _yourRejectLab->hide();
  _yourUndecidedLab->hide();
  if (iconflag & (T_ICO_OFFER | T_ICO_REPLY)) _yourOfferLab->show();
  if (iconflag & T_ICO_ACCEPT) _yourAcceptLab->show();
  if (iconflag & T_ICO_REJECT) _yourRejectLab->show();
  if (iconflag & T_ICO_UNDECIDED) _yourUndecidedLab->show();
}


//      _top[i] = new QGridLayout( _page[i], 3,3, spacingHint() );
//      QGridLayout *top = _top[i];

//      top->setRowStretch(1,1);
//      top->addColSpacing(1,20);
//      top->setColStretch(0,1);
//      top->setColStretch(2,1);

//      _myHeadLab[i] = new QLabel( page );
//      _myHeadLab[i]->setText(i18n("Your Offer"));
//      top->addWidget( _myHeadLab[i], 0,0, AlignCenter );

//      _yourHeadLab[i] = new QLabel( page );
//      _yourHeadLab[i]->setText(i18n("Counter proposal"));
//      top->addWidget( _yourHeadLab[i], 0,2, AlignCenter );

//      _myOfferFrm[i] = new QFrame( page );
//      _myOfferFrm[i]->setMinimumSize( QSize(200,200) );
//      _myOfferFrm[i]->setFrameStyle( QFrame::Box | QFrame::Plain );
//      _myOfferFrm[i]->setBackgroundColor( Qt::white );
//      top->addWidget( _myOfferFrm[i], 1,0, AlignCenter );

//      _yourOfferFrm[i] = new QFrame( page );
//      _yourOfferFrm[i]->setMinimumSize( QSize(200,200) );
//      _yourOfferFrm[i]->setFrameStyle( QFrame::Box | QFrame::Plain );
//      _yourOfferFrm[i]->setBackgroundColor( Qt::white );
//      top->addWidget( _yourOfferFrm[i], 1,2, AlignCenter );

//      // Layouts inside the myOfferFrm
//      _myTop[i] = new QGridLayout(_myOfferFrm[i], 3,1, 5);
//      _myProps[i] = new QGridLayout((QWidget *)0, 1,BOARD_N_MONOPS, 5);
//      _myMoney[i] = new QGridLayout((QWidget *)0, 1,1, 5);
//      _myJailCards[i] = new QGridLayout((QWidget *)0, 1,2, 5);

//      _myTop[i]->setRowStretch(0,3);
//      _myTop[i]->setRowStretch(1,1);
//      _myTop[i]->setRowStretch(2,1);
//      _myTop[i]->addLayout(_myProps[i], 0,0);
//      _myTop[i]->addLayout(_myMoney[i], 1,0);
//      _myTop[i]->addLayout(_myJailCards[i], 2,0);
    
//      // Properties (add them to _myProps[i])
    
//      f = _app->game()->board()->field(BOARDWALK);
//      p = dynamic_cast<const Property *>(f);
//      d1 = new PropertyDeed(_app, _myOfferFrm[i], p);
//      d2 = new PropertyDeed(_app, _myOfferFrm[i], p);
//      d3 = new PropertyDeed(_app, _myOfferFrm[i], p);
//      d4 = new PropertyDeed(_app, _myOfferFrm[i], p);
//      d5 = new PropertyDeed(_app, _myOfferFrm[i], p);
//      money = new QLabel(_myOfferFrm[i]);
//      money->setBackgroundColor(_myOfferFrm[i]->backgroundColor());
//      money->setText("+ $235");
//      font = money->font();
//      font.setPointSize(font.pointSize()+4);
//      font.setBold(true);
//      money->setFont(font);
//      jcard = new QLabel(_myOfferFrm[i]);
//      jcard->setBackgroundColor(_myOfferFrm[i]->backgroundColor());
//      jcard->setPixmap(_app->view()->getPixmap("jcardowned"));

//      _myProps[i]->addWidget(d1, 0,0);
//      _myProps[i]->addWidget(d2, 0,1);
//      _myProps[i]->addWidget(d3, 0,2);
//      _myProps[i]->addWidget(d4, 0,3);
//      _myProps[i]->addWidget(d5, 0,4);
//      _myMoney[i]->addWidget(money, 0,0, AlignCenter);
//      _myJailCards[i]->addWidget(jcard, 0,0, AlignCenter);
    
//      // Layouts inside the yourOfferFrm
//      _yourTop[i] = new QGridLayout(_yourOfferFrm[i], 3,1, 5);
//      _yourProps[i] = new QGridLayout((QWidget *)0, 1,BOARD_N_MONOPS, 5);
//      _yourMoney[i] = new QGridLayout((QWidget *)0, 1,1, 5);
//      _yourJailCards[i] = new QGridLayout((QWidget *)0, 1,2, 5);

//      _yourTop[i]->setRowStretch(0,3);
//      _yourTop[i]->setRowStretch(1,1);
//      _yourTop[i]->setRowStretch(2,1);
//      _yourTop[i]->addLayout(_yourProps[i], 0,0);
//      _yourTop[i]->addLayout(_yourMoney[i], 1,0);
//      _yourTop[i]->addLayout(_yourJailCards[i], 2,0);
    
//      // Properties (add them to _yourProps[i])
    
//      f = _app->game()->board()->field(BOARDWALK);
//      p = dynamic_cast<const Property *>(f);
//      d1 = new PropertyDeed(_app, _yourOfferFrm[i], p);
//      d2 = new PropertyDeed(_app, _yourOfferFrm[i], p);
//      d3 = new PropertyDeed(_app, _yourOfferFrm[i], p);
//      d4 = new PropertyDeed(_app, _yourOfferFrm[i], p);
//      d5 = new PropertyDeed(_app, _yourOfferFrm[i], p);
    
//      _yourProps[i]->addWidget(d1, 0,0);
//      _yourProps[i]->addWidget(d2, 0,1);
//      _yourProps[i]->addWidget(d3, 0,2);
//      _yourProps[i]->addWidget(d4, 0,3);
//      _yourProps[i]->addWidget(d5, 0,4);

