// -*- C++ -*-
/***************************************************************************
                          tradeview.h  -  description
                             -------------------
    begin                : Mon Jul  9 2001
    copyright            : (C) 2001 by Daniel Hermann
    email                : Daniel.Hermann@stud.uni-karlsruhe.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TRADEVIEW_H
#define TRADEVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

//
// Qt
//

#include <qframe.h>

class KIntNumInput;

class QLabel;
class QGridLayout;
class QListBox;
class QPushButton;

//
// App
//

#include <global.h>

class KapitalistApp;

//
// Game
//

class Trade;
class Player;


//
// View
// 


class TradeView : public QFrame

{
  Q_OBJECT

public:

  enum { 
    T_BUT_NONE = 0,
    T_BUT_OFFER = 1,
    T_BUT_REPLY = 2,
    T_BUT_AGREE = 4,
    T_BUT_REJECT = 8,
    T_BUT_ALL = 15
  };

  enum {
    T_ICO_NONE = 0,
    T_ICO_OFFER = 1,
    T_ICO_REPLY = 2,
    T_ICO_ACCEPT = 4,
    T_ICO_REJECT = 8,
    T_ICO_UNDECIDED = 16,
    T_ICO_ALL = 31
  };

public:

  TradeView( KapitalistApp *app, QWidget *parent, Player *dest);
  ~TradeView();

public slots:

  void slotOffer();
  void slotAccept();
  void slotReject();

  void update(struct packet_trade *);
  void updateOffer(struct packet_trade *);

  void updateMyHoldings();
  void updateYourHoldings();
  void updateHoldings(const Player *);

protected:

  KapitalistApp *_app;

  Trade *_trade;

  Player *_me;
  Player *_dest;

  QGridLayout *_top;

  QLabel *_myHeadLab;
  QLabel *_yourHeadLab;
  QLabel *_offerLab;

  QListBox *_myPropsBox;
  QListBox *_yourPropsBox;
  QListBox *_offerBox;

  QPushButton *_myAddBut;
  QPushButton *_myDelBut;
  QPushButton *_yourAddBut;
  QPushButton *_yourDelBut;
  QPushButton *_myRefreshBut;
  QPushButton *_yourRefreshBut;


  QLabel *_myBudgetLab;
  QLabel *_yourBudgetLab;

  QPushButton *_moneyDirection;
  KIntNumInput *_moneyEdit;
  
  int _myPropsIDByIndex[BOARD_N_FIELDS];
  int _yourPropsIDByIndex[BOARD_N_FIELDS];
  int _offerPropsIDByIndex[2*BOARD_N_FIELDS];

  bool _giveMoney;

//    QHBoxLayout *_but;
  QPushButton *_offerBut;
  QPushButton *_acceptBut;
  QPushButton *_rejectBut;

  QLabel *_myOfferLab;
  QLabel *_myAcceptLab;
  QLabel *_myRejectLab;
  QLabel *_myUndecidedLab;

  QLabel *_yourOfferLab;
  QLabel *_yourAcceptLab;
  QLabel *_yourRejectLab;
  QLabel *_yourUndecidedLab;

//    QGridLayout *_yourTop;
//    QGridLayout *_yourProps;
//    QGridLayout *_yourMoney;
//    QGridLayout *_yourJailCards;

//    QLabel *_centerLab;

protected slots:

  void slotMyAddBut();
  void slotYourAddBut();

  void slotMyDelBut();
  void slotYourDelBut();

  void slotMyRefreshBut();
  void slotYourRefreshBut();

  void slotToggleMoneyDirection();
  void slotMoneyChanged(int dummy);

  void slotEnableButtons(int buttonsflag);
  void slotDisableButtons(int buttonsflag);

  void slotShowMyIcon(int iconflag);
  void slotShowYourIcon(int iconflag);
};

#endif // TRADEVIEW_H

